@@ -12,11 +12,11 @@
 #pragma clang diagnostic ignored "-Wunused-function"
 #endif
 
-#include <ode/common.h>
-#include <ode/collision.h>
-#include <ode/matrix.h>
-#include <ode/rotation.h>
-#include <ode/odemath.h>
+#include <gazebo/ode/common.h>
+#include <gazebo/ode/collision.h>
+#include <gazebo/ode/matrix.h>
+#include <gazebo/ode/rotation.h>
+#include <gazebo/ode/odemath.h>
 #include "config.h"
 #include "collision_kernel.h"
 #include "collision_std.h"
@@ -880,7 +880,7 @@ dGeomID dCreateHeightfield( dSpaceID space, dHeightfieldDataID data, int bPlacea
 void dGeomHeightfieldSetHeightfieldData( dGeomID g, dHeightfieldDataID d )
 {
     dxHeightfield* geom = (dxHeightfield*) g;
-    geom->data = d;
+    geom->m_p_data = d;
 }
 
 
