@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,10 +16,10 @@
 */
 
 #ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-  #define snprintf _snprintf
+  // snprintf is available since VS 2015
+  #if defined(_MSC_VER) && (_MSC_VER < 1900)
+    #define snprintf _snprintf
+  #endif
 #endif
 
 #include <stdio.h>
@@ -30,9 +30,20 @@
 #include <boost/lexical_cast.hpp>
 #include <boost/program_options.hpp>
 
+#include <sdf/parser.hh>
 #include <sdf/sdf.hh>
 
 #include <ignition/math/Rand.hh>
+#include <ignition/math/SemanticVersion.hh>
+#include <ignition/common/Profiler.hh>
+#include <ignition/common/Filesystem.hh>
+#include <ignition/common/URI.hh>
+#ifdef _WIN32
+  // DELETE is defined in winnt.h and causes a problem with
+  // ignition::fuel_tools::REST::DELETE
+  #undef DELETE
+#endif
+#include <ignition/fuel_tools/Interface.hh>
 
 #include "gazebo/gazebo.hh"
 #include "gazebo/transport/transport.hh"
@@ -56,6 +67,8 @@
 #include "gazebo/physics/World.hh"
 #include "gazebo/physics/Base.hh"
 
+#include "gazebo/rendering/RenderingIface.hh"
+
 #include "gazebo/Master.hh"
 #include "gazebo/Server.hh"
 
@@ -66,6 +79,26 @@ namespace gazebo
 {
   struct ServerPrivate
   {
+    void InspectSDFElement(const sdf::ElementPtr _elem)
+    {
+      if (common::getEnv("GAZEBO11_BACKWARDS_COMPAT_WARNINGS_ERRORS"))
+        return;
+
+      // SDF 1.7 and above consider invalid sibling elements of ANY type with
+      // repeated names while SDF 1.6 and lower only elements of the SAME type
+      auto sdfVersion =
+        ignition::math::SemanticVersion(_elem->OriginalVersion());
+      bool result;
+      if (sdfVersion >= ignition::math::SemanticVersion(1, 7))
+        result = sdf::recursiveSiblingUniqueNames(_elem);
+      else
+        result = sdf::recursiveSameTypeUniqueNames(_elem);
+
+      if (!result)
+        gzerr << "SDF is not valid, see errors above. "
+              << "This can lead to an unexpected behaviour." << "\n";
+    }
+
     /// \brief Boolean used to stop the server.
     static bool stop;
 
@@ -98,6 +131,9 @@ namespace gazebo
 
     /// \brief Save argv for access by system plugins.
     char **systemPluginsArgv;
+
+    /// \brief Set whether to lockstep physics and rendering
+    bool lockstep = false;
   };
 }
 
@@ -147,6 +183,7 @@ bool Server::ParseArgs(int _argc, char **_argv)
     ("verbose", "Increase the messages written to the terminal.")
     ("help,h", "Produce this help message.")
     ("pause,u", "Start the server in a paused state.")
+    ("lockstep", "Lockstep simulation so sensor update rates are respected.")
     ("physics,e", po::value<std::string>(),
      "Specify a physics engine (ode|bullet|dart|simbody).")
     ("play,p", po::value<std::string>(), "Play a log file.")
@@ -155,7 +192,14 @@ bool Server::ParseArgs(int _argc, char **_argv)
      "Compression encoding format for log data (zlib|bz2|txt).")
     ("record_path", po::value<std::string>()->default_value(""),
      "Absolute path in which to store state data")
+    ("record_period", po::value<double>()->default_value(-1),
+     "Recording period (seconds).")
+    ("record_filter", po::value<std::string>()->default_value(""),
+     "Recording filter (supports wildcard and regular expression).")
+    ("record_resources", "Recording with model meshes and materials.")
     ("seed",  po::value<double>(), "Start with a given random number seed.")
+    ("initial_sim_time", po::value<double>(),
+     "Initial simulation time (seconds). This time is also used after reset.")
     ("iters",  po::value<unsigned int>(), "Number of iterations to simulate.")
     ("minimal_comms", "Reduce the TCP/IP traffic output by gzserver")
     ("server-plugin,s", po::value<std::vector<std::string> >(),
@@ -172,6 +216,8 @@ bool Server::ParseArgs(int _argc, char **_argv)
     // <some_gui_plugin.so> as a world file.
     ("gui-plugin,g", po::value<std::vector<std::string> >(),
      "Gui plugin ignored.")
+    ("gui-client-plugin", po::value<std::vector<std::string> >(),
+     "Gui plugin ignored.")
     ("world_file", po::value<std::string>(), "SDF world to load.")
     ("pass_through", po::value<std::vector<std::string> >(),
      "not used, passed through to system plugins.");
@@ -226,7 +272,6 @@ bool Server::ParseArgs(int _argc, char **_argv)
   {
     try
     {
-      math::Rand::SetSeed(this->dataPtr->vm["seed"].as<double>());
       ignition::math::Rand::Seed(this->dataPtr->vm["seed"].as<double>());
     }
     catch(boost::bad_any_cast &_e)
@@ -252,9 +297,11 @@ bool Server::ParseArgs(int _argc, char **_argv)
   if (this->dataPtr->vm.count("record"))
   {
     this->dataPtr->params["record"] =
-        this->dataPtr->vm["record_path"].as<std::string>();
+      this->dataPtr->vm["record_path"].as<std::string>();
     this->dataPtr->params["record_encoding"] =
-        this->dataPtr->vm["record_encoding"].as<std::string>();
+      this->dataPtr->vm["record_encoding"].as<std::string>();
+    if (this->dataPtr->vm.count("record_resources"))
+      this->dataPtr->params["record_resources"] = "true";
   }
 
   if (this->dataPtr->vm.count("iters"))
@@ -272,10 +319,11 @@ bool Server::ParseArgs(int _argc, char **_argv)
     }
   }
 
-  if (this->dataPtr->vm.count("pause"))
-    this->dataPtr->params["pause"] = "true";
-  else
-    this->dataPtr->params["pause"] = "false";
+  if (this->dataPtr->vm.count("lockstep"))
+  {
+    this->dataPtr->lockstep = true;
+  }
+  rendering::set_lockstep_enabled(this->dataPtr->lockstep);
 
   if (!this->PreLoad())
   {
@@ -287,7 +335,7 @@ bool Server::ParseArgs(int _argc, char **_argv)
   // this->dataPtr->ProcessPrarams.
   //
   // Set the parameter to playback a log file. The log file contains the
-  // world description, so don't try to reead the world file from the
+  // world description, so don't try to read the world file from the
   // command line.
   if (this->dataPtr->vm.count("play"))
   {
@@ -334,15 +382,21 @@ bool Server::ParseArgs(int _argc, char **_argv)
     {
       gzwarn << "Falling back on worlds/empty.world\n";
       if (!this->LoadFile("worlds/empty.world", physics))
+      {
+        gzerr << "worlds/empty.world could not be opened, "
+              << "probably because it was not found. "
+              << "Your GAZEBO_RESOURCE_PATH is probably improperly set. "
+              << "Have you sourced <prefix>/share/gazebo/setup.bash?\n";
         return false;
+      }
     }
 
     if (this->dataPtr->vm.count("profile"))
     {
       std::string profileName = this->dataPtr->vm["profile"].as<std::string>();
-      if (physics::get_world()->GetPresetManager()->HasProfile(profileName))
+      if (physics::get_world()->PresetMgr()->HasProfile(profileName))
       {
-        physics::get_world()->GetPresetManager()->CurrentProfile(profileName);
+        physics::get_world()->PresetMgr()->CurrentProfile(profileName);
         gzmsg << "Setting physics profile to [" << profileName << "]."
               << std::endl;
       }
@@ -354,6 +408,25 @@ bool Server::ParseArgs(int _argc, char **_argv)
     }
   }
 
+  if (this->dataPtr->vm.count("initial_sim_time"))
+  {
+    try
+    {
+      common::Time initialSimTime {
+        this->dataPtr->vm["initial_sim_time"].as<double>()};
+      physics::get_world()->SetSimTime(initialSimTime);
+      physics::get_world()->SetInitialSimTime(initialSimTime);
+      gzmsg << "Setting initial sim time to [" <<
+        physics::get_world()->SimTime() << "]\n" << std::endl;
+    }
+    catch(...)
+    {
+      gzerr << "Unable to cast initial_sim_time[" <<
+        this->dataPtr->vm["initial_sim_time"].as<double>() << "] "
+        << "to double for setting initial sim time\n" << std::endl;
+    }
+  }
+
   this->ProcessParams();
 
   return true;
@@ -369,15 +442,6 @@ bool Server::GetInitialized() const
 bool Server::LoadFile(const std::string &_filename,
                       const std::string &_physics)
 {
-  // Quick test for a valid file
-  FILE *test = fopen(common::find_file(_filename).c_str(), "r");
-  if (!test)
-  {
-    gzerr << "Could not open file[" << _filename << "]\n";
-    return false;
-  }
-  fclose(test);
-
   // Load the world file
   sdf::SDFPtr sdf(new sdf::SDF);
   if (!sdf::init(sdf))
@@ -386,12 +450,60 @@ bool Server::LoadFile(const std::string &_filename,
     return false;
   }
 
-  if (!sdf::readFile(common::find_file(_filename), sdf))
-  {
-    gzerr << "Unable to read sdf file[" << _filename << "]\n";
-    return false;
-  }
+  std::string filename = _filename;
+  auto filenameUri = ignition::common::URI(filename);
 
+  if (filenameUri.Scheme() == "http" || filenameUri.Scheme() == "https")
+  {
+    std::string downloadedDir = common::FuelModelDatabase::Instance()->WorldPath(filename);
+    // Find the first sdf file in the world path for now, the later intention is
+    // to load an optional world config file first and if that does not exist,
+    // continue to load the first sdf file found as done below
+    for (ignition::common::DirIter file(downloadedDir);
+         file != ignition::common::DirIter(); ++file)
+    {
+      std::string current(*file);
+      if (ignition::common::isFile(current))
+      {
+        std::string fileName = ignition::common::basename(current);
+        std::string::size_type fileExtensionIndex = fileName.rfind(".");
+        std::string fileExtension = fileName.substr(fileExtensionIndex + 1);
+
+        if (fileExtension == "sdf" || fileExtension == "world")
+        {
+          filename = current.c_str();
+          if (!sdf::readFile(filename, sdf))
+          {
+            gzerr << "Unable to read SDF from URL[" << filename << "]\n";
+            return false;
+          }
+          gzmsg << "Loading world file [" << filename << "]" << std::endl;
+        }
+      }
+    }
+  }
+  else
+  {
+    auto foundFile = common::find_file(filename);
+
+    // Quick test for a valid file
+    FILE *test = fopen(foundFile.c_str(), "r");
+    if (!test)
+    {
+      gzerr << "Could not open file[" << filename << "]\n";
+      return false;
+    }
+    fclose(test);
+
+    if (!sdf::readFile(foundFile, sdf))
+    {
+      gzerr << "Unable to read sdf file[" << filename << "]\n";
+      return false;
+    }
+
+    gzmsg << "Loading world file [" << foundFile << "]" << std::endl;
+  }
+  
   return this->LoadImpl(sdf->Root(), _physics);
 }
 
@@ -427,6 +539,8 @@ bool Server::PreLoad()
 bool Server::LoadImpl(sdf::ElementPtr _elem,
                       const std::string &_physics)
 {
+  this->dataPtr->InspectSDFElement(_elem);
+
   // If a physics engine is specified,
   if (_physics.length())
   {
@@ -492,7 +606,11 @@ bool Server::LoadImpl(sdf::ElementPtr _elem,
       << " seconds for namespaces. Giving up.\n";
   }
 
-  physics::init_worlds();
+  if (this->dataPtr->lockstep)
+    physics::init_worlds(rendering::update_scene_poses);
+  else
+    physics::init_worlds(nullptr);
+
   this->dataPtr->stop = false;
 
   return true;
@@ -501,6 +619,7 @@ bool Server::LoadImpl(sdf::ElementPtr _elem,
 /////////////////////////////////////////////////
 void Server::SigInt(int)
 {
+  event::Events::stop();
   ServerPrivate::stop = true;
 
   // Signal to plugins/etc that a shutdown event has occured
@@ -510,6 +629,7 @@ void Server::SigInt(int)
 /////////////////////////////////////////////////
 void Server::Stop()
 {
+  event::Events::stop();
   this->dataPtr->stop = true;
 }
 
@@ -523,9 +643,11 @@ void Server::Fini()
 /////////////////////////////////////////////////
 void Server::Run()
 {
-#ifndef _WIN32
   // Now that we're about to run, install a signal handler to allow for
   // graceful shutdown on Ctrl-C.
+#ifdef _WIN32
+  signal(SIGINT, Server::SigInt);
+#else
   struct sigaction sigact;
   sigact.sa_flags = 0;
   sigact.sa_handler = Server::SigInt;
@@ -533,6 +655,15 @@ void Server::Run()
     std::cerr << "sigemptyset failed while setting up for SIGINT" << std::endl;
   if (sigaction(SIGINT, &sigact, NULL))
     std::cerr << "sigaction(2) failed while setting up for SIGINT" << std::endl;
+
+  // The following was added in
+  // https://osrf-migration.github.io/gazebo-gh-pages/#!/osrf/gazebo/pull-requests/2923,
+  // but it is causing shutdown issues when gazebo is used with ros.
+  // if (sigaction(SIGTERM, &sigact, NULL))
+  // {
+  //   std::cerr << "sigaction(15) failed while setting up for SIGTERM"
+  //             << std::endl;
+  // }
 #endif
 
   if (this->dataPtr->stop)
@@ -566,12 +697,37 @@ void Server::Run()
 
   this->dataPtr->initialized = true;
 
-  // Update the sensors.
-  while (!this->dataPtr->stop && physics::worlds_running())
+  IGN_PROFILE_THREAD_NAME("gzserver");
+  // Stay on this loop until Gazebo needs to be shut down
+  // The server and sensor manager outlive worlds
+  while (!this->dataPtr->stop)
   {
+    IGN_PROFILE("Server::Run");
+    IGN_PROFILE_BEGIN("ProcessControlMsgs");
+    if (this->dataPtr->lockstep)
+      rendering::wait_for_render_request("", 0.100);
+    // bool ret = rendering::wait_for_render_request("", 0.100);
+    // if (ret == false)
+    //   gzerr << "time out reached!" << std::endl;
+
     this->ProcessControlMsgs();
-    sensors::run_once();
-    common::Time::MSleep(1);
+    IGN_PROFILE_END();
+
+    if (physics::worlds_running())
+    {
+      IGN_PROFILE_BEGIN("run_once");
+      sensors::run_once();
+      IGN_PROFILE_END();
+    }
+    else if (sensors::running())
+    {
+      IGN_PROFILE_BEGIN("stop");
+      sensors::stop();
+      IGN_PROFILE_END();
+    }
+
+    if (!this->dataPtr->lockstep)
+      common::Time::MSleep(1);
   }
 
   // Shutdown gazebo
@@ -581,39 +737,24 @@ void Server::Run()
 /////////////////////////////////////////////////
 void Server::ProcessParams()
 {
+  bool p = this->dataPtr->vm.count("pause") > 0;
+  physics::pause_worlds(p);
   common::StrStr_M::const_iterator iter;
   for (iter = this->dataPtr->params.begin();
        iter != this->dataPtr->params.end();
        ++iter)
   {
-    if (iter->first == "pause")
+    if (iter->first == "record")
     {
-      bool p = false;
-      try
-      {
-        p = boost::lexical_cast<bool>(iter->second);
-      }
-      catch(...)
-      {
-        // Unable to convert via lexical_cast, so try "true/false" string
-        std::string str = iter->second;
-        boost::to_lower(str);
+      util::LogRecordParams params;
 
-        if (str == "true")
-          p = true;
-        else if (str == "false")
-          p = false;
-        else
-          gzerr << "Invalid param value[" << iter->first << ":"
-                << iter->second << "]\n";
-      }
-
-      physics::pause_worlds(p);
-    }
-    else if (iter->first == "record")
-    {
-      util::LogRecord::Instance()->Start(
-          this->dataPtr->params["record_encoding"], iter->second);
+      params.encoding = this->dataPtr->params["record_encoding"];
+      params.path = iter->second;
+      params.period = this->dataPtr->vm["record_period"].as<double>();
+      params.filter = this->dataPtr->vm["record_filter"].as<std::string>();
+      params.recordResources =
+          this->dataPtr->params.count("record_resources") > 0;
+      util::LogRecord::Instance()->Start(params);
     }
   }
 }
@@ -719,8 +860,19 @@ void Server::ProcessControlMsgs()
     }
     else if ((*iter).has_save_world_name())
     {
-      physics::WorldPtr world = physics::get_world((*iter).save_world_name());
-      if ((*iter).has_save_filename())
+      // Get the world pointer.
+      physics::WorldPtr world;
+      try
+      {
+        world = physics::get_world((*iter).save_world_name());
+      }
+      catch(const common::Exception &)
+      {
+        gzerr << "Unable to save world. Unknown world ["
+               << (*iter).save_world_name() << "]" << std::endl;
+      }
+
+      if (world && (*iter).has_save_filename())
         world->Save((*iter).save_filename());
       else
         gzerr << "No filename specified.\n";
