@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,10 +19,13 @@
 #include <string>
 #include <map>
 
-#include "gazebo/math/Plane.hh"
-#include "gazebo/math/Matrix3.hh"
-#include "gazebo/math/Matrix4.hh"
-#include "gazebo/math/Vector2i.hh"
+#include <ignition/math/Helpers.hh>
+#include <ignition/math/Matrix3.hh>
+#include <ignition/math/Matrix4.hh>
+#include <ignition/math/Plane.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Vector2.hh>
+#include <ignition/math/Vector3.hh>
 
 #include "gazebo/common/CommonIface.hh"
 #include "gazebo/common/Exception.hh"
@@ -41,8 +44,36 @@
 
 #include "gazebo/common/MeshManager.hh"
 
-using namespace gazebo;
-using namespace common;
+namespace gazebo
+{
+namespace common
+{
+//////////////////////////////////////////////////
+class MeshManagerPrivate
+{
+  /// \brief 3D mesh loader for COLLADA files
+  public: ColladaLoader *colladaLoader = nullptr;
+
+  /// \brief 3D mesh exporter for COLLADA files
+  public: ColladaExporter *colladaExporter = nullptr;
+
+  /// \brief 3D mesh loader for STL files
+  public: STLLoader *stlLoader = nullptr;
+
+  // \brief 3D mesh loader for FBX files
+  // \todo The FBX loader needs to be implemented.
+  // public: FBXLoader *fbxLoader = nullptr;
+
+  /// \brief Dictionary of meshes, indexed by name
+  public: std::map<std::string, Mesh*> meshes;
+
+  /// \brief supported file extensions for meshes
+  public: std::vector<std::string> fileExtensions;
+
+  /// \brief Mutex to protect from loading the same mesh in different threads
+  /// at the same time.
+  public: boost::mutex mutex;
+};
 
 // added here for ABI compatibility
 // TODO move to header / private class when merging forward.
@@ -50,10 +81,11 @@ static OBJLoader objLoader;
 
 //////////////////////////////////////////////////
 MeshManager::MeshManager()
+  : dataPtr(new MeshManagerPrivate)
 {
-  this->colladaLoader = new ColladaLoader();
-  this->colladaExporter = new ColladaExporter();
-  this->stlLoader = new STLLoader();
+  this->dataPtr->colladaLoader = new ColladaLoader();
+  this->dataPtr->colladaExporter = new ColladaExporter();
+  this->dataPtr->stlLoader = new STLLoader();
 
   // Create some basic shapes
   this->CreatePlane("unit_plane",
@@ -68,6 +100,12 @@ MeshManager::MeshManager()
       ignition::math::Vector2d(0.014, 0.014));
   this->CreateBox("unit_box", ignition::math::Vector3d(1, 1, 1),
       ignition::math::Vector2d(1, 1));
+  // Note: axis_box is added and currently only used by SelectionObj in
+  // replacement of unit_box to avoid a weird ogre 1.9 problem in
+  // UNIT_Projection_TEST on OSX
+  this->CreateBox("axis_box", ignition::math::Vector3d::One,
+      ignition::math::Vector2d::One);
+
   this->CreateCylinder("unit_cylinder", 0.5, 1.0, 1, 32);
   this->CreateCone("unit_cone", 0.5, 1.0, 5, 32);
   this->CreateCamera("unit_camera", 0.5);
@@ -77,21 +115,26 @@ MeshManager::MeshManager()
 
   this->CreateTube("selection_tube", 1.0, 1.2, 0.01, 1, 64);
 
-  this->fileExtensions.push_back("stl");
-  this->fileExtensions.push_back("dae");
-  this->fileExtensions.push_back("obj");
+  this->dataPtr->fileExtensions.push_back("stl");
+  this->dataPtr->fileExtensions.push_back("stlb");
+  this->dataPtr->fileExtensions.push_back("dae");
+  this->dataPtr->fileExtensions.push_back("obj");
 }
 
 //////////////////////////////////////////////////
 MeshManager::~MeshManager()
 {
-  delete this->colladaLoader;
-  delete this->colladaExporter;
-  delete this->stlLoader;
-  std::map<std::string, Mesh*>::iterator iter;
-  for (iter = this->meshes.begin(); iter != this->meshes.end(); ++iter)
-    delete iter->second;
-  this->meshes.clear();
+  delete this->dataPtr->colladaLoader;
+  delete this->dataPtr->colladaExporter;
+  delete this->dataPtr->stlLoader;
+  for (auto &pairNameMesh : this->dataPtr->meshes)
+  {
+    delete pairNameMesh.second;
+  }
+  this->dataPtr->meshes.clear();
+
+  delete this->dataPtr;
+  this->dataPtr = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -100,26 +143,27 @@ const Mesh *MeshManager::Load(const std::string &_filename)
   if (!this->IsValidFilename(_filename))
   {
     gzerr << "Invalid mesh filename extension[" << _filename << "]\n";
-    return NULL;
+    return nullptr;
   }
 
-  Mesh *mesh = NULL;
+  Mesh *mesh = nullptr;
 
   std::string extension;
 
   if (this->HasMesh(_filename))
   {
-    return this->meshes[_filename];
+    return this->dataPtr->meshes[_filename];
 
     // This breaks trimesh geom. Each new trimesh should have a unique name.
     /*
-    // erase mesh from this->meshes. This allows a mesh to be modified and
+    // erase mesh from this->dataPtr->meshes.
+    // This allows a mesh to be modified and
     // inserted into gazebo again without closing gazebo.
     std::map<std::string, Mesh*>::iterator iter;
-    iter = this->meshes.find(_filename);
+    iter = this->dataPtr->meshes.find(_filename);
     delete iter->second;
-    iter->second = NULL;
-    this->meshes.erase(iter);
+    iter->second = nullptr;
+    this->dataPtr->meshes.erase(iter);
     */
   }
 
@@ -130,12 +174,12 @@ const Mesh *MeshManager::Load(const std::string &_filename)
     extension = fullname.substr(fullname.rfind(".")+1, fullname.size());
     std::transform(extension.begin(), extension.end(),
         extension.begin(), ::tolower);
-    MeshLoader *loader = NULL;
+    MeshLoader *loader = nullptr;
 
     if (extension == "stl" || extension == "stlb" || extension == "stla")
-      loader = this->stlLoader;
+      loader = this->dataPtr->stlLoader;
     else if (extension == "dae")
-      loader = this->colladaLoader;
+      loader = this->dataPtr->colladaLoader;
     else if (extension == "obj")
       loader = &objLoader;
     else
@@ -148,20 +192,20 @@ const Mesh *MeshManager::Load(const std::string &_filename)
     {
       // This mutex prevents two threads from loading the same mesh at the
       // same time.
-      boost::mutex::scoped_lock lock(this->mutex);
+      boost::mutex::scoped_lock lock(this->dataPtr->mutex);
       if (!this->HasMesh(_filename))
       {
-        if ((mesh = loader->Load(fullname)) != NULL)
+        if ((mesh = loader->Load(fullname)) != nullptr)
         {
           mesh->SetName(_filename);
-          this->meshes.insert(std::make_pair(_filename, mesh));
+          this->dataPtr->meshes.insert(std::make_pair(_filename, mesh));
         }
         else
           gzerr << "Unable to load mesh[" << fullname << "]\n";
       }
       else
       {
-        mesh = this->meshes[_filename];
+        mesh = this->dataPtr->meshes[_filename];
       }
     }
     catch(gazebo::common::Exception &e)
@@ -183,7 +227,7 @@ void MeshManager::Export(const Mesh *_mesh, const std::string &_filename,
 {
   if (_extension == "dae")
   {
-    this->colladaExporter->Export(_mesh, _filename, _exportTextures);
+    this->dataPtr->colladaExporter->Export(_mesh, _filename, _exportTextures);
   }
   else
   {
@@ -202,8 +246,10 @@ bool MeshManager::IsValidFilename(const std::string &_filename)
   std::transform(extension.begin(), extension.end(),
                  extension.begin(), ::tolower);
 
-  return std::find(this->fileExtensions.begin(), this->fileExtensions.end(),
-      extension) != this->fileExtensions.end();
+  return std::find(
+      this->dataPtr->fileExtensions.begin(),
+      this->dataPtr->fileExtensions.end(),
+      extension) != this->dataPtr->fileExtensions.end();
 }
 
 //////////////////////////////////////////////////
@@ -212,7 +258,7 @@ void MeshManager::GetMeshAABB(const Mesh *_mesh,
     ignition::math::Vector3d &_minXYZ, ignition::math::Vector3d &_maxXYZ)
 {
   if (this->HasMesh(_mesh->GetName()))
-    this->meshes[_mesh->GetName()]->GetAABB(_center, _minXYZ, _maxXYZ);
+    this->dataPtr->meshes[_mesh->GetName()]->GetAABB(_center, _minXYZ, _maxXYZ);
 }
 
 //////////////////////////////////////////////////
@@ -220,14 +266,14 @@ void MeshManager::GenSphericalTexCoord(const Mesh *_mesh,
     const ignition::math::Vector3d &_center)
 {
   if (this->HasMesh(_mesh->GetName()))
-    this->meshes[_mesh->GetName()]->GenSphericalTexCoord(_center);
+    this->dataPtr->meshes[_mesh->GetName()]->GenSphericalTexCoord(_center);
 }
 
 //////////////////////////////////////////////////
 void MeshManager::AddMesh(Mesh *_mesh)
 {
   if (!this->HasMesh(_mesh->GetName()))
-    this->meshes[_mesh->GetName()] = _mesh;
+    this->dataPtr->meshes[_mesh->GetName()] = _mesh;
 }
 
 //////////////////////////////////////////////////
@@ -235,12 +281,12 @@ const Mesh *MeshManager::GetMesh(const std::string &_name) const
 {
   std::map<std::string, Mesh*>::const_iterator iter;
 
-  iter = this->meshes.find(_name);
+  iter = this->dataPtr->meshes.find(_name);
 
-  if (iter != this->meshes.end())
+  if (iter != this->dataPtr->meshes.end())
     return iter->second;
 
-  return NULL;
+  return nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -250,9 +296,9 @@ bool MeshManager::HasMesh(const std::string &_name) const
     return false;
 
   std::map<std::string, Mesh*>::const_iterator iter;
-  iter = this->meshes.find(_name);
+  iter = this->dataPtr->meshes.find(_name);
 
-  return iter != this->meshes.end();
+  return iter != this->dataPtr->meshes.end();
 }
 
 //////////////////////////////////////////////////
@@ -272,7 +318,7 @@ void MeshManager::CreateSphere(const std::string &name, float radius,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(name);
-  this->meshes.insert(std::make_pair(name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -340,7 +386,7 @@ void MeshManager::CreatePlane(const std::string &_name,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(_name);
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -359,8 +405,13 @@ void MeshManager::CreatePlane(const std::string &_name,
 
   rot = rot3;
 
-  xlate.Translate(_normal * -_d);
+  xlate.SetTranslation(_normal * -_d);
   xform = xlate * rot;
+  if (!xform.IsAffine())
+  {
+    gzerr << "Matrix is not affine, plane creation failed\n";
+    return;
+  }
 
   ignition::math::Vector3d vec;
   ignition::math::Vector3d norm(0, 0, 1);
@@ -385,11 +436,11 @@ void MeshManager::CreatePlane(const std::string &_name,
         vec.X() = (x * xSpace) - halfWidth;
         vec.Y() = (y * ySpace) - halfHeight;
         vec.Z() = -z;
-        vec = xform.TransformAffine(vec);
+        xform.TransformAffine(vec, vec);
         subMesh->AddVertex(vec);
 
         // Compute the normal
-        vec = xform.TransformAffine(norm);
+        xform.TransformAffine(norm, vec);
         subMesh->AddNormal(vec);
 
         // Compute the texture coordinate
@@ -415,7 +466,7 @@ void MeshManager::CreateBox(const std::string &_name,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(_name);
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -599,7 +650,7 @@ void MeshManager::CreateExtrudedPolyline(const std::string &_name,
         {
           int index = (ev0 + k + 1) % triangle.size();
           ignition::math::Vector3d triV = triangle[index];
-          if (math::Vector2d(triV.X(), triV.Y()) == edgeV1)
+          if (ignition::math::Vector2d(triV.X(), triV.Y()) == edgeV1)
           {
             // found another vertex in triangle that matches the vertex of the
             // other edge.
@@ -716,7 +767,7 @@ void MeshManager::CreateExtrudedPolyline(const std::string &_name,
     }
   }
 
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
   return;
 }
 
@@ -732,7 +783,7 @@ void MeshManager::CreateCamera(const std::string &_name, float _scale)
 
   Mesh *mesh = new Mesh();
   mesh->SetName(_name);
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -833,7 +884,7 @@ void MeshManager::CreateCylinder(const std::string &name, float radius,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(name);
-  this->meshes.insert(std::make_pair(name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -950,7 +1001,7 @@ void MeshManager::CreateCone(const std::string &name, float radius,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(name);
-  this->meshes.insert(std::make_pair(name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(name, mesh));
 
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
@@ -1067,7 +1118,7 @@ void MeshManager::CreateTube(const std::string &_name, float _innerRadius,
 
   Mesh *mesh = new Mesh();
   mesh->SetName(_name);
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
   SubMesh *subMesh = new SubMesh();
   mesh->AddSubMesh(subMesh);
 
@@ -1288,7 +1339,7 @@ void MeshManager::CreateBoolean(const std::string &_name, const Mesh *_m1,
   MeshCSG csg;
   Mesh *mesh = csg.CreateBoolean(_m1, _m2, _operation, _offset);
   mesh->SetName(_name);
-  this->meshes.insert(std::make_pair(_name, mesh));
+  this->dataPtr->meshes.insert(std::make_pair(_name, mesh));
 }
 #endif
 
@@ -1343,3 +1394,18 @@ void MeshManager::ConvertPolylinesToVerticesAndEdges(
     }
   }
 }
+
+//////////////////////////////////////////////////
+MeshManager* MeshManager::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<MeshManager>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
+}
+}
