@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,9 +16,12 @@
  */
 #include <list>
 
+#include <ignition/math/Matrix3.hh>
+#include <ignition/math/Matrix4.hh>
+#include <ignition/math/Vector3.hh>
+
 #include "gazebo/common/Skeleton.hh"
 #include "gazebo/common/SkeletonAnimation.hh"
-#include "gazebo/math/Angle.hh"
 
 using namespace gazebo;
 using namespace common;
@@ -54,6 +57,12 @@ SkeletonNode* Skeleton::GetRootNode()
   return this->root;
 }
 
+//////////////////////////////////////////////////
+const SkeletonNode* Skeleton::GetRootNode() const
+{
+  return this->root;
+}
+
 //////////////////////////////////////////////////
 SkeletonNode* Skeleton::GetNodeByName(std::string _name)
 {
@@ -62,7 +71,7 @@ SkeletonNode* Skeleton::GetNodeByName(std::string _name)
     if (iter->second->GetName() == _name)
       return iter->second;
 
-  return NULL;
+  return nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -73,7 +82,7 @@ SkeletonNode* Skeleton::GetNodeById(std::string _id)
     if (iter->second->GetId() == _id)
       return iter->second;
 
-  return NULL;
+  return nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -111,7 +120,7 @@ void Skeleton::Scale(double _scale)
     SkeletonNode *node = iter->second;
     ignition::math::Matrix4d trans(node->Transform());
     ignition::math::Vector3d pos(trans.Translation());
-    trans.Translate(pos * _scale);
+    trans.SetTranslation(pos * _scale);
     node->SetTransform(trans, false);
   }
 
@@ -145,24 +154,12 @@ void Skeleton::BuildNodeMap()
   }
 }
 
-//////////////////////////////////////////////////
-void Skeleton::SetBindShapeTransform(math::Matrix4 _trans)
-{
-  this->SetBindShapeTransform(_trans.Ign());
-}
-
 //////////////////////////////////////////////////
 void Skeleton::SetBindShapeTransform(const ignition::math::Matrix4d &_trans)
 {
   this->bindShapeTransform = _trans;
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 Skeleton::GetBindShapeTransform()
-{
-  return this->BindShapeTransform();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d Skeleton::BindShapeTransform()
 {
@@ -211,6 +208,7 @@ NodeMap Skeleton::GetNodes()
 //////////////////////////////////////////////////
 void Skeleton::SetNumVertAttached(unsigned int _vertices)
 {
+  this->rawNW.clear();
   this->rawNW.resize(_vertices);
 }
 
@@ -224,6 +222,8 @@ void Skeleton::AddVertNodeWeight(unsigned int _vertex, std::string _node,
 //////////////////////////////////////////////////
 unsigned int Skeleton::GetNumVertNodeWeights(unsigned int _vertex)
 {
+  if (_vertex >= this->rawNW.size())
+    return 0;
   return this->rawNW[_vertex].size();
 }
 
@@ -243,7 +243,7 @@ unsigned int Skeleton::GetNumAnimations()
 SkeletonAnimation *Skeleton::GetAnimation(const unsigned int _i)
 {
   if (_i >= this->anims.size())
-    return NULL;
+    return nullptr;
 
   return this->anims[_i];
 }
@@ -301,6 +301,12 @@ std::string SkeletonNode::GetName()
   return this->name;
 }
 
+//////////////////////////////////////////////////
+std::string SkeletonNode::Name() const
+{
+  return this->name;
+}
+
 //////////////////////////////////////////////////
 void SkeletonNode::SetId(std::string _id)
 {
@@ -328,19 +334,13 @@ bool SkeletonNode::IsJoint()
     return false;
 }
 
-//////////////////////////////////////////////////
-void SkeletonNode::SetTransform(math::Matrix4 _trans, bool _updateChildren)
-{
-  this->SetTransform(_trans.Ign(), _updateChildren);
-}
-
 //////////////////////////////////////////////////
 void SkeletonNode::SetTransform(const ignition::math::Matrix4d &_trans,
     bool _updateChildren)
 {
   this->transform = _trans;
 
-  if (this->parent == NULL)
+  if (this->parent == nullptr)
     this->modelTransform = _trans;
   else
     this->modelTransform = this->parent->ModelTransform() * _trans;
@@ -350,12 +350,6 @@ void SkeletonNode::SetTransform(const ignition::math::Matrix4d &_trans,
     this->UpdateChildrenTransforms();
 }
 
-//////////////////////////////////////////////////
-void SkeletonNode::SetInitialTransform(math::Matrix4 _trans)
-{
-  this->SetInitialTransform(_trans.Ign());
-}
-
 //////////////////////////////////////////////////
 void SkeletonNode::SetInitialTransform(const ignition::math::Matrix4d &_trans)
 {
@@ -392,31 +386,19 @@ void SkeletonNode::UpdateChildrenTransforms()
   }
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 SkeletonNode::GetTransform()
-{
-  return this->Transform();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d SkeletonNode::Transform()
 {
   return this->transform;
 }
 
-//////////////////////////////////////////////////
-void SkeletonNode::SetModelTransform(math::Matrix4 _trans, bool _updateChildren)
-{
-  this->SetModelTransform(_trans.Ign(), _updateChildren);
-}
-
 //////////////////////////////////////////////////
 void SkeletonNode::SetModelTransform(
     const ignition::math::Matrix4d &_trans, const bool _updateChildren)
 {
   this->modelTransform = _trans;
 
-  if (this->parent == NULL)
+  if (this->parent == nullptr)
   {
     this->transform = _trans;
   }
@@ -431,12 +413,6 @@ void SkeletonNode::SetModelTransform(
     this->UpdateChildrenTransforms();
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 SkeletonNode::GetModelTransform()
-{
-  return this->ModelTransform();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d SkeletonNode::ModelTransform() const
 {
@@ -489,7 +465,7 @@ SkeletonNode* SkeletonNode::GetChildByName(std::string _name)
     if (this->children[i]->GetName() == _name)
       return this->children[i];
 
-  return NULL;
+  return nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -499,7 +475,7 @@ SkeletonNode* SkeletonNode::GetChildById(std::string _id)
     if (this->children[i]->GetId() == _id)
       return this->children[i];
 
-  return NULL;
+  return nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -514,12 +490,6 @@ unsigned int SkeletonNode::GetHandle()
   return this->handle;
 }
 
-//////////////////////////////////////////////////
-void SkeletonNode::SetInverseBindTransform(math::Matrix4 _invBM)
-{
-  this->SetInverseBindTransform(_invBM.Ign());
-}
-
 //////////////////////////////////////////////////
 void SkeletonNode::SetInverseBindTransform(
     const ignition::math::Matrix4d &_invBM)
@@ -527,18 +497,18 @@ void SkeletonNode::SetInverseBindTransform(
   this->invBindTransform = _invBM;
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 SkeletonNode::GetInverseBindTransform()
-{
-  return this->InverseBindTransform();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d SkeletonNode::InverseBindTransform()
 {
   return this->invBindTransform;
 }
 
+//////////////////////////////////////////////////
+bool SkeletonNode::HasInvBindTransform()
+{
+  return this->invBindTransform != ignition::math::Matrix4d::Zero;
+}
+
 //////////////////////////////////////////////////
 std::vector<NodeTransform> SkeletonNode::GetRawTransforms()
 {
@@ -584,15 +554,6 @@ NodeTransform::NodeTransform(TransformType _type)
     ignition::math::Matrix4d(ignition::math::Matrix4d::Identity);
 }
 
-//////////////////////////////////////////////////
-NodeTransform::NodeTransform(math::Matrix4 _mat, std::string _sid,
-    TransformType _type)
-{
-  this->sid = _sid;
-  this->type = _type;
-  this->transform = _mat.Ign();
-}
-
 //////////////////////////////////////////////////
 NodeTransform::NodeTransform(const ignition::math::Matrix4d &_mat,
     const std::string &_sid, TransformType _type)
@@ -607,12 +568,6 @@ NodeTransform::~NodeTransform()
 {
 }
 
-//////////////////////////////////////////////////
-void NodeTransform::Set(math::Matrix4 _mat)
-{
-  this->Set(_mat.Ign());
-}
-
 //////////////////////////////////////////////////
 void NodeTransform::Set(const ignition::math::Matrix4d &_mat)
 {
@@ -631,12 +586,6 @@ void NodeTransform::SetSID(std::string _sid)
   this->sid = _sid;
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 NodeTransform::Get()
-{
-  return this->GetTransform();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d NodeTransform::GetTransform() const
 {
@@ -661,12 +610,6 @@ void NodeTransform::SetComponent(unsigned int _idx, double _value)
   this->source[_idx] = _value;
 }
 
-//////////////////////////////////////////////////
-void NodeTransform::SetSourceValues(math::Matrix4 _mat)
-{
-  this->SetSourceValues(_mat.Ign());
-}
-
 //////////////////////////////////////////////////
 void NodeTransform::SetSourceValues(const ignition::math::Matrix4d &_mat)
 {
@@ -682,12 +625,6 @@ void NodeTransform::SetSourceValues(const ignition::math::Matrix4d &_mat)
   }
 }
 
-//////////////////////////////////////////////////
-void NodeTransform::SetSourceValues(math::Vector3 _vec)
-{
-  this->SetSourceValues(_vec.Ign());
-}
-
 //////////////////////////////////////////////////
 void NodeTransform::SetSourceValues(const ignition::math::Vector3d &_vec)
 {
@@ -697,12 +634,6 @@ void NodeTransform::SetSourceValues(const ignition::math::Vector3d &_vec)
   this->source[2] = _vec.Z();
 }
 
-//////////////////////////////////////////////////
-void NodeTransform::SetSourceValues(math::Vector3 _axis, double _angle)
-{
-  this->SetSourceValues(_axis.Ign(), _angle);
-}
-
 //////////////////////////////////////////////////
 void NodeTransform::SetSourceValues(
     const ignition::math::Vector3d &_axis, const double _angle)
@@ -729,7 +660,7 @@ void NodeTransform::RecalculateMatrix()
   else
     if (this->type == TRANSLATE)
     {
-      this->transform.Translate(
+      this->transform.SetTranslation(
           ignition::math::Vector3d(this->source[0],
             this->source[1], this->source[2]));
     }
@@ -765,16 +696,6 @@ ignition::math::Matrix4d NodeTransform::operator*(NodeTransform _t)
   return m;
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 NodeTransform::operator*(math::Matrix4 _m)
-{
-  math::Matrix4 m;
-
-  m = gazebo::math::Matrix4(this->transform) * _m;
-
-  return m;
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d NodeTransform::operator*(
     const ignition::math::Matrix4d &_m)
