@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,12 @@
  *
 */
 
+#include <ignition/math/Helpers.hh>
+#include <ignition/math/Matrix4.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Quaternion.hh>
+#include <ignition/math/Vector3.hh>
+
 #include "gazebo/common/SkeletonAnimation.hh"
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Assert.hh"
@@ -47,12 +53,6 @@ std::string NodeAnimation::GetName() const
   return this->name;
 }
 
-//////////////////////////////////////////////////
-void NodeAnimation::AddKeyFrame(const double _time, const math::Matrix4 &_trans)
-{
-  this->AddKeyFrame(_time, _trans.Ign());
-}
-
 //////////////////////////////////////////////////
 void NodeAnimation::AddKeyFrame(const double _time,
     const ignition::math::Matrix4d &_trans)
@@ -63,18 +63,12 @@ void NodeAnimation::AddKeyFrame(const double _time,
   this->keyFrames[_time] = _trans;
 }
 
-//////////////////////////////////////////////////
-void NodeAnimation::AddKeyFrame(const double _time, const math::Pose &_pose)
-{
-  this->AddKeyFrame(_time, _pose.Ign());
-}
-
 //////////////////////////////////////////////////
 void NodeAnimation::AddKeyFrame(const double _time,
     const ignition::math::Pose3d &_pose)
 {
   ignition::math::Matrix4d mat(_pose.Rot());
-  mat.Translate(_pose.Pos());
+  mat.SetTranslation(_pose.Pos());
 
   this->AddKeyFrame(_time, mat);
 }
@@ -85,15 +79,6 @@ unsigned int NodeAnimation::GetFrameCount() const
   return this->keyFrames.size();
 }
 
-//////////////////////////////////////////////////
-void NodeAnimation::GetKeyFrame(const unsigned int _i, double &_time,
-        math::Matrix4 &_trans) const
-{
-  ignition::math::Matrix4d trans;
-  this->GetKeyFrame(_i, _time, trans);
-  _trans = trans;
-}
-
 //////////////////////////////////////////////////
 void NodeAnimation::GetKeyFrame(const unsigned int _i, double &_time,
         ignition::math::Matrix4d &_trans) const
@@ -115,13 +100,6 @@ void NodeAnimation::GetKeyFrame(const unsigned int _i, double &_time,
   }
 }
 
-//////////////////////////////////////////////////
-std::pair<double, math::Matrix4> NodeAnimation::GetKeyFrame(
-  const unsigned int _i) const
-{
-  return this->KeyFrame(_i);
-}
-
 //////////////////////////////////////////////////
 std::pair<double, ignition::math::Matrix4d> NodeAnimation::KeyFrame(
   const unsigned int _i) const
@@ -139,12 +117,6 @@ double NodeAnimation::GetLength() const
   return this->length;
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 NodeAnimation::GetFrameAt(double _time, bool _loop) const
-{
-  return this->FrameAt(_time, _loop);
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d NodeAnimation::FrameAt(double _time, bool _loop) const
 {
@@ -169,16 +141,21 @@ ignition::math::Matrix4d NodeAnimation::FrameAt(double _time, bool _loop) const
 
   std::map<double, ignition::math::Matrix4d>::const_iterator it2 = it1--;
 
-  if (it1 == this->keyFrames.begin() || ignition::math::equal(it1->first, time))
-    return it1->second;
+  if (ignition::math::equal(it2->first, time))
+    return it2->second;
 
   double nextKey = it2->first;
   ignition::math::Matrix4d nextTrans = it2->second;
   double prevKey = it1->first;
-  ignition::math::Matrix4d prevTrans = it2->second;
+  ignition::math::Matrix4d prevTrans = it1->second;
 
   double t = (time - prevKey) / (nextKey - prevKey);
-  GZ_ASSERT(t >= 0.0 && t <= 1.0, "t is not in the range 0.0..1.0");
+  if (t < 0.0 || t > 1.0)
+  {
+    gzerr << "Invalid time range for node animation: previous [" << prevKey
+          << "], next [" << nextKey << "]" << std::endl;
+    return ignition::math::Matrix4d();
+  }
 
   ignition::math::Vector3d nextPos = nextTrans.Translation();
   ignition::math::Vector3d prevPos = prevTrans.Translation();
@@ -193,7 +170,7 @@ ignition::math::Matrix4d NodeAnimation::FrameAt(double _time, bool _loop) const
       prevRot, nextRot, true);
 
   ignition::math::Matrix4d trans(rot);
-  trans.Translate(pos);
+  trans.SetTranslation(pos);
 
   return trans;
 }
@@ -205,7 +182,7 @@ void NodeAnimation::Scale(const double _scale)
   {
     ignition::math::Matrix4d *mat = &frame.second;
     ignition::math::Vector3d pos = mat->Translation();
-    mat->Translate(pos * _scale);
+    mat->SetTranslation(pos * _scale);
   }
 }
 
@@ -269,13 +246,6 @@ bool SkeletonAnimation::HasNode(const std::string& _node) const
   return (this->animations.find(_node) != this->animations.end());
 }
 
-//////////////////////////////////////////////////
-void SkeletonAnimation::AddKeyFrame(const std::string &_node,
-    const double _time, const math::Matrix4 &_mat)
-{
-  this->AddKeyFrame(_node, _time, _mat.Ign());
-}
-
 //////////////////////////////////////////////////
 void SkeletonAnimation::AddKeyFrame(const std::string& _node,
     const double _time, const ignition::math::Matrix4d &_mat)
@@ -289,13 +259,6 @@ void SkeletonAnimation::AddKeyFrame(const std::string& _node,
   this->animations[_node]->AddKeyFrame(_time, _mat);
 }
 
-//////////////////////////////////////////////////
-void SkeletonAnimation::AddKeyFrame(const std::string &_node,
-      const double _time, const math::Pose &_pose)
-{
-  this->AddKeyFrame(_node, _time, _pose.Ign());
-}
-
 //////////////////////////////////////////////////
 void SkeletonAnimation::AddKeyFrame(const std::string &_node,
       const double _time, const ignition::math::Pose3d &_pose)
@@ -309,13 +272,6 @@ void SkeletonAnimation::AddKeyFrame(const std::string &_node,
   this->animations[_node]->AddKeyFrame(_time, _pose);
 }
 
-//////////////////////////////////////////////////
-math::Matrix4 SkeletonAnimation::GetNodePoseAt(const std::string &_nod,
-    const double _time, const bool _loop)
-{
-  return this->NodePoseAt(_nod, _time, _loop);
-}
-
 //////////////////////////////////////////////////
 ignition::math::Matrix4d SkeletonAnimation::NodePoseAt(
     const std::string &_node, const double _time, const bool _loop)
@@ -328,19 +284,6 @@ ignition::math::Matrix4d SkeletonAnimation::NodePoseAt(
   return mat;
 }
 
-//////////////////////////////////////////////////
-std::map<std::string, math::Matrix4> SkeletonAnimation::GetPoseAt(
-    const double _time, const bool _loop) const
-{
-  std::map<std::string, ignition::math::Matrix4d> tmp;
-  std::map<std::string, math::Matrix4> result;
-  tmp = this->PoseAt(_time, _loop);
-
-  for (auto const &t : tmp)
-    result[t.first] = t.second;
-  return result;
-}
-
 //////////////////////////////////////////////////
 std::map<std::string, ignition::math::Matrix4d> SkeletonAnimation::PoseAt(
     const double _time, const bool _loop) const
@@ -358,19 +301,6 @@ std::map<std::string, ignition::math::Matrix4d> SkeletonAnimation::PoseAt(
   return pose;
 }
 
-//////////////////////////////////////////////////
-std::map<std::string, math::Matrix4> SkeletonAnimation::GetPoseAtX(
-    const double _x, const std::string &_node, const bool _loop) const
-{
-  std::map<std::string, ignition::math::Matrix4d> tmp;
-  std::map<std::string, math::Matrix4> result;
-  tmp = this->PoseAtX(_x, _node, _loop);
-
-  for (auto const &t : tmp)
-    result[t.first] = t.second;
-  return result;
-}
-
 //////////////////////////////////////////////////
 std::map<std::string, ignition::math::Matrix4d> SkeletonAnimation::PoseAtX(
     const double _x, const std::string &_node, const bool _loop) const
