@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,11 @@
 #include <string>
 #include <math.h>
 
+#include <ignition/math/Angle.hh>
+#include <ignition/math/Helpers.hh>
+#include <ignition/math/Matrix3.hh>
+#include <ignition/math/Vector3.hh>
+
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/SphericalCoordinates.hh"
 #include "gazebo/common/SphericalCoordinatesPrivate.hh"
@@ -39,12 +44,30 @@ const double g_EarthWGS84Flattening = 1.0/298.257223563;
 // Radius of the Earth (meters).
 const double g_EarthRadius = 6371000.0;
 
+// Radius of the Moon (meters).
+// Source: https://lunar.gsfc.nasa.gov/library/451-SCI-000958.pdf
+const double g_MoonRadius = 1737400.0;
+
+// a: Equatorial radius of the Moon.
+// Source : https://nssdc.gsfc.nasa.gov/planetary/factsheet/moonfact.html
+const double g_MoonAxisEquatorial = 1738100.0;
+
+// b: Polar radius of the Moon.
+// Source : https://nssdc.gsfc.nasa.gov/planetary/factsheet/moonfact.html
+const double g_MoonAxisPolar = 1736000.0;
+
+// if: Unitless flattening parameter for the Moon.
+// Source : https://nssdc.gsfc.nasa.gov/planetary/factsheet/moonfact.html
+const double g_MoonFlattening = 0.0012;
+
 //////////////////////////////////////////////////
 SphericalCoordinates::SurfaceType SphericalCoordinates::Convert(
   const std::string &_str)
 {
   if ("EARTH_WGS84" == _str)
     return EARTH_WGS84;
+  else if ("MOON_SCS" == _str)
+    return MOON_SCS;
 
   gzerr << "SurfaceType string not recognized, "
         << "EARTH_WGS84 returned by default" << std::endl;
@@ -67,27 +90,6 @@ SphericalCoordinates::SphericalCoordinates(const SurfaceType _type)
   this->SetElevationReference(0.0);
 }
 
-//////////////////////////////////////////////////
-SphericalCoordinates::SphericalCoordinates(const SurfaceType _type,
-    const math::Angle &_latitude,
-    const math::Angle &_longitude,
-    double _elevation,
-    const math::Angle &_heading)
-: dataPtr(new SphericalCoordinatesPrivate)
-{
-  // Set the reference and calculate ellipse parameters
-  this->SetSurfaceType(_type);
-
-  // Set the coordinate transform parameters
-  this->dataPtr->latitudeReference = _latitude.Radian();
-  this->dataPtr->longitudeReference = _longitude.Radian();
-  this->dataPtr->elevationReference = _elevation;
-  this->dataPtr->headingOffset = _heading.Radian();
-
-  // Generate transformation matrix
-  this->UpdateTransformationMatrix();
-}
-
 //////////////////////////////////////////////////
 SphericalCoordinates::SphericalCoordinates(const SurfaceType _type,
     const ignition::math::Angle &_latitude,
@@ -113,7 +115,7 @@ SphericalCoordinates::SphericalCoordinates(const SurfaceType _type,
 SphericalCoordinates::~SphericalCoordinates()
 {
   delete this->dataPtr;
-  this->dataPtr = NULL;
+  this->dataPtr = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -122,24 +124,12 @@ SphericalCoordinates::SurfaceType SphericalCoordinates::GetSurfaceType() const
   return this->dataPtr->surfaceType;
 }
 
-//////////////////////////////////////////////////
-math::Angle SphericalCoordinates::GetLatitudeReference() const
-{
-  return this->LatitudeReference();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Angle SphericalCoordinates::LatitudeReference() const
 {
   return this->dataPtr->latitudeReference;
 }
 
-//////////////////////////////////////////////////
-math::Angle SphericalCoordinates::GetLongitudeReference() const
-{
-  return this->LongitudeReference();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Angle SphericalCoordinates::LongitudeReference() const
 {
@@ -152,12 +142,6 @@ double SphericalCoordinates::GetElevationReference() const
   return this->dataPtr->elevationReference;
 }
 
-//////////////////////////////////////////////////
-math::Angle SphericalCoordinates::GetHeadingOffset() const
-{
-  return this->HeadingOffset();
-}
-
 //////////////////////////////////////////////////
 ignition::math::Angle SphericalCoordinates::HeadingOffset() const
 {
@@ -182,6 +166,30 @@ void SphericalCoordinates::SetSurfaceType(const SurfaceType &_type)
       // Set the flattening parameter
       this->dataPtr->ellF = g_EarthWGS84Flattening;
 
+      // Set the first eccentricity ellipse parameter
+      // https://en.wikipedia.org/wiki/Eccentricity_(mathematics)#Ellipses
+      this->dataPtr->ellE = sqrt(1.0 -
+          std::pow(this->dataPtr->ellB, 2) / std::pow(this->dataPtr->ellA, 2));
+
+      // Set the second eccentricity ellipse parameter
+      // https://en.wikipedia.org/wiki/Eccentricity_(mathematics)#Ellipses
+      this->dataPtr->ellP = sqrt(
+          std::pow(this->dataPtr->ellA, 2) / std::pow(this->dataPtr->ellB, 2) -
+          1.0);
+
+      break;
+      }
+    case MOON_SCS:
+      {
+      // Set the semi-major axis
+      this->dataPtr->ellA = g_MoonAxisEquatorial;
+
+      // Set the semi-minor axis
+      this->dataPtr->ellB = g_MoonAxisPolar;
+
+      // Set the flattening parameter
+      this->dataPtr->ellF = g_MoonFlattening;
+
       // Set the first eccentricity ellipse parameter
       // https://en.wikipedia.org/wiki/Eccentricity_(mathematics)#Ellipses
       this->dataPtr->ellE = sqrt(1.0 -
@@ -201,13 +209,6 @@ void SphericalCoordinates::SetSurfaceType(const SurfaceType &_type)
   }
 }
 
-//////////////////////////////////////////////////
-void SphericalCoordinates::SetLatitudeReference(
-    const math::Angle &_angle)
-{
-  this->SetLatitudeReference(ignition::math::Angle(_angle.Radian()));
-}
-
 //////////////////////////////////////////////////
 void SphericalCoordinates::SetLatitudeReference(
     const ignition::math::Angle &_angle)
@@ -216,13 +217,6 @@ void SphericalCoordinates::SetLatitudeReference(
   this->UpdateTransformationMatrix();
 }
 
-//////////////////////////////////////////////////
-void SphericalCoordinates::SetLongitudeReference(
-    const math::Angle &_angle)
-{
-  this->SetLongitudeReference(ignition::math::Angle(_angle.Radian()));
-}
-
 //////////////////////////////////////////////////
 void SphericalCoordinates::SetLongitudeReference(
     const ignition::math::Angle &_angle)
@@ -238,12 +232,6 @@ void SphericalCoordinates::SetElevationReference(double _elevation)
   this->UpdateTransformationMatrix();
 }
 
-//////////////////////////////////////////////////
-void SphericalCoordinates::SetHeadingOffset(const math::Angle &_angle)
-{
-  this->SetHeadingOffset(_angle.Ign());
-}
-
 //////////////////////////////////////////////////
 void SphericalCoordinates::SetHeadingOffset(const ignition::math::Angle &_angle)
 {
@@ -251,20 +239,6 @@ void SphericalCoordinates::SetHeadingOffset(const ignition::math::Angle &_angle)
   this->UpdateTransformationMatrix();
 }
 
-//////////////////////////////////////////////////
-math::Vector3 SphericalCoordinates::SphericalFromLocal(
-    const math::Vector3 &_xyz) const
-{
-  return this->SphericalFromLocal(_xyz.Ign());
-}
-
-//////////////////////////////////////////////////
-math::Vector3 SphericalCoordinates::GlobalFromLocal(
-    const math::Vector3 &_xyz) const
-{
-  return this->GlobalFromLocal(_xyz.Ign());
-}
-
 //////////////////////////////////////////////////
 ignition::math::Vector3d SphericalCoordinates::SphericalFromLocal(
     const ignition::math::Vector3d &_xyz) const
@@ -300,19 +274,6 @@ ignition::math::Vector3d SphericalCoordinates::LocalFromGlobal(
   return this->VelocityTransform(_xyz, GLOBAL, LOCAL);
 }
 
-//////////////////////////////////////////////////
-/// Based on Haversine formula (http://en.wikipedia.org/wiki/Haversine_formula).
-double SphericalCoordinates::Distance(const math::Angle &_latA,
-                                      const math::Angle &_lonA,
-                                      const math::Angle &_latB,
-                                      const math::Angle &_lonB)
-{
-  return Distance(ignition::math::Angle(_latA.Radian()),
-                  ignition::math::Angle(_lonA.Radian()),
-                  ignition::math::Angle(_latB.Radian()),
-                  ignition::math::Angle(_lonB.Radian()));
-}
-
 //////////////////////////////////////////////////
 /// Based on Haversine formula (http://en.wikipedia.org/wiki/Haversine_formula).
 double SphericalCoordinates::Distance(const ignition::math::Angle &_latA,
@@ -332,6 +293,54 @@ double SphericalCoordinates::Distance(const ignition::math::Angle &_latA,
   return d;
 }
 
+//////////////////////////////////////////////////
+/// Based on Haversine formula (http://en.wikipedia.org/wiki/Haversine_formula).
+/// This takes into account the surface type.
+double SphericalCoordinates::DistanceBetweenPoints(
+    const ignition::math::Angle &_latA,
+    const ignition::math::Angle &_lonA,
+    const ignition::math::Angle &_latB,
+    const ignition::math::Angle &_lonB)
+{
+  ignition::math::Angle dLat = _latB - _latA;
+  ignition::math::Angle dLon = _lonB - _lonA;
+
+  double a = sin(dLat.Radian() / 2) * sin(dLat.Radian() / 2) +
+             sin(dLon.Radian() / 2) * sin(dLon.Radian() / 2) *
+             cos(_latA.Radian()) * cos(_latB.Radian());
+
+  double c = 2 * atan2(sqrt(a), sqrt(1 - a));
+  return this->SurfaceRadius() * c;
+}
+
+//////////////////////////////////////////////////
+double SphericalCoordinates::SurfaceRadius() const
+{
+  if (this->dataPtr->surfaceType ==
+      SphericalCoordinates::SurfaceType::MOON_SCS)
+    return g_MoonRadius;
+  else
+    return g_EarthRadius;
+}
+
+//////////////////////////////////////////////////
+double SphericalCoordinates::SurfaceAxisEquatorial() const
+{
+  return this->dataPtr->ellA;
+}
+
+//////////////////////////////////////////////////
+double SphericalCoordinates::SurfaceAxisPolar() const
+{
+  return this->dataPtr->ellB;
+}
+
+//////////////////////////////////////////////////
+double SphericalCoordinates::SurfaceFlattening() const
+{
+  return this->dataPtr->ellF;
+}
+
 //////////////////////////////////////////////////
 void SphericalCoordinates::UpdateTransformationMatrix()
 {
@@ -403,6 +412,7 @@ ignition::math::Vector3d SphericalCoordinates::PositionTransform(
         tmp.Y(-_pos.X() * this->dataPtr->sinHea - _pos.Y() *
             this->dataPtr->cosHea);
       }
+      [[fallthrough]];
 
     case GLOBAL:
       {
@@ -511,6 +521,7 @@ ignition::math::Vector3d SphericalCoordinates::VelocityTransform(
             this->dataPtr->sinHea);
       tmp.Y(-_vel.X() * this->dataPtr->sinHea - _vel.Y() *
             this->dataPtr->cosHea);
+      [[fallthrough]];
     // spherical
     case GLOBAL:
       tmp = this->dataPtr->rotGlobalToECEF * tmp;
