@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include <sstream>
 
 #include <boost/filesystem.hpp>
+#include <ignition/common/StringUtils.hh>
 #include <sdf/sdf.hh>
 
 #include <sys/stat.h>
@@ -37,10 +38,25 @@
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/ModelDatabase.hh"
 #include "gazebo/common/SystemPaths.hh"
+#include "gazebo/common/CommonIface.hh"
 
 using namespace gazebo;
 using namespace common;
 
+//////////////////////////////////////////////////
+char pathDelimiter()
+{
+#ifdef _WIN32
+  return ';';
+#else
+  return ':';
+#endif
+}
+
+/// \brief Callbacks to be called in order in case a file can't be found.
+/// TODO(chapulina): Move to member variable when porting forward
+std::vector<std::function<std::string (const std::string &)>> g_findFileCbs;
+
 //////////////////////////////////////////////////
 SystemPaths::SystemPaths()
 {
@@ -66,10 +82,10 @@ SystemPaths::SystemPaths()
     return;
   }
 
-  char *homePath = getenv("HOME");
+  char *homePath = getenv(HOMEDIR);
   std::string home;
   if (!homePath)
-    home = this->GetTmpPath() + "/gazebo";
+    home = this->TmpPath() + "/gazebo";
   else
     home = homePath;
 
@@ -81,7 +97,7 @@ SystemPaths::SystemPaths()
   std::string fullPath;
   if (!path)
   {
-    if (home != this->GetTmpPath() + "/gazebo")
+    if (home != this->TmpPath() + "/gazebo")
       fullPath = home + "/.gazebo";
     else
       fullPath = home;
@@ -93,7 +109,7 @@ SystemPaths::SystemPaths()
   if (!dir)
   {
 #ifdef _WIN32
-    mkdir(fullPath.c_str());
+    _mkdir(fullPath.c_str());
 #else
     mkdir(fullPath.c_str(), S_IRWXU | S_IRGRP | S_IROTH);
 #endif
@@ -159,27 +175,26 @@ const std::list<std::string> &SystemPaths::GetOgrePaths()
 }
 
 /////////////////////////////////////////////////
-std::string SystemPaths::GetTmpPath()
+const std::string &SystemPaths::TmpPath() const
 {
   return this->tmpPath.string();
 }
 
 /////////////////////////////////////////////////
-std::string SystemPaths::GetTmpInstancePath()
+const std::string &SystemPaths::TmpInstancePath() const
 {
   return this->tmpInstancePath.string();
 }
 
 /////////////////////////////////////////////////
-std::string SystemPaths::GetDefaultTestPath()
+std::string SystemPaths::DefaultTestPath() const
 {
-  return this->GetTmpInstancePath() + "/gazebo_test";
+  return this->TmpInstancePath() + "/gazebo_test";
 }
 
 /////////////////////////////////////////////////
 void SystemPaths::UpdateModelPaths()
 {
-  std::string delim(":");
   std::string path;
 
   char *pathCStr = getenv("GAZEBO_MODEL_PATH");
@@ -189,25 +204,24 @@ void SystemPaths::UpdateModelPaths()
     path = GAZEBO_MODEL_PATH;
   }
   else
-    path = pathCStr;
-
-  /// \TODO: Use boost to split string.
-  size_t pos1 = 0;
-  size_t pos2 = path.find(delim);
-  while (pos2 != std::string::npos)
   {
-    sdf::addURIPath("model://", path.substr(pos1, pos2-pos1));
-    this->InsertUnique(path.substr(pos1, pos2-pos1), this->modelPaths);
-    pos1 = pos2+1;
-    pos2 = path.find(delim, pos2+1);
+    path = pathCStr;
+  }
+
+  auto delimitedPaths = ignition::common::Split(path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
+  {
+    if (!delimitedPath.empty())
+    {
+      sdf::addURIPath("model://", delimitedPath);
+      this->InsertUnique(delimitedPath, this->modelPaths);
+    }
   }
-  this->InsertUnique(path.substr(pos1, path.size()-pos1), this->modelPaths);
 }
 
 /////////////////////////////////////////////////
 void SystemPaths::UpdateGazeboPaths()
 {
-  std::string delim(":");
   std::string path;
 
   char *pathCStr = getenv("GAZEBO_RESOURCE_PATH");
@@ -217,23 +231,23 @@ void SystemPaths::UpdateGazeboPaths()
     path = GAZEBO_RESOURCE_PATH;
   }
   else
-    path = pathCStr;
-
-  size_t pos1 = 0;
-  size_t pos2 = path.find(delim);
-  while (pos2 != std::string::npos)
   {
-    this->InsertUnique(path.substr(pos1, pos2-pos1), this->gazeboPaths);
-    pos1 = pos2+1;
-    pos2 = path.find(delim, pos2+1);
+    path = pathCStr;
+  }
+
+  auto delimitedPaths = ignition::common::Split(path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
+  {
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->gazeboPaths);
+    }
   }
-  this->InsertUnique(path.substr(pos1, path.size()-pos1), this->gazeboPaths);
 }
 
 //////////////////////////////////////////////////
 void SystemPaths::UpdatePluginPaths()
 {
-  std::string delim(":");
   std::string path;
 
   char *pathCStr = getenv("GAZEBO_PLUGIN_PATH");
@@ -243,23 +257,23 @@ void SystemPaths::UpdatePluginPaths()
     path = GAZEBO_PLUGIN_PATH;
   }
   else
-    path = pathCStr;
-
-  size_t pos1 = 0;
-  size_t pos2 = path.find(delim);
-  while (pos2 != std::string::npos)
   {
-    this->InsertUnique(path.substr(pos1, pos2-pos1), this->pluginPaths);
-    pos1 = pos2+1;
-    pos2 = path.find(delim, pos2+1);
+    path = pathCStr;
+  }
+
+  auto delimitedPaths = ignition::common::Split(path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
+  {
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->pluginPaths);
+    }
   }
-  this->InsertUnique(path.substr(pos1, path.size()-pos1), this->pluginPaths);
 }
 
 //////////////////////////////////////////////////
 void SystemPaths::UpdateOgrePaths()
 {
-  std::string delim(":");
   std::string path;
 
   char *pathCStr = getenv("OGRE_RESOURCE_PATH");
@@ -269,17 +283,18 @@ void SystemPaths::UpdateOgrePaths()
     path = OGRE_RESOURCE_PATH;
   }
   else
-    path = pathCStr;
-
-  size_t pos1 = 0;
-  size_t pos2 = path.find(delim);
-  while (pos2 != std::string::npos)
   {
-    this->InsertUnique(path.substr(pos1, pos2-pos1), this->ogrePaths);
-    pos1 = pos2+1;
-    pos2 = path.find(delim, pos2+1);
+    path = pathCStr;
+  }
+
+  auto delimitedPaths = ignition::common::Split(path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
+  {
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->ogrePaths);
+    }
   }
-  this->InsertUnique(path.substr(pos1, path.size()-pos1), this->ogrePaths);
 }
 
 
@@ -297,8 +312,8 @@ std::string SystemPaths::FindFileURI(const std::string &_uri)
   std::string suffix = _uri.substr(index + 3, _uri.size() - index - 3);
   std::string filename;
 
-  // If trying to find a model, return the path to the users home
-  // .gazebo/models
+  // If trying to find a model, look through all currently registered model
+  // paths
   if (prefix == "model")
   {
     boost::filesystem::path path;
@@ -313,21 +328,25 @@ std::string SystemPaths::FindFileURI(const std::string &_uri)
       }
     }
 
-    // Try to download the model if it wasn't found.
+    // Try to download the model from models.gazebosim.org if it wasn't found.
     if (filename.empty())
       filename = ModelDatabase::Instance()->GetModelPath(_uri, true);
   }
   else if (prefix.empty() || prefix == "file")
   {
-    // First try to find the file on the current system
+    // Try to find the file on the current system
     filename = this->FindFile(suffix);
   }
-  else if (prefix != "http" && prefix != "https")
-    gzerr << "Unknown URI prefix[" << prefix << "]\n";
 
   return filename;
 }
 
+static bool isAbsolute(const std::string &_filename)
+{
+  boost::filesystem::path path(_filename);
+  return path.is_absolute();
+}
+
 //////////////////////////////////////////////////
 std::string SystemPaths::FindFile(const std::string &_filename,
                                   bool _searchLocalPath)
@@ -337,18 +356,38 @@ std::string SystemPaths::FindFile(const std::string &_filename,
   if (_filename.empty())
     return path.string();
 
+  // Handle as URI
   if (_filename.find("://") != std::string::npos)
   {
     path = boost::filesystem::path(this->FindFileURI(_filename));
   }
-  else if (_filename[0] == '/')
+  // Handle as local absolute path
+  else if (isAbsolute(_filename))
   {
     path = boost::filesystem::path(_filename);
+    // absolute paths are not portable, e.g. when running world or
+    // or log files on a different machine. To workaround this problem,
+    // we'll further look for these files in one of gazebo model paths.
+    // e.g. /tmp/path/to/my_file
+    //      =>  ${GAZEBO_MODEL_PATH}/tmp/path/to/my_file
+    // Gazebo log playback makes use of this feature
+    if (!boost::filesystem::exists(path))
+    {
+      for (std::list<std::string>::iterator iter = this->modelPaths.begin();
+           iter != this->modelPaths.end(); ++iter)
+      {
+        auto modelPath = boost::filesystem::path(*iter) / path;
+        if (boost::filesystem::exists(modelPath))
+        {
+          path = modelPath;
+          break;
+        }
+      }
+    }
   }
+  // Try appending to Gazebo paths
   else
   {
-    bool found = false;
-
     try
     {
       path = boost::filesystem::operator/(boost::filesystem::current_path(),
@@ -362,16 +401,16 @@ std::string SystemPaths::FindFile(const std::string &_filename,
 
     if (_searchLocalPath && boost::filesystem::exists(path))
     {
-      found = true;
+      // Do nothing
     }
     else if ((_filename[0] == '/' || _filename[0] == '.' || _searchLocalPath)
              && boost::filesystem::exists(boost::filesystem::path(_filename)))
     {
       path = boost::filesystem::path(_filename);
-      found = true;
     }
     else
     {
+      bool found = false;
       std::list<std::string> paths = this->GetGazeboPaths();
 
       for (std::list<std::string>::const_iterator iter = paths.begin();
@@ -399,21 +438,40 @@ std::string SystemPaths::FindFile(const std::string &_filename,
           }
         }
       }
-    }
 
     if (!found)
-      return std::string();
+      path = std::string();
+    }
+  }
+
+  // If still not found, try custom callbacks
+  if (path.empty())
+  {
+    for (auto cb : g_findFileCbs)
+    {
+      path = cb(_filename);
+      if (!path.empty())
+        break;
+    }
   }
 
   if (!boost::filesystem::exists(path))
   {
-    gzerr << "File or path does not exist[" << path << "]\n";
+    gzwarn << "File or path does not exist [" << path << "] ["
+           << _filename << "]" << std::endl;
     return std::string();
   }
 
   return path.string();
 }
 
+/////////////////////////////////////////////////
+void SystemPaths::AddFindFileCallback(
+    std::function<std::string (const std::string &)> _cb)
+{
+  g_findFileCbs.push_back(_cb);
+}
+
 /////////////////////////////////////////////////
 void SystemPaths::ClearGazeboPaths()
 {
@@ -441,62 +499,53 @@ void SystemPaths::ClearModelPaths()
 /////////////////////////////////////////////////
 void SystemPaths::AddGazeboPaths(const std::string &_path)
 {
-  std::string delim(":");
-
-  size_t pos1 = 0;
-  size_t pos2 = _path.find(delim);
-  while (pos2 != std::string::npos)
+  auto delimitedPaths = ignition::common::Split(_path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
   {
-    this->InsertUnique(_path.substr(pos1, pos2-pos1), this->gazeboPaths);
-    pos1 = pos2+1;
-    pos2 = _path.find(delim, pos2+1);
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->gazeboPaths);
+    }
   }
-  this->InsertUnique(_path.substr(pos1, _path.size()-pos1), this->gazeboPaths);
 }
 
 /////////////////////////////////////////////////
 void SystemPaths::AddOgrePaths(const std::string &_path)
 {
-  std::string delim(":");
-  size_t pos1 = 0;
-  size_t pos2 = _path.find(delim);
-  while (pos2 != std::string::npos)
+  auto delimitedPaths = ignition::common::Split(_path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
   {
-    this->InsertUnique(_path.substr(pos1, pos2-pos1), this->ogrePaths);
-    pos1 = pos2+1;
-    pos2 = _path.find(delim, pos2+1);
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->ogrePaths);
+    }
   }
-  this->InsertUnique(_path.substr(pos1, _path.size()-pos1), this->ogrePaths);
 }
 
 /////////////////////////////////////////////////
 void SystemPaths::AddPluginPaths(const std::string &_path)
 {
-  std::string delim(":");
-  size_t pos1 = 0;
-  size_t pos2 = _path.find(delim);
-  while (pos2 != std::string::npos)
+  auto delimitedPaths = ignition::common::Split(_path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
   {
-    this->InsertUnique(_path.substr(pos1, pos2-pos1), this->pluginPaths);
-    pos1 = pos2+1;
-    pos2 = _path.find(delim, pos2+1);
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->pluginPaths);
+    }
   }
-  this->InsertUnique(_path.substr(pos1, _path.size()-pos1), this->pluginPaths);
 }
 
 /////////////////////////////////////////////////
 void SystemPaths::AddModelPaths(const std::string &_path)
 {
-  std::string delim(":");
-  size_t pos1 = 0;
-  size_t pos2 = _path.find(delim);
-  while (pos2 != std::string::npos)
+  auto delimitedPaths = ignition::common::Split(_path, pathDelimiter());
+  for (const auto &delimitedPath : delimitedPaths)
   {
-    this->InsertUnique(_path.substr(pos1, pos2-pos1), this->modelPaths);
-    pos1 = pos2+1;
-    pos2 = _path.find(delim, pos2+1);
+    if (!delimitedPath.empty())
+    {
+      this->InsertUnique(delimitedPath, this->modelPaths);
+    }
   }
-  this->InsertUnique(_path.substr(pos1, _path.size()-pos1), this->modelPaths);
 }
 
 /////////////////////////////////////////////////
@@ -529,3 +578,16 @@ void SystemPaths::AddSearchPathSuffix(const std::string &_suffix)
 
   this->suffixPaths.push_back(s);
 }
+
+//////////////////////////////////////////////////
+SystemPaths* SystemPaths::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<SystemPaths>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
