@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,9 +16,6 @@
  */
 
 #ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
   // For _access()
   #include <io.h>
 #endif
@@ -28,49 +25,54 @@
 
 #include "gazebo/common/CommonIface.hh"
 #include "gazebo/common/SystemPaths.hh"
-
+#include "gazebo/gui/DataLogger.hh"
+#include "gazebo/gui/DataLoggerPrivate.hh"
 #include "gazebo/msgs/msgs.hh"
 #include "gazebo/transport/transport.hh"
-#include "gazebo/gui/DataLogger.hh"
 
 using namespace gazebo;
 using namespace gui;
 
 /////////////////////////////////////////////////
 DataLogger::DataLogger(QWidget *_parent)
-  : QDialog(_parent)
+  : QDialog(_parent),
+    dataPtr(new DataLoggerPrivate())
 {
+  this->setWindowFlags(Qt::Window | Qt::WindowTitleHint |
+      Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint);
+
   // This name is used in the qt style sheet
   this->setObjectName("dataLogger");
   this->setWindowIcon(QIcon(":/images/gazebo.svg"));
   this->setWindowTitle(tr("Gazebo: Data Logger"));
-  // Tool stays on top of parent window
-  this->setWindowFlags(this->windowFlags() & Qt::Tool);
+  this->setWindowFlags(Qt::Window | Qt::WindowCloseButtonHint |
+      Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint);
 
   // The record button allows the user to start and pause data recording
-  this->recordButton = new QToolButton(this);
-  this->recordButton->setIcon(QPixmap(":/images/record.png"));
-  this->recordButton->setStatusTip(tr("Record a log file"));
-  this->recordButton->setCheckable(true);
-  this->recordButton->setChecked(false);
-  this->recordButton->setIconSize(QSize(30, 30));
-  this->recordButton->setObjectName("dataLoggerRecordButton");
-  connect(this->recordButton, SIGNAL(toggled(bool)),
+  this->dataPtr->recordButton = new QToolButton(this);
+  this->dataPtr->recordButton->setIcon(QPixmap(":/images/record.png"));
+  this->dataPtr->recordButton->setStatusTip(tr("Record a log file"));
+  this->dataPtr->recordButton->setCheckable(true);
+  this->dataPtr->recordButton->setChecked(false);
+  this->dataPtr->recordButton->setIconSize(QSize(30, 30));
+  this->dataPtr->recordButton->setObjectName("dataLoggerRecordButton");
+  connect(this->dataPtr->recordButton, SIGNAL(toggled(bool)),
           this, SLOT(OnRecord(bool)));
 
   // Textual status information
-  this->statusLabel = new QLabel("Ready");
-  this->statusLabel->setObjectName("dataLoggerStatusLabel");
-  this->statusLabel->setFixedWidth(80);
+  this->dataPtr->statusLabel = new QLabel("Ready");
+  this->dataPtr->statusLabel->setObjectName("dataLoggerStatusLabel");
+  this->dataPtr->statusLabel->setStyleSheet(
+      "QLabel{padding-left: 14px; padding-top: 9px}");
 
   // Duration of logging
-  this->timeLabel = new QLabel("00:00:00.000");
-  this->timeLabel->setObjectName("dataLoggerTimeLabel");
-  this->timeLabel->setFixedWidth(90);
+  this->dataPtr->timeLabel = new QLabel("00:00:00.000");
+  this->dataPtr->timeLabel->setObjectName("dataLoggerTimeLabel");
+  this->dataPtr->timeLabel->setFixedWidth(90);
 
   // Size of log file
-  this->sizeLabel = new QLabel("(0.00 B)");
-  this->sizeLabel->setObjectName("dataLoggerSizeLabel");
+  this->dataPtr->sizeLabel = new QLabel("(0.00 B)");
+  this->dataPtr->sizeLabel->setObjectName("dataLoggerSizeLabel");
 
   // Horizontal separator
   QFrame *separator = new QFrame();
@@ -83,25 +85,45 @@ DataLogger::DataLogger(QWidget *_parent)
   uriLabel->setStyleSheet(
       "QLabel {color: #aeaeae; font-size: 11px; background: transparent}");
 
+  this->dataPtr->recordResourcesBox = new QCheckBox("Record model resources");
+  this->dataPtr->recordResourcesBox->setChecked(false);
+  connect(this->dataPtr->recordResourcesBox, SIGNAL(clicked(bool)), this,
+    SLOT(OnRecordResources(bool)));
+  this->dataPtr->recordResourcesBox->setStyleSheet(
+    "QCheckBox {color: #aeaeae; font-size: 11px; background: transparent}");
+
   // Address URI Line Edit
-  this->destURI = new QLineEdit;
-  this->destURI->setReadOnly(true);
-  this->destURI->setObjectName("dataLoggerDestnationURILabel");
-  this->destURI->setStyleSheet(
+  this->dataPtr->destURI = new QLineEdit;
+  this->dataPtr->destURI->setReadOnly(true);
+  this->dataPtr->destURI->setObjectName("dataLoggerDestnationURILabel");
+  this->dataPtr->destURI->setStyleSheet(
       "QLineEdit {color: #aeaeae; font-size: 11px; background: transparent}");
 
   // "Save to" label
   QLabel *pathLabel = new QLabel("Save to: ");
   pathLabel->setStyleSheet(
-      "QLabel {color: #aeaeae; font-size: 11px; background: transparent}");
+      "QLabel {\
+         color: #aeaeae;\
+         font-size: 11px;\
+         background: transparent;\
+         padding-top: 4px;\
+       }");
 
-  // Destination path Line Edit
-  this->destPath = new QLineEdit();
-  this->destPath->setObjectName("dataLoggerDestnationPathLabel");
-  this->destPath->setMinimumWidth(300);
-  this->destPath->setReadOnly(true);
-  this->destPath->setStyleSheet(
-      "QLineEdit {color: #aeaeae; font-size: 11px; background: transparent}");
+  // Destination path Text Edit
+  this->dataPtr->destPath = new QPlainTextEdit();
+  this->dataPtr->destPath->setObjectName("dataLoggerDestnationPathLabel");
+  this->dataPtr->destPath->setMinimumWidth(200);
+  this->dataPtr->destPath->setMaximumHeight(50);
+  this->dataPtr->destPath->setReadOnly(true);
+  this->dataPtr->destPath->setFrameStyle(QFrame::NoFrame);
+  this->dataPtr->destPath->setLineWrapMode(QPlainTextEdit::WidgetWidth);
+  this->dataPtr->destPath->setWordWrapMode(QTextOption::WrapAnywhere);
+  this->dataPtr->destPath->setStyleSheet(
+      "QPlainTextEdit {\
+         color: #aeaeae;\
+         font-size: 11px;\
+         background: transparent;\
+       }");
 
   // Browser button
   QPushButton *browseButton = new QPushButton("Browse");
@@ -129,24 +151,30 @@ DataLogger::DataLogger(QWidget *_parent)
 
   // Insert widgets in the top layout
   QGridLayout *topLayout = new QGridLayout();
-  topLayout->addWidget(this->recordButton, 0, 0, 2, 1);
-  topLayout->addWidget(this->statusLabel, 0, 1, 2, 2);
-  topLayout->addWidget(this->timeLabel, 0, 3);
-  topLayout->addWidget(this->sizeLabel, 1, 3);
+  topLayout->addWidget(this->dataPtr->recordButton, 0, 0, 2, 1);
+  topLayout->addWidget(this->dataPtr->statusLabel, 0, 1, 2, 2);
+  topLayout->addWidget(this->dataPtr->timeLabel, 0, 3);
+  topLayout->addWidget(this->dataPtr->sizeLabel, 1, 3);
   topLayout->addWidget(separator, 2, 0, 1, 4);
-  topLayout->addWidget(uriLabel, 3, 0);
-  topLayout->addWidget(this->destURI, 3, 1, 1, 3);
-  topLayout->addWidget(pathLabel, 4, 0);
-  topLayout->addWidget(this->destPath, 4, 1, 1, 2);
-  topLayout->addWidget(browseButton, 4, 3);
-  topLayout->addWidget(recordingsButton, 5, 0, 1, 4);
+  topLayout->addWidget(this->dataPtr->recordResourcesBox, 3, 0, 1, 4);
+  topLayout->addWidget(uriLabel, 4, 0);
+  topLayout->addWidget(this->dataPtr->destURI, 4, 1, 1, 3);
+  topLayout->addWidget(pathLabel, 5, 0);
+  topLayout->addWidget(this->dataPtr->destPath, 5, 1, 1, 2);
+  topLayout->addWidget(browseButton, 5, 3);
+  topLayout->addWidget(recordingsButton, 6, 0, 1, 4);
 
   // Align widgets within layout
-  topLayout->setAlignment(this->statusLabel, Qt::AlignCenter);
-  topLayout->setAlignment(this->timeLabel, Qt::AlignRight);
-  topLayout->setAlignment(this->sizeLabel, Qt::AlignRight | Qt::AlignTop);
+  topLayout->setAlignment(this->dataPtr->recordResourcesBox, Qt::AlignLeft);
+  topLayout->setAlignment(pathLabel, Qt::AlignTop | Qt::AlignRight);
+  topLayout->setAlignment(browseButton, Qt::AlignTop);
+  topLayout->setAlignment(this->dataPtr->destURI, Qt::AlignLeft);
+  topLayout->setAlignment(this->dataPtr->destPath, Qt::AlignLeft);
+  topLayout->setAlignment(this->dataPtr->statusLabel, Qt::AlignLeft);
+  topLayout->setAlignment(this->dataPtr->timeLabel, Qt::AlignRight);
+  topLayout->setAlignment(this->dataPtr->sizeLabel,
+      Qt::AlignRight | Qt::AlignTop);
   topLayout->setAlignment(uriLabel, Qt::AlignRight);
-  topLayout->setAlignment(pathLabel, Qt::AlignRight);
 
   // Put the layout in a widget to be able to control size
   QWidget *topWidget = new QWidget();
@@ -154,38 +182,38 @@ DataLogger::DataLogger(QWidget *_parent)
   topWidget->setLayout(topLayout);
 
   // List of recorded logs
-  this->logList = new QTextBrowser(this);
-  this->logList->setObjectName("dataLoggerRecordingsList");
+  this->dataPtr->logList = new QTextBrowser(this);
+  this->dataPtr->logList->setObjectName("dataLoggerRecordingsList");
 
   // Layout to hold the list
   QVBoxLayout *settingsLayout = new QVBoxLayout;
   settingsLayout->setContentsMargins(2, 2, 2, 2);
-  settingsLayout->addWidget(logList);
+  settingsLayout->addWidget(this->dataPtr->logList);
 
   // Frame that can be hidden by toggling the expand button
-  this->settingsFrame = new QFrame();
-  this->settingsFrame->setObjectName("dataLoggerSettingFrame");
-  this->settingsFrame->setLayout(settingsLayout);
-  this->settingsFrame->setSizePolicy(QSizePolicy::Expanding,
+  this->dataPtr->settingsFrame = new QFrame();
+  this->dataPtr->settingsFrame->setObjectName("dataLoggerSettingFrame");
+  this->dataPtr->settingsFrame->setLayout(settingsLayout);
+  this->dataPtr->settingsFrame->setSizePolicy(QSizePolicy::Expanding,
       QSizePolicy::Expanding);
-  this->settingsFrame->hide();
+  this->dataPtr->settingsFrame->hide();
 
   // Mainlayout for the whole widget
   QVBoxLayout *mainLayout = new QVBoxLayout;
   mainLayout->setSizeConstraint(QLayout::SetMinAndMaxSize);
   mainLayout->addWidget(topWidget);
-  mainLayout->addWidget(this->settingsFrame);
+  mainLayout->addWidget(this->dataPtr->settingsFrame);
 
   // Assign the mainlayout to this widget
   this->setLayout(mainLayout);
 
   // Create a QueuedConnection to set time. This is used for thread safety.
-  connect(this, SIGNAL(SetTime(QString)),
-          this->timeLabel, SLOT(setText(QString)), Qt::QueuedConnection);
+  connect(this, SIGNAL(SetTime(QString)), this->dataPtr->timeLabel,
+      SLOT(setText(QString)), Qt::QueuedConnection);
 
   // Create a QueuedConnection to set size. This is used for thread safety.
-  connect(this, SIGNAL(SetSize(QString)),
-          this->sizeLabel, SLOT(setText(QString)), Qt::QueuedConnection);
+  connect(this, SIGNAL(SetSize(QString)), this->dataPtr->sizeLabel,
+      SLOT(setText(QString)), Qt::QueuedConnection);
 
   // Create a QueuedConnection to set destination path.
   // This is used for thread safety.
@@ -200,56 +228,60 @@ DataLogger::DataLogger(QWidget *_parent)
   connect(this, SIGNAL(rejected()), this, SLOT(OnCancel()));
 
   // Timer used to blink the status label
-  this->statusTimer = new QTimer();
-  connect(this->statusTimer, SIGNAL(timeout()), this, SLOT(OnBlinkStatus()));
-  this->statusTime = 0;
+  this->dataPtr->statusTimer = new QTimer();
+  connect(this->dataPtr->statusTimer,
+      SIGNAL(timeout()), this, SLOT(OnBlinkStatus()));
+  this->dataPtr->statusTime = 0;
 
   // Timer used to hide the confirmation dialog
-  this->confirmationDialog = NULL;
-  this->confirmationTimer = new QTimer(this);
-  connect(this->confirmationTimer, SIGNAL(timeout()), this,
+  this->dataPtr->confirmationDialog = NULL;
+  this->dataPtr->confirmationTimer = new QTimer(this);
+  connect(this->dataPtr->confirmationTimer, SIGNAL(timeout()), this,
       SLOT(OnConfirmationTimeout()));
 
   // Create a node from communication.
-  this->node = transport::NodePtr(new transport::Node());
-  this->node->Init();
+  this->dataPtr->node = transport::NodePtr(new transport::Node());
+  this->dataPtr->node->TryInit(common::Time::Maximum());
 
   // Advertise on the log control topic. The server listens to log control
   // messages.
-  this->pub = this->node->Advertise<msgs::LogControl>("~/log/control");
+  this->dataPtr->pub =
+      this->dataPtr->node->Advertise<msgs::LogControl>("~/log/control");
 
   // Subscribe to the log status topic. The server publishes log status
   // messages.
-  this->sub = this->node->Subscribe<msgs::LogStatus>("~/log/status",
-      &DataLogger::OnStatus, this);
+  this->dataPtr->sub = this->dataPtr->node->Subscribe<msgs::LogStatus>(
+      "~/log/status",  &DataLogger::OnStatus, this);
 
   // Fill the path with the home folder - duplicated from util/LogRecord
-#ifndef _WIN32
-  const char *homePath = common::getEnv("HOME");
-#else
-  const char *homePath = common::getEnv("HOMEPATH");
-#endif
+  const char *homePath = common::getEnv(HOMEDIR);
 
   GZ_ASSERT(homePath, "HOME environment variable is missing");
 
   if (!homePath)
   {
     common::SystemPaths *paths = common::SystemPaths::Instance();
-    this->basePath = QString::fromStdString(paths->GetTmpPath() + "/gazebo");
+    this->dataPtr->basePath =
+        QString::fromStdString(paths->TmpPath() + "/gazebo");
   }
   else
   {
-    this->basePath =
+    this->dataPtr->basePath =
         QString::fromStdString(boost::filesystem::path(homePath).string());
   }
 
-  this->basePath = this->basePath + "/.gazebo/log/";
-  this->SetDestinationPath(this->basePath);
+  this->dataPtr->basePath = this->dataPtr->basePath + "/.gazebo/log/";
+  this->SetDestinationPath(this->dataPtr->basePath);
 }
 
 /////////////////////////////////////////////////
 DataLogger::~DataLogger()
 {
+  this->dataPtr->sub.reset();
+  this->dataPtr->pub.reset();
+  if (this->dataPtr->node)
+    this->dataPtr->node->Fini();
+  this->dataPtr->node.reset();
 }
 
 /////////////////////////////////////////////////
@@ -259,58 +291,65 @@ void DataLogger::OnRecord(bool _toggle)
   if (_toggle)
   {
     // Switch the icon
-    this->recordButton->setIcon(QPixmap(":/images/record_stop.png"));
+    this->dataPtr->recordButton->setIcon(QPixmap(":/images/record_stop.png"));
 
-    this->statusLabel->setText("Recording...");
-    this->statusTimer->start(100);
+    this->dataPtr->statusLabel->setText("Recording...");
+    this->dataPtr->statusTimer->start(100);
 
     // Tell the server to start data logging
     msgs::LogControl msg;
     msg.set_start(true);
-    this->pub->Publish(msg);
+    this->dataPtr->pub->Publish(msg);
   }
   // Otherwise pause data logging
   else
   {
     // Switch the icon
-    this->recordButton->setIcon(QPixmap(":/images/record.png"));
+    this->dataPtr->recordButton->setIcon(QPixmap(":/images/record.png"));
 
     // Display confirmation
-    this->confirmationTimer->start(2000);
+    this->dataPtr->confirmationTimer->start(2000);
     QLabel *confirmationLabel = new QLabel("Saved to \n" +
-        this->destPath->text());
+        this->dataPtr->destPath->toPlainText());
     confirmationLabel->setObjectName("dataLoggerConfirmationLabel");
     QHBoxLayout *confirmationLayout = new QHBoxLayout();
     confirmationLayout->addWidget(confirmationLabel);
 
-    if (this->confirmationDialog)
-      this->confirmationDialog->close();
-    this->confirmationDialog = new QDialog(this, Qt::FramelessWindowHint);
-    this->confirmationDialog->setObjectName("dataLoggerConfirmationDialog");
-    this->confirmationDialog->setLayout(confirmationLayout);
-    this->confirmationDialog->setStyleSheet(
+    if (this->dataPtr->confirmationDialog)
+      this->dataPtr->confirmationDialog->close();
+    this->dataPtr->confirmationDialog =
+        new QDialog(this, Qt::FramelessWindowHint);
+    this->dataPtr->confirmationDialog->setObjectName(
+        "dataLoggerConfirmationDialog");
+    this->dataPtr->confirmationDialog->setLayout(confirmationLayout);
+    this->dataPtr->confirmationDialog->setStyleSheet(
         "QDialog {background-color: #eee}\
          QLabel {color: #111}");
-    this->confirmationDialog->setModal(false);
-    this->confirmationDialog->show();
-    this->confirmationDialog->move(this->mapToGlobal(
-        QPoint((this->width()-this->confirmationDialog->width())*0.5,
+    this->dataPtr->confirmationDialog->setModal(false);
+    this->dataPtr->confirmationDialog->show();
+    this->dataPtr->confirmationDialog->move(this->mapToGlobal(
+        QPoint((this->width()-this->dataPtr->confirmationDialog->width())*0.5,
         this->height() + 10)));
 
     // Change the status
-    this->statusLabel->setText("Ready");
-    this->statusLabel->setStyleSheet("QLabel{color: #aeaeae}");
-    this->statusTimer->stop();
+    this->dataPtr->statusLabel->setText("Ready");
+    this->dataPtr->statusLabel->setStyleSheet(
+        "QLabel {\
+           color: #aeaeae;\
+           padding-left: 14px;\
+           padding-top: 9px;\
+         }");
+    this->dataPtr->statusTimer->stop();
 
     // Change the Save to box
-    this->SetDestinationPath(this->basePath);
+    this->SetDestinationPath(this->dataPtr->basePath);
 
     // Tell the server to stop data logging
     msgs::LogControl msg;
     msg.set_stop(true);
-    this->pub->Publish(msg);
+    this->dataPtr->pub->Publish(msg);
 
-    this->logList->append(this->destPath->text());
+    this->dataPtr->logList->append(this->dataPtr->destPath->toPlainText());
   }
 }
 
@@ -343,7 +382,8 @@ void DataLogger::OnStatus(ConstLogStatusPtr &_msg)
   if (_msg->has_log_file())
   {
     // If there is file name information and we're recording...
-    if (_msg->log_file().has_base_path() && this->recordButton->isChecked())
+    if (_msg->log_file().has_base_path() &&
+        this->dataPtr->recordButton->isChecked())
     {
       std::string logBasePath = _msg->log_file().base_path();
 
@@ -397,25 +437,33 @@ void DataLogger::OnStatus(ConstLogStatusPtr &_msg)
 void DataLogger::OnSetDestinationPath(QString _filename)
 {
   if (!_filename.isEmpty())
-    this->destPath->setText(_filename);
+    this->dataPtr->destPath->setPlainText(_filename);
 }
 
 /////////////////////////////////////////////////
 void DataLogger::OnSetDestinationURI(QString _uri)
 {
   if (!_uri.isEmpty())
-    this->destURI->setText(_uri);
+    this->dataPtr->destURI->setText(_uri);
   else
-    this->destURI->setText("");
+    this->dataPtr->destURI->setText("");
 }
 
 /////////////////////////////////////////////////
 void DataLogger::OnToggleSettings(bool _checked)
 {
   if (_checked)
-    this->settingsFrame->show();
+    this->dataPtr->settingsFrame->show();
   else
-    this->settingsFrame->hide();
+    this->dataPtr->settingsFrame->hide();
+}
+
+/////////////////////////////////////////////////
+void DataLogger::OnRecordResources(bool _checked)
+{
+  msgs::LogControl msg;
+  msg.set_record_resources(_checked);
+  this->dataPtr->pub->Publish(msg);
 }
 
 /////////////////////////////////////////////////
@@ -426,6 +474,8 @@ void DataLogger::OnBrowse()
   fileDialog.setFilter(QDir::AllDirs | QDir::Hidden);
   fileDialog.setOptions(QFileDialog::ShowDirsOnly
       | QFileDialog::DontResolveSymlinks);
+  fileDialog.setWindowFlags(Qt::Window | Qt::WindowCloseButtonHint |
+      Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint);
 
   if (fileDialog.exec() != QDialog::Accepted)
     return;
@@ -480,38 +530,39 @@ void DataLogger::OnBrowse()
   // Set the new base path
   msgs::LogControl msg;
   msg.set_base_path(path.string());
-  this->pub->Publish(msg);
+  this->dataPtr->pub->Publish(msg);
 
-  this->basePath = QString::fromStdString(path.string());
-  this->SetDestinationPath(this->basePath);
+  this->dataPtr->basePath = QString::fromStdString(path.string());
+  this->SetDestinationPath(this->dataPtr->basePath);
 }
 
 /////////////////////////////////////////////////
 void DataLogger::OnBlinkStatus()
 {
-  this->statusTime += 1.0/10;
+  this->dataPtr->statusTime += 1.0/10;
 
-  if (this->statusTime >= 1)
-    this->statusTime = 0;
+  if (this->dataPtr->statusTime >= 1)
+    this->dataPtr->statusTime = 0;
 
-  this->statusLabel->setStyleSheet(QString::fromStdString(
+  this->dataPtr->statusLabel->setStyleSheet(QString::fromStdString(
       "QLabel{color: rgb("+
-          std::to_string(255+(128*(this->statusTime-1)))+", "+
-          std::to_string(255+(128*(this->statusTime-1)))+", "+
-          std::to_string(255+(128*(this->statusTime-1)))+
-      ")}"));
+          std::to_string(255+(128*(this->dataPtr->statusTime-1)))+", "+
+          std::to_string(255+(128*(this->dataPtr->statusTime-1)))+", "+
+          std::to_string(255+(128*(this->dataPtr->statusTime-1)))+
+      "); "+
+      "padding-left: 15px; padding-top: 9px}"));
 }
 
 /////////////////////////////////////////////////
 void DataLogger::OnConfirmationTimeout()
 {
-  this->confirmationDialog->close();
-  this->confirmationTimer->stop();
+  this->dataPtr->confirmationDialog->close();
+  this->dataPtr->confirmationTimer->stop();
 }
 
 /////////////////////////////////////////////////
 void DataLogger::OnCancel()
 {
-  if (this->recordButton->isChecked())
-    this->recordButton->click();
+  if (this->dataPtr->recordButton->isChecked())
+    this->dataPtr->recordButton->click();
 }
