@@ -18,6 +18,8 @@
 #include <functional>
 
 #include <ignition/math/Pose3.hh>
+#include <ignition/transport/Node.hh>
+#include <qlabel.h>
 #include <sdf/sdf.hh>
 #include <boost/algorithm/string.hpp>
 
@@ -27,6 +29,7 @@
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Events.hh"
 #include "gazebo/common/Exception.hh"
+#include "gazebo/common/CommonIface.hh"
 
 #include "gazebo/msgs/msgs.hh"
 
@@ -361,8 +364,19 @@ void MainWindow::Init()
                                             "/gazebo/world/modify",
                                             &MainWindow::OnWorldModify, this);
 
-  this->dataPtr->requestMsg = msgs::CreateRequest("scene_info");
-  this->dataPtr->requestPub->Publish(*this->dataPtr->requestMsg);
+  // Get scene info from physics::World with ignition transport service
+  ignition::transport::Node node;
+  const std::string serviceName = "/scene_info";
+  std::vector<ignition::transport::ServicePublisher> publishers;
+  if (!node.ServiceInfo(serviceName, publishers) ||
+      !node.Request(serviceName, &MainWindow::OnSceneInfo, this))
+  {
+    gzwarn << "Ignition transport [" << serviceName << "] service call failed,"
+           << " falling back to gazebo transport [scene_info] request."
+           << std::endl;
+    this->dataPtr->requestMsg = msgs::CreateRequest("scene_info");
+    this->dataPtr->requestPub->Publish(*this->dataPtr->requestMsg);
+  }
 
   gui::Events::mainWindowReady();
 }
@@ -463,10 +477,10 @@ void MainWindow::Open()
 /////////////////////////////////////////////////
 void MainWindow::SaveINI()
 {
-  char *home = getenv("HOME");
+  char *home = getenv(HOMEDIR);
   if (!home)
   {
-    gzerr << "HOME environment variable not found. "
+    gzerr << HOMEDIR << " environment variable not found. "
       "Unable to save configuration file\n";
     return;
   }
@@ -613,7 +627,17 @@ void MainWindow::About()
     "<td>";
   helpTxt += GAZEBO_VERSION_HEADER;
   helpTxt += "</td></tr></table>";
-
+  helpTxt += R"(
+<table><tr>
+  <td style='background-color: #ffcc00;padding: 20px; font-weight: bold'>
+    This version of Gazebo, now called Gazebo classic, reaches end-of-life
+    in January 2025. Users are highly encouraged to migrate to the new Gazebo
+    using our <a style='color: #f58113'
+    href='https://gazebosim.org/docs/latest/gazebo_classic_migration/'>
+      migration guides
+    </a>
+  </td></tr></table>
+)";
   helpTxt += "<div style='margin-left: 10px'>"
   "<div>"
     "<table>"
@@ -1653,9 +1677,26 @@ void MainWindow::ShowMenuBar(QMenuBar *_bar)
 
   this->dataPtr->menuLayout->addWidget(this->dataPtr->menuBar);
 
-  this->dataPtr->menuLayout->addStretch(5);
+  if (!this->dataPtr->eolNotice) {
+    const char *ignoreEol = common::getEnv("GAZEBO_SUPPRESS_EOL_WARNING");
+    if (ignoreEol == nullptr || strncmp(ignoreEol, "1", 1) != 0) {
+      // Add Gazebo classic EOL notice label
+      this->dataPtr->eolNotice = new QLabel(this->dataPtr->menuBar);
+      this->dataPtr->eolNotice->setText(R"(<font color='#ff9966'>
+          This version of Gazebo reaches end-of-life in January 2025.
+          Consider <a style='color: #ffcc00' 
+          href='https://gazebosim.org/docs/latest/gazebo_classic_migration/
+          '>migrating to the new Gazebo</a></font>)");
+      this->dataPtr->menuLayout->addStretch(1);
+      this->dataPtr->menuLayout->addWidget(this->dataPtr->eolNotice);
+    }
+  }
+
+
+  this->dataPtr->menuLayout->addStretch(4);
   this->dataPtr->menuLayout->setContentsMargins(0, 0, 0, 0);
 
+
   // OSX:
   // There is a problem on osx with the qt5 menubar being out of focus when
   // the application is launched from a terminal, so prevent using a native
@@ -2149,8 +2190,19 @@ void MainWindow::OnResponse(ConstResponsePtr &_msg)
 
   if (_msg->has_type() && _msg->type() == sceneMsg.GetTypeName())
   {
-    sceneMsg.ParseFromString(_msg->serialized_data());
+    bool parseResult = sceneMsg.ParseFromString(_msg->serialized_data());
+    this->OnSceneInfo(sceneMsg, parseResult);
+  }
 
+  delete this->dataPtr->requestMsg;
+  this->dataPtr->requestMsg = nullptr;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::OnSceneInfo(const msgs::Scene &sceneMsg, const bool _result)
+{
+  if (_result)
+  {
     for (int i = 0; i < sceneMsg.model_size(); ++i)
     {
       this->dataPtr->entities[sceneMsg.model(i).name()] =
@@ -2176,9 +2228,10 @@ void MainWindow::OnResponse(ConstResponsePtr &_msg)
       gui::Events::lightUpdate(sceneMsg.light(i));
     }
   }
-
-  delete this->dataPtr->requestMsg;
-  this->dataPtr->requestMsg = nullptr;
+  else
+  {
+    gzerr << "Error when requesting scene_info" << std::endl;
+  }
 }
 
 /////////////////////////////////////////////////
