@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,12 +15,6 @@
  *
 */
 
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include "gazebo/transport/transport.hh"
 
 #include "gazebo/rendering/RenderEvents.hh"
@@ -46,28 +40,30 @@ using namespace gui;
 ModelManipulator::ModelManipulator()
   : dataPtr(new ModelManipulatorPrivate)
 {
-  this->dataPtr->manipMode = "";
-  this->dataPtr->globalManip = false;
-  this->dataPtr->initialized = false;
 }
 
 /////////////////////////////////////////////////
 ModelManipulator::~ModelManipulator()
 {
   this->Clear();
-  delete this->dataPtr;
-  this->dataPtr = NULL;
 }
 
 /////////////////////////////////////////////////
 void ModelManipulator::Clear()
 {
   this->dataPtr->userCmdPub.reset();
-  this->dataPtr->selectionObj.reset();
-  this->dataPtr->userCamera.reset();
-  this->dataPtr->scene.reset();
+  if (this->dataPtr->node)
+    this->dataPtr->node->Fini();
   this->dataPtr->node.reset();
+
+  if (this->dataPtr->selectionObj)
+    this->dataPtr->selectionObj->Fini();
+  this->dataPtr->selectionObj.reset();
   this->dataPtr->mouseMoveVis.reset();
+
+  this->dataPtr->scene.reset();
+  this->dataPtr->userCamera.reset();
+
   this->dataPtr->mouseChildVisualScale.clear();
   this->dataPtr->manipMode = "";
   this->dataPtr->globalManip = false;
@@ -91,7 +87,7 @@ void ModelManipulator::Init()
   this->dataPtr->scene =  cam->GetScene();
 
   this->dataPtr->node = transport::NodePtr(new transport::Node());
-  this->dataPtr->node->Init();
+  this->dataPtr->node->TryInit(common::Time::Maximum());
   this->dataPtr->userCmdPub =
       this->dataPtr->node->Advertise<msgs::UserCmd>("~/user_cmd");
 
@@ -99,6 +95,8 @@ void ModelManipulator::Init()
       this->dataPtr->scene->WorldVisual()));
   this->dataPtr->selectionObj->Load();
 
+  this->dataPtr->transparent = false;
+
   this->dataPtr->initialized = true;
 }
 
@@ -112,23 +110,23 @@ void ModelManipulator::Detach()
 
 /////////////////////////////////////////////////
 void ModelManipulator::RotateEntity(rendering::VisualPtr &_vis,
-    const math::Vector3 &_axis, bool _local)
+    const ignition::math::Vector3d &_axis, const bool _local)
 {
   ignition::math::Vector3d normal;
 
   if (_local)
   {
-    if (_axis.x > 0)
-      normal = this->dataPtr->mouseMoveVisStartPose.rot.GetXAxis().Ign();
-    else if (_axis.y > 0)
-      normal = this->dataPtr->mouseMoveVisStartPose.rot.GetYAxis().Ign();
-    else if (_axis.z > 0)
-      normal = this->dataPtr->mouseMoveVisStartPose.rot.GetZAxis().Ign();
+    if (_axis.X() > 0)
+      normal = this->dataPtr->mouseMoveVisStartPose.Rot().XAxis();
+    else if (_axis.Y() > 0)
+      normal = this->dataPtr->mouseMoveVisStartPose.Rot().YAxis();
+    else if (_axis.Z() > 0)
+      normal = this->dataPtr->mouseMoveVisStartPose.Rot().ZAxis();
   }
   else
-    normal = _axis.Ign();
+    normal = _axis;
 
-  double offset = this->dataPtr->mouseMoveVisStartPose.pos.Dot(normal);
+  double offset = this->dataPtr->mouseMoveVisStartPose.Pos().Dot(normal);
 
   ignition::math::Vector3d pressPoint;
   this->dataPtr->userCamera->WorldPointOnPlane(
@@ -143,9 +141,9 @@ void ModelManipulator::RotateEntity(rendering::VisualPtr &_vis,
       ignition::math::Planed(normal, offset), newPoint);
 
   ignition::math::Vector3d v1 = pressPoint -
-    this->dataPtr->mouseMoveVisStartPose.pos.Ign();
+    this->dataPtr->mouseMoveVisStartPose.Pos();
   ignition::math::Vector3d v2 = newPoint -
-    this->dataPtr->mouseMoveVisStartPose.pos.Ign();
+    this->dataPtr->mouseMoveVisStartPose.Pos();
   v1 = v1.Normalize();
   v2 = v2.Normalize();
   double signTest = v1.Cross(v2).Dot(normal);
@@ -159,20 +157,20 @@ void ModelManipulator::RotateEntity(rendering::VisualPtr &_vis,
   if (QApplication::keyboardModifiers() & Qt::ControlModifier)
     angle = rint(angle / (M_PI * 0.25)) * (M_PI * 0.25);
 
-  math::Quaternion rot(_axis, angle);
+  ignition::math::Quaterniond rot(_axis, angle);
 
   if (_local)
-    rot = this->dataPtr->mouseMoveVisStartPose.rot * rot;
+    rot = this->dataPtr->mouseMoveVisStartPose.Rot() * rot;
   else
-    rot = rot * this->dataPtr->mouseMoveVisStartPose.rot;
+    rot = rot * this->dataPtr->mouseMoveVisStartPose.Rot();
 
   _vis->SetWorldRotation(rot);
+  Events::moveEntity(_vis->Name(), _vis->WorldPose(), false);
 }
 
 /////////////////////////////////////////////////
-math::Vector3 ModelManipulator::GetMousePositionOnPlane(
-    rendering::CameraPtr _camera,
-    const common::MouseEvent &_event)
+ignition::math::Vector3d ModelManipulator::MousePositionOnPlane(
+    rendering::CameraPtr _camera, const common::MouseEvent &_event)
 {
   ignition::math::Vector3d origin1, dir1, p1;
 
@@ -190,87 +188,90 @@ math::Vector3 ModelManipulator::GetMousePositionOnPlane(
 }
 
 /////////////////////////////////////////////////
-math::Vector3 ModelManipulator::SnapPoint(const math::Vector3 &_point,
-    double _interval, double _sensitivity)
+ignition::math::Vector3d ModelManipulator::SnapPoint(
+    const ignition::math::Vector3d &_point,
+    const double _interval, const double _sensitivity)
 {
   if (_interval < 0)
   {
     gzerr << "Interval distance must be greater than or equal to 0"
         << std::endl;
-    return math::Vector3::Zero;
+    return ignition::math::Vector3d::Zero;
   }
 
   if (_sensitivity < 0 || _sensitivity > 1.0)
   {
     gzerr << "Sensitivity must be between 0 and 1" << std::endl;
-    return math::Vector3::Zero;
+    return ignition::math::Vector3d::Zero;
   }
 
-  math::Vector3 point = _point;
+  ignition::math::Vector3d point = _point;
   double snap = _interval * _sensitivity;
 
-  double remainder = fmod(point.x, _interval);
+  double remainder = fmod(point.X(), _interval);
   int sign = remainder >= 0 ? 1 : -1;
   if (fabs(remainder) < snap)
-      point.x -= remainder;
+      point.X() -= remainder;
   else if (fabs(remainder) > (_interval - snap))
-      point.x = point.x - remainder + _interval * sign;
+      point.X() = point.X() - remainder + _interval * sign;
 
-  remainder = fmod(point.y, _interval);
+  remainder = fmod(point.Y(), _interval);
   sign = remainder >= 0 ? 1 : -1;
   if (fabs(remainder) < snap)
-      point.y -= remainder;
+      point.Y() -= remainder;
   else if (fabs(remainder) > (_interval - snap))
-      point.y = point.y - remainder + _interval * sign;
+      point.Y() = point.Y() - remainder + _interval * sign;
 
-  remainder = fmod(point.z, _interval);
+  remainder = fmod(point.Z(), _interval);
   sign = remainder >= 0 ? 1 : -1;
   if (fabs(remainder) < snap)
-      point.z -= remainder;
+      point.Z() -= remainder;
   else if (fabs(remainder) > (_interval - snap))
-      point.z = point.z - remainder + _interval * sign;
+      point.Z() = point.Z() - remainder + _interval * sign;
 
   return point;
 }
 
 /////////////////////////////////////////////////
-math::Vector3 ModelManipulator::GetMouseMoveDistance(
+ignition::math::Vector3d ModelManipulator::MouseMoveDistance(
     rendering::CameraPtr _camera,
-    const math::Vector2i &_start, const math::Vector2i &_end,
-    const math::Pose &_pose, const math::Vector3 &_axis, bool _local)
+    const ignition::math::Vector2i &_start,
+    const ignition::math::Vector2i &_end,
+    const ignition::math::Pose3d &_pose,
+    const ignition::math::Vector3d &_axis, const bool _local)
 {
-  ignition::math::Pose3d pose = _pose.Ign();
+  ignition::math::Pose3d pose = _pose;
 
   ignition::math::Vector3d origin1, dir1, p1;
   ignition::math::Vector3d origin2, dir2, p2;
 
   // Cast two rays from the camera into the world
-  _camera->CameraToViewportRay(_end.x, _end.y, origin1, dir1);
-  _camera->CameraToViewportRay(_start.x, _start.y, origin2, dir2);
+  _camera->CameraToViewportRay(_end.X(), _end.Y(), origin1, dir1);
+  _camera->CameraToViewportRay(_start.X(), _start.Y(), origin2, dir2);
 
   ignition::math::Vector3d planeNorm(0, 0, 0);
   ignition::math::Vector3d projNorm(0, 0, 0);
 
   ignition::math::Vector3d planeNormOther(0, 0, 0);
 
-  if (_axis.x > 0 && _axis.y > 0)
+  if (_axis.X() > 0 && _axis.Y() > 0)
   {
     planeNorm.Z(1);
     projNorm.Z(1);
   }
-  else if (_axis.z > 0)
+  else if (_axis.Z() > 0)
   {
     planeNorm.Y(1);
     projNorm.X(1);
     planeNormOther.X(1);
   }
-  else if (_axis.x > 0)
+  else if (_axis.X() > 0)
   {
     planeNorm.Z(1);
     projNorm.Y(1);
     planeNormOther.Y(1);
   }
-  else if (_axis.y > 0)
+  else if (_axis.Y() > 0)
   {
     planeNorm.Z(1);
     projNorm.X(1);
@@ -312,32 +313,32 @@ math::Vector3 ModelManipulator::GetMouseMoveDistance(
   ignition::math::Vector3d distance = p1 - p2;
 
   if (!_local)
-    distance *= _axis.Ign();
+    distance *= _axis;
 
   return distance;
 }
 
 /////////////////////////////////////////////////
-math::Vector3 ModelManipulator::GetMouseMoveDistance(const math::Pose &_pose,
-    const math::Vector3 &_axis, bool _local) const
+ignition::math::Vector3d ModelManipulator::MouseMoveDistance(
+    const ignition::math::Pose3d &_pose,
+    const ignition::math::Vector3d &_axis, const bool _local) const
 {
-  return GetMouseMoveDistance(this->dataPtr->userCamera,
+  return MouseMoveDistance(this->dataPtr->userCamera,
       this->dataPtr->mouseStart,
-      math::Vector2i(this->dataPtr->mouseEvent.Pos().X(),
+      ignition::math::Vector2i(this->dataPtr->mouseEvent.Pos().X(),
       this->dataPtr->mouseEvent.Pos().Y()), _pose, _axis, _local);
 }
 
 /////////////////////////////////////////////////
 void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
-    const math::Vector3 &_axis, bool _local)
+    const ignition::math::Vector3d &_axis, const bool _local)
 {
-  math::Box bbox = this->dataPtr->mouseVisualBbox;
-  math::Pose pose = _vis->GetWorldPose();
-  math::Vector3 distance =  this->GetMouseMoveDistance(pose, _axis, _local);
+  auto bbox = this->dataPtr->mouseVisualBbox;
+  auto pose = _vis->WorldPose();
+  auto distance =  this->MouseMoveDistance(pose, _axis, _local);
 
-  math::Vector3 bboxSize = bbox.GetSize();
-  math::Vector3 scale = (bboxSize + pose.rot.RotateVectorReverse(distance))
-      / bboxSize;
+  auto bboxSize = bbox.Size();
+  auto scale = (bboxSize + pose.Rot().RotateVectorReverse(distance)) / bboxSize;
 
   // extended scaling to work in model editor mode by checking geometry
   // type of first visual child.
@@ -349,7 +350,7 @@ void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
     {
       rendering::VisualPtr childVis = _vis->GetChild(i);
 
-      if (childVis->GetPose().pos != math::Vector3::Zero)
+      if (childVis->Pose().Pos() != ignition::math::Vector3d::Zero)
       {
         gzwarn << "Scaling is currently limited to simple shapes with their "
             << "origin in the centroid." << std::endl;
@@ -361,7 +362,7 @@ void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
         rendering::VisualPtr grandChildVis = childVis->GetChild(j);
         std::string thisGeomType = grandChildVis->GetGeometryType();
 
-        if (grandChildVis->GetPose().pos != math::Vector3::Zero)
+        if (grandChildVis->Pose().Pos() != ignition::math::Vector3d::Zero)
         {
           gzwarn << "Scaling is currently limited to simple shapes with their "
               << "origin in the centroid." << std::endl;
@@ -405,18 +406,18 @@ void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
       return;
     }
 
-    math::Vector3 newScale = this->dataPtr->mouseVisualScale * scale.GetAbs();
+    auto newScale = this->dataPtr->mouseVisualScale * scale.Abs();
 
     if (QApplication::keyboardModifiers() & Qt::ControlModifier)
     {
-      newScale = SnapPoint(newScale);
+      newScale = this->SnapPoint(newScale);
       // prevent setting zero scale
-      newScale.x = std::max(1e-4, newScale.x);
-      newScale.y = std::max(1e-4, newScale.y);
-      newScale.z = std::max(1e-4, newScale.z);
+      newScale.X(std::max(1e-4, newScale.X()));
+      newScale.Y(std::max(1e-4, newScale.Y()));
+      newScale.Z(std::max(1e-4, newScale.Z()));
     }
     _vis->SetScale(newScale);
-    Events::scaleEntity(_vis->GetName(), newScale);
+    Events::scaleEntity(_vis->Name(), newScale);
   }
   else
   {
@@ -435,7 +436,7 @@ void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
       if (childVis != this->dataPtr->selectionObj &&
           geomType != "" && geomType != "mesh")
       {
-        math::Vector3 geomScale;
+        ignition::math::Vector3d geomScale;
         if (QApplication::keyboardModifiers() & Qt::ShiftModifier)
         {
           geomScale = this->UpdateScale(_axis, scale, "sphere");
@@ -445,57 +446,58 @@ void ModelManipulator::ScaleEntity(rendering::VisualPtr &_vis,
           geomScale = this->UpdateScale(_axis, scale, geomType);
         }
 
-        math::Vector3 newScale = this->dataPtr->mouseChildVisualScale[i]
-            * geomScale.GetAbs();
+        auto newScale = this->dataPtr->mouseChildVisualScale[i]
+            * geomScale.Abs();
 
         if (QApplication::keyboardModifiers() & Qt::ControlModifier)
         {
-          newScale = SnapPoint(newScale);
+          newScale = this->SnapPoint(newScale);
           // prevent setting zero scale
-          newScale.x = std::max(1e-4, newScale.x);
-          newScale.y = std::max(1e-4, newScale.y);
-          newScale.z = std::max(1e-4, newScale.z);
+          newScale.X(std::max(1e-4, newScale.X()));
+          newScale.Y(std::max(1e-4, newScale.Y()));
+          newScale.Z(std::max(1e-4, newScale.Z()));
         }
 
         childVis->SetScale(newScale);
-        Events::scaleEntity(childVis->GetName(), newScale);
+        Events::scaleEntity(childVis->Name(), newScale);
       }
     }
   }
 }
 
 /////////////////////////////////////////////////
-math::Vector3 ModelManipulator::UpdateScale(const math::Vector3 &_axis,
-    const math::Vector3 &_scale, const std::string &_geom)
+ignition::math::Vector3d ModelManipulator::UpdateScale(
+    const ignition::math::Vector3d &_axis,
+    const ignition::math::Vector3d &_scale, const std::string &_geom)
 {
-  math::Vector3 scale = _scale;
+  ignition::math::Vector3d scale = _scale;
   if (_geom == "sphere")
   {
-    if (_axis.x > 0)
+    if (_axis.X() > 0)
     {
-      scale.y = scale.x;
-      scale.z = scale.x;
+      scale.Y(scale.X());
+      scale.Z(scale.X());
     }
-    else if (_axis.y > 0)
+    else if (_axis.Y() > 0)
     {
-      scale.x = scale.y;
-      scale.z = scale.y;
+      scale.X(scale.Y());
+      scale.Z(scale.Y());
     }
-    else if (_axis.z > 0)
+    else if (_axis.Z() > 0)
     {
-      scale.x = scale.z;
-      scale.y = scale.z;
+      scale.X(scale.Z());
+      scale.Y(scale.Z());
     }
   }
   else if (_geom == "cylinder")
   {
-    if (_axis.x > 0)
+    if (_axis.X() > 0)
     {
-      scale.y = scale.x;
+      scale.Y(scale.X());
     }
-    else if (_axis.y > 0)
+    else if (_axis.Y() > 0)
     {
-      scale.x = scale.y;
+      scale.X(scale.Y());
     }
   }
 
@@ -504,22 +506,23 @@ math::Vector3 ModelManipulator::UpdateScale(const math::Vector3 &_axis,
 
 /////////////////////////////////////////////////
 void ModelManipulator::TranslateEntity(rendering::VisualPtr &_vis,
-    const math::Vector3 &_axis, bool _local)
+    const ignition::math::Vector3d &_axis, const bool _local)
 {
-  math::Pose pose = _vis->GetWorldPose();
-  math::Vector3 distance =  this->GetMouseMoveDistance(pose, _axis, _local);
+  auto pose = _vis->WorldPose();
+  auto distance =  this->MouseMoveDistance(pose, _axis, _local);
 
-  pose.pos = this->dataPtr->mouseMoveVisStartPose.pos + distance;
+  pose.Pos() = this->dataPtr->mouseMoveVisStartPose.Pos() + distance;
 
   if (QApplication::keyboardModifiers() & Qt::ControlModifier)
   {
-    pose.pos = SnapPoint(pose.pos);
+    pose.Pos() = SnapPoint(pose.Pos());
   }
 
-  if (!(_axis.z > 0) && !_local)
-    pose.pos.z = _vis->GetWorldPose().pos.z;
+  if (!(_axis.Z() > 0) && !_local)
+    pose.Pos().Z() = _vis->WorldPose().Pos().Z();
 
   _vis->SetWorldPose(pose);
+  Events::moveEntity(_vis->Name(), pose, false);
 }
 
 /////////////////////////////////////////////////
@@ -546,7 +549,7 @@ void ModelManipulator::PublishVisualPose(rendering::VisualPtr _vis)
   }
 
   msgs::UserCmd userCmdMsg;
-  userCmdMsg.set_description(description + _vis->GetName() + "]");
+  userCmdMsg.set_description(description + _vis->Name() + "]");
   userCmdMsg.set_type(msgs::UserCmd::MOVING);
 
   // Only publish for models
@@ -554,30 +557,29 @@ void ModelManipulator::PublishVisualPose(rendering::VisualPtr _vis)
   {
     msgs::Model msg;
 
-    auto id = gui::get_entity_id(_vis->GetName());
+    auto id = gui::get_entity_id(_vis->Name());
     if (id)
       msg.set_id(id);
 
-    msg.set_name(_vis->GetName());
-    msgs::Set(msg.mutable_pose(), _vis->GetWorldPose().Ign());
+    msg.set_name(_vis->Name());
+    msgs::Set(msg.mutable_pose(), _vis->WorldPose());
 
     auto modelMsg = userCmdMsg.add_model();
     modelMsg->CopyFrom(msg);
   }
   // Otherwise, check to see if the visual is a light
-  else if (this->dataPtr->scene->GetLight(_vis->GetName()))
+  else if (this->dataPtr->scene->LightByName(_vis->Name()))
   {
     msgs::Light msg;
-    msg.set_name(_vis->GetName());
-    msgs::Set(msg.mutable_pose(), _vis->GetWorldPose().Ign());
+    msg.set_name(_vis->Name());
+    msgs::Set(msg.mutable_pose(), _vis->WorldPose());
 
     auto lightMsg = userCmdMsg.add_light();
     lightMsg->CopyFrom(msg);
   }
   else
   {
-    gzerr << "Visual [" << _vis->GetName() << "] isn't a model or a light"
-        << std::endl;
+    // We get here in the model editor for example
     return;
   }
   this->dataPtr->userCmdPub->Publish(userCmdMsg);
@@ -594,17 +596,17 @@ void ModelManipulator::PublishVisualScale(rendering::VisualPtr _vis)
 
   // Register user command on server
   msgs::UserCmd userCmdMsg;
-  userCmdMsg.set_description("Scale [" + _vis->GetName() + "]");
+  userCmdMsg.set_description("Scale [" + _vis->Name() + "]");
   userCmdMsg.set_type(msgs::UserCmd::SCALING);
 
   msgs::Model msg;
 
-  auto id = gui::get_entity_id(_vis->GetName());
+  auto id = gui::get_entity_id(_vis->Name());
   if (id)
     msg.set_id(id);
 
-  msg.set_name(_vis->GetName());
-  msgs::Set(msg.mutable_scale(), _vis->GetScale().Ign());
+  msg.set_name(_vis->Name());
+  msgs::Set(msg.mutable_scale(), _vis->Scale());
 
   auto modelMsg = userCmdMsg.add_model();
   modelMsg->CopyFrom(msg);
@@ -623,7 +625,7 @@ void ModelManipulator::OnMousePressEvent(const common::MouseEvent &_event)
   rendering::VisualPtr vis;
   std::string manipState;
   rendering::VisualPtr mouseVis
-      = this->dataPtr->userCamera->GetVisual(this->dataPtr->mouseStart,
+      = this->dataPtr->userCamera->Visual(this->dataPtr->mouseStart,
       manipState);
 
   this->dataPtr->selectionObj->SetState(manipState);
@@ -650,38 +652,32 @@ void ModelManipulator::OnMousePressEvent(const common::MouseEvent &_event)
     // Root visual
     rendering::VisualPtr rootVis = vis->GetRootVisual();
 
-    // Root visual's immediate child
-    rendering::VisualPtr topLevelVis = vis->GetNthAncestor(2);
+    // If the root visual's ID can be found, it is a model in simulation mode,
+    // so we select it instead of a child link/visual/collision.
 
-    // If the root visual's ID can be found, it is a model in the main window
-    // TODO gui::get_entity_id always return 0 in QTestFixture due to NULL
+    // If the visual's depth is less than 2, it is a root visual already. This
+    // is the case of lights in simulation mode for example.
+
+    // TODO gui::get_entity_id always return 0 in QTestFixture due to nullptr
     // g_main_win
-    if (gui::get_entity_id(rootVis->GetName()))
+    if (gui::get_entity_id(rootVis->Name()) || vis->GetDepth() < 2)
     {
       // select model
       vis = rootVis;
     }
-    // If it is not a model and its parent is either a direct child or
-    // grandchild of the world, this is a light, so just keep vis = vis
-    else if (vis->GetParent() == rootVis ||
-        vis->GetParent() == this->dataPtr->scene->WorldVisual())
-    {
-      // select light
-    }
-    // Otherwise, this is a visual in the model editor, so we want to get its
-    // top level visual below the root.
+    // Otherwise, we assume we're in the model editor and get the first child
+    // visual after the root, which may be a link or a nested model.
     else
     {
-      // select link / nested model
-      vis = topLevelVis;
+      vis = vis->GetNthAncestor(2);
     }
 
-    this->dataPtr->mouseMoveVisStartPose = vis->GetWorldPose();
+    this->dataPtr->mouseMoveVisStartPose = vis->WorldPose();
 
     this->SetMouseMoveVisual(vis);
 
     event::Events::setSelectedEntity(
-        this->dataPtr->mouseMoveVis->GetName(), "move");
+        this->dataPtr->mouseMoveVis->Name(), "move");
     QApplication::setOverrideCursor(Qt::ClosedHandCursor);
 
     if (this->dataPtr->mouseMoveVis && !this->dataPtr->mouseMoveVis->IsPlane())
@@ -709,18 +705,24 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
     if (this->dataPtr->mouseMoveVis &&
         this->dataPtr->mouseEvent.Button() == common::MouseEvent::LEFT)
     {
-      math::Vector3 axis = math::Vector3::Zero;
+      if (this->dataPtr->transparent)
+      {
+        this->dataPtr->mouseMoveVis->SetTransparency(
+          (1.0 - this->dataPtr->mouseMoveVis->GetTransparency()) * 0.5);
+        this->dataPtr->transparent = false;
+      }
+      ignition::math::Vector3d axis = ignition::math::Vector3d::Zero;
       if (this->dataPtr->keyEvent.key == Qt::Key_X)
-        axis.x = 1;
+        axis.X() = 1;
       else if (this->dataPtr->keyEvent.key == Qt::Key_Y)
-        axis.y = 1;
+        axis.Y() = 1;
       else if (this->dataPtr->keyEvent.key == Qt::Key_Z)
-        axis.z = 1;
+        axis.Z() = 1;
 
       if (this->dataPtr->selectionObj->GetMode() ==
           rendering::SelectionObj::TRANS)
       {
-        if (axis != math::Vector3::Zero)
+        if (axis != ignition::math::Vector3d::Zero)
         {
           this->TranslateEntity(this->dataPtr->mouseMoveVis, axis, false);
         }
@@ -728,30 +730,30 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
             == rendering::SelectionObj::TRANS_X)
         {
           this->TranslateEntity(this->dataPtr->mouseMoveVis,
-              math::Vector3::UnitX, !this->dataPtr->globalManip);
+              ignition::math::Vector3d::UnitX, !this->dataPtr->globalManip);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::TRANS_Y)
         {
           this->TranslateEntity(this->dataPtr->mouseMoveVis,
-              math::Vector3::UnitY, !this->dataPtr->globalManip);
+              ignition::math::Vector3d::UnitY, !this->dataPtr->globalManip);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::TRANS_Z)
         {
           this->TranslateEntity(this->dataPtr->mouseMoveVis,
-            math::Vector3::UnitZ, !this->dataPtr->globalManip);
+            ignition::math::Vector3d::UnitZ, !this->dataPtr->globalManip);
         }
         else
         {
           this->TranslateEntity(
-              this->dataPtr->mouseMoveVis, math::Vector3(1, 1, 0));
+              this->dataPtr->mouseMoveVis, ignition::math::Vector3d(1, 1, 0));
         }
       }
       else if (this->dataPtr->selectionObj->GetMode()
           == rendering::SelectionObj::ROT)
       {
-        if (axis != math::Vector3::Zero)
+        if (axis != ignition::math::Vector3d::Zero)
         {
           this->RotateEntity(this->dataPtr->mouseMoveVis, axis, false);
         }
@@ -759,28 +761,31 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
             == rendering::SelectionObj::ROT_X
             || this->dataPtr->keyEvent.key == Qt::Key_X)
         {
-          this->RotateEntity(this->dataPtr->mouseMoveVis, math::Vector3::UnitX,
+          this->RotateEntity(this->dataPtr->mouseMoveVis,
+              ignition::math::Vector3d::UnitX,
               !this->dataPtr->globalManip);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::ROT_Y
             || this->dataPtr->keyEvent.key == Qt::Key_Y)
         {
-          this->RotateEntity(this->dataPtr->mouseMoveVis, math::Vector3::UnitY,
+          this->RotateEntity(this->dataPtr->mouseMoveVis,
+              ignition::math::Vector3d::UnitY,
               !this->dataPtr->globalManip);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::ROT_Z
             || this->dataPtr->keyEvent.key == Qt::Key_Z)
         {
-          this->RotateEntity(this->dataPtr->mouseMoveVis, math::Vector3::UnitZ,
+          this->RotateEntity(this->dataPtr->mouseMoveVis,
+              ignition::math::Vector3d::UnitZ,
               !this->dataPtr->globalManip);
         }
       }
       else if (this->dataPtr->selectionObj->GetMode()
           == rendering::SelectionObj::SCALE)
       {
-        if (axis != math::Vector3::Zero)
+        if (axis != ignition::math::Vector3d::Zero)
         {
           this->ScaleEntity(this->dataPtr->mouseMoveVis, axis, false);
         }
@@ -789,21 +794,21 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
             || this->dataPtr->keyEvent.key == Qt::Key_X)
         {
           this->ScaleEntity(this->dataPtr->mouseMoveVis,
-              math::Vector3::UnitX, true);
+              ignition::math::Vector3d::UnitX, true);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::SCALE_Y
             || this->dataPtr->keyEvent.key == Qt::Key_Y)
         {
           this->ScaleEntity(this->dataPtr->mouseMoveVis,
-              math::Vector3::UnitY, true);
+              ignition::math::Vector3d::UnitY, true);
         }
         else if (this->dataPtr->selectionObj->GetState()
             == rendering::SelectionObj::SCALE_Z
             || this->dataPtr->keyEvent.key == Qt::Key_Z)
         {
           this->ScaleEntity(this->dataPtr->mouseMoveVis,
-              math::Vector3::UnitZ, true);
+              ignition::math::Vector3d::UnitZ, true);
         }
       }
     }
@@ -813,7 +818,7 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
   else
   {
     std::string manipState;
-    this->dataPtr->userCamera->GetVisual(this->dataPtr->mouseEvent.Pos(),
+    this->dataPtr->userCamera->Visual(this->dataPtr->mouseEvent.Pos(),
         manipState);
     this->dataPtr->selectionObj->SetState(manipState);
 
@@ -821,7 +826,7 @@ void ModelManipulator::OnMouseMoveEvent(const common::MouseEvent &_event)
       QApplication::setOverrideCursor(Qt::OpenHandCursor);
     else
     {
-      rendering::VisualPtr vis = this->dataPtr->userCamera->GetVisual(
+      rendering::VisualPtr vis = this->dataPtr->userCamera->Visual(
           this->dataPtr->mouseEvent.Pos());
 
       if (vis && !vis->IsPlane())
@@ -843,6 +848,8 @@ void ModelManipulator::OnMouseReleaseEvent(const common::MouseEvent &_event)
     // server
     if (this->dataPtr->mouseMoveVis)
     {
+      this->dataPtr->mouseMoveVis->SetTransparency(
+        std::abs(this->dataPtr->mouseMoveVis->GetTransparency()*2.0-1.0));
       if (this->dataPtr->manipMode == "scale")
       {
         this->dataPtr->selectionObj->UpdateSize();
@@ -860,7 +867,7 @@ void ModelManipulator::OnMouseReleaseEvent(const common::MouseEvent &_event)
     if (this->dataPtr->mouseEvent.Button() == common::MouseEvent::LEFT)
     {
       rendering::VisualPtr vis =
-        this->dataPtr->userCamera->GetVisual(this->dataPtr->mouseEvent.Pos());
+        this->dataPtr->userCamera->Visual(this->dataPtr->mouseEvent.Pos());
       if (vis && vis->IsPlane())
       {
         this->dataPtr->selectionObj->SetMode(
@@ -892,10 +899,10 @@ void ModelManipulator::SetAttachedVisual(rendering::VisualPtr _vis)
 {
   rendering::VisualPtr vis = _vis;
 
-  if (gui::get_entity_id(vis->GetRootVisual()->GetName()))
+  if (gui::get_entity_id(vis->GetRootVisual()->Name()))
     vis = vis->GetRootVisual();
 
-  this->dataPtr->mouseMoveVisStartPose = vis->GetWorldPose();
+  this->dataPtr->mouseMoveVisStartPose = vis->WorldPose();
 
   this->SetMouseMoveVisual(vis);
 
@@ -909,19 +916,20 @@ void ModelManipulator::SetMouseMoveVisual(rendering::VisualPtr _vis)
   this->dataPtr->mouseMoveVis = _vis;
   if (_vis)
   {
-    this->dataPtr->mouseVisualScale = _vis->GetScale();
+    this->dataPtr->transparent = true;
+    this->dataPtr->mouseVisualScale = _vis->Scale();
     this->dataPtr->mouseChildVisualScale.clear();
     // keep track of all child visual scale for scaling to work in
     // model editor mode.
     for (unsigned int i = 0; i < _vis->GetChildCount(); ++i)
     {
       rendering::VisualPtr childVis = _vis->GetChild(i);
-      this->dataPtr->mouseChildVisualScale.push_back(childVis->GetScale());
+      this->dataPtr->mouseChildVisualScale.push_back(childVis->Scale());
     }
-    this->dataPtr->mouseVisualBbox = _vis->GetBoundingBox();
+    this->dataPtr->mouseVisualBbox = _vis->BoundingBox();
   }
   else
-    this->dataPtr->mouseVisualScale = math::Vector3::One;
+    this->dataPtr->mouseVisualScale = ignition::math::Vector3d::One;
 }
 
 //////////////////////////////////////////////////
@@ -940,10 +948,10 @@ void ModelManipulator::OnKeyPressEvent(const common::KeyEvent &_event)
       if (this->dataPtr->mouseMoveVis)
       {
         this->dataPtr->mouseMoveVisStartPose =
-            this->dataPtr->mouseMoveVis->GetWorldPose();
+            this->dataPtr->mouseMoveVis->WorldPose();
       }
     }
-    else if (QApplication::keyboardModifiers() & Qt::ShiftModifier)
+    else if (this->dataPtr->keyEvent.shift)
     {
       this->dataPtr->globalManip = true;
       this->dataPtr->selectionObj->SetGlobal(this->dataPtr->globalManip);
@@ -967,10 +975,10 @@ void ModelManipulator::OnKeyReleaseEvent(const common::KeyEvent &_event)
       if (this->dataPtr->mouseMoveVis)
       {
         this->dataPtr->mouseMoveVisStartPose =
-            this->dataPtr->mouseMoveVis->GetWorldPose();
+            this->dataPtr->mouseMoveVis->WorldPose();
       }
     }
-    else if (this->dataPtr->keyEvent.key == Qt::Key_Shift)
+    else if (!this->dataPtr->keyEvent.shift)
     {
       this->dataPtr->globalManip = false;
       this->dataPtr->selectionObj->SetGlobal(this->dataPtr->globalManip);
@@ -979,6 +987,19 @@ void ModelManipulator::OnKeyReleaseEvent(const common::KeyEvent &_event)
   this->dataPtr->keyEvent.key = 0;
 }
 
+//////////////////////////////////////////////////
+ModelManipulator* ModelManipulator::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<ModelManipulator>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
+
 // Function migrated here from GLWidget.cc and commented out since it doesn't
 // seem like it's currently used. Kept here for future references
 /////////////////////////////////////////////////
@@ -988,57 +1009,58 @@ void ModelManipulator::OnKeyReleaseEvent(const common::KeyEvent &_event)
     return;
 
   // Get the point on the plane which correspoinds to the mouse
-  math::Vector3 pp;
+  ignition::math::Vector3d pp;
 
   // Rotate the visual using the middle mouse button
   if (this->dataPtr->mouseEvent.buttons == common::MouseEvent::MIDDLE)
   {
-    math::Vector3 rpy = this->dataPtr->mouseMoveVisStartPose.rot.GetAsEuler();
-    math::Vector2i delta = this->dataPtr->mouseEvent.pos -
+    auto rpy = this->dataPtr->mouseMoveVisStartPose.Rot().Euler();
+    auto delta = this->dataPtr->mouseEvent.Pos() -
         this->dataPtr->mouseEvent.pressPos;
-    double yaw = (delta.x * 0.01) + rpy.z;
+    double yaw = (delta.X() * 0.01) + rpy.Z();
     if (!this->dataPtr->mouseEvent.shift)
     {
       double snap = rint(yaw / (M_PI * .25)) * (M_PI * 0.25);
 
-      if (fabs(yaw - snap) < GZ_DTOR(10))
+      if (fabs(yaw - snap) < IGN_DTOR(10))
         yaw = snap;
     }
 
-    _vis->SetWorldRotation(math::Quaternion(rpy.x, rpy.y, yaw));
+    _vis->SetWorldRotation(ignition::math::Quaterniond(rpy.X(), rpy.Y(), yaw));
   }
   else if (this->dataPtr->mouseEvent.buttons == common::MouseEvent::RIGHT)
   {
-    math::Vector3 rpy = this->dataPtr->mouseMoveVisStartPose.rot.GetAsEuler();
-    math::Vector2i delta = this->dataPtr->mouseEvent.pos -
+    auto rpy = this->dataPtr->mouseMoveVisStartPose.Rot().Euler();
+    auto delta = this->dataPtr->mouseEvent.Pos() -
         this->dataPtr->mouseEvent.pressPos;
-    double pitch = (delta.y * 0.01) + rpy.y;
+    double pitch = (delta.Y() * 0.01) + rpy.Y();
     if (!this->dataPtr->mouseEvent.shift)
     {
       double snap = rint(pitch / (M_PI * .25)) * (M_PI * 0.25);
 
-      if (fabs(pitch - snap) < GZ_DTOR(10))
+      if (fabs(pitch - snap) < IGN_DTOR(10))
         pitch = snap;
     }
 
-    _vis->SetWorldRotation(math::Quaternion(rpy.x, pitch, rpy.z));
+    _vis->SetWorldRotation(
+        ignition::math::Quaterniond(rpy.X(), pitch, rpy.Z()));
   }
   else if (this->dataPtr->mouseEvent.buttons & common::MouseEvent::LEFT &&
            this->dataPtr->mouseEvent.buttons & common::MouseEvent::RIGHT)
   {
-    math::Vector3 rpy = this->dataPtr->mouseMoveVisStartPose.rot.GetAsEuler();
-    math::Vector2i delta = this->dataPtr->mouseEvent.pos -
+    auto rpy = this->dataPtr->mouseMoveVisStartPose.Rot().Euler();
+    auto delta = this->dataPtr->mouseEvent.Pos() -
         this->dataPtr->mouseEvent.pressPos;
-    double roll = (delta.x * 0.01) + rpy.x;
+    double roll = (delta.X() * 0.01) + rpy.X();
     if (!this->dataPtr->mouseEvent.shift)
     {
       double snap = rint(roll / (M_PI * .25)) * (M_PI * 0.25);
 
-      if (fabs(roll - snap) < GZ_DTOR(10))
+      if (fabs(roll - snap) < IGN_DTOR(10))
         roll = snap;
     }
 
-    _vis->SetWorldRotation(math::Quaternion(roll, rpy.y, rpy.z));
+    _vis->SetWorldRotation(ignition::math::Quaterniond(roll, rpy.Y(), rpy.Z()));
   }
   else
   {
