@@ -13,7 +13,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
-*/
+ */
+
+#include <functional>
+
+#include <ignition/math/Matrix3.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Quaternion.hh>
+#include <ignition/math/Vector3.hh>
 
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Console.hh"
@@ -51,10 +58,15 @@ DARTLink::~DARTLink()
 //////////////////////////////////////////////////
 void DARTLink::Load(sdf::ElementPtr _sdf)
 {
-  this->dataPtr->dartPhysics = boost::dynamic_pointer_cast<DARTPhysics>(
-      this->GetWorld()->GetPhysicsEngine());
+  Link::Load(_sdf);
 
-  if (this->dataPtr->dartPhysics == NULL)
+  // Name
+  std::string bodyName = this->GetName();
+
+  this->dataPtr->dartPhysics = boost::dynamic_pointer_cast<DARTPhysics>(
+      this->GetWorld()->Physics());
+
+  if (this->dataPtr->dartPhysics == nullptr)
     gzthrow("Not using the dart physics engine");
 
   // Check if soft_contact element is contained in this link. If so,
@@ -84,7 +96,7 @@ void DARTLink::Load(sdf::ElementPtr _sdf)
 
           if (softContactElem->HasElement("dart"))
           {
-            if (dartElem != NULL)
+            if (dartElem)
             {
               gzerr << "DART supports only one deformable body in a link.\n";
               break;
@@ -101,60 +113,53 @@ void DARTLink::Load(sdf::ElementPtr _sdf)
     }
   }
 
-  if (dartElem != NULL)
+  if (dartElem)
   {
-    // Create DART SoftBodyNode
-    dart::dynamics::SoftBodyNode *dtSoftBodyNode
-        = new dart::dynamics::SoftBodyNode();
+    dart::dynamics::SoftBodyNode::UniqueProperties softProperties;
 
     // Mass
     double fleshMassFraction = dartElem->Get<double>("flesh_mass_fraction");
 
     // bone_attachment (Kv)
+    double boneAttachment = dart::dynamics::DART_DEFAULT_VERTEX_STIFFNESS;
     if (dartElem->HasElement("bone_attachment"))
-    {
-      double kv = dartElem->Get<double>("bone_attachment");
-      dtSoftBodyNode->setVertexSpringStiffness(kv);
-    }
+      boneAttachment = dartElem->Get<double>("bone_attachment");
 
     // stiffness (Ke)
+    double stiffness = dart::dynamics::DART_DEFAULT_EDGE_STIFNESS;
     if (dartElem->HasElement("stiffness"))
-    {
-      double ke = dartElem->Get<double>("stiffness");
-      dtSoftBodyNode->setEdgeSpringStiffness(ke);
-    }
+      stiffness = dartElem->Get<double>("stiffness");
 
     // damping
+    double damping = dart::dynamics::DART_DEFAULT_DAMPING_COEFF;
     if (dartElem->HasElement("damping"))
-    {
-      double damping = dartElem->Get<double>("damping");
-      dtSoftBodyNode->setDampingCoefficient(damping);
-    }
+      damping = dartElem->Get<double>("damping");
 
     // pose
     Eigen::Isometry3d T = Eigen::Isometry3d::Identity();
     gzdbg << "pose" << T.matrix() << std::endl;
+    // TODO(addisu) Check if this needs semantic pose
     if (softCollElem->HasElement("pose"))
     {
-      T = DARTTypes::ConvPose(softCollElem->Get<math::Pose>("pose"));
+      T = DARTTypes::ConvPose(
+          softCollElem->Get<ignition::math::Pose3d>("pose"));
     }
 
     // geometry
     if (softGeomElem->HasElement("box"))
     {
       sdf::ElementPtr boxEle = softGeomElem->GetElement("box");
-      Eigen::Vector3d size
-          = DARTTypes::ConvVec3(boxEle->Get<math::Vector3>("size"));
-      dart::dynamics::SoftBodyNodeHelper::setBox(
-            dtSoftBodyNode, size, T, fleshMassFraction);
-      dtSoftBodyNode->addCollisionShape(
-            new dart::dynamics::SoftMeshShape(dtSoftBodyNode));
+      Eigen::Vector3d size =
+          DARTTypes::ConvVec3(boxEle->Get<ignition::math::Vector3d>("size"));
+      softProperties = dart::dynamics::SoftBodyNodeHelper::makeBoxProperties(
+            size, T, fleshMassFraction, boneAttachment, stiffness, damping);
     }
 //    else if (geomElem->HasElement("ellipsoid"))
 //    {
 //      sdf::ElementPtr ellipsoidEle = geomElem->GetElement("ellipsoid");
 //      Eigen::Vector3d size
-//          = DARTTypes::ConvVec3(ellipsoidEle->Get<math::Vector3>("size"));
+//          = DARTTypes::ConvVec3(ellipsoidEle->Get<
+//          ignition::math::Vector3d>("size"));
 //      double nSlices = ellipsoidEle->Get<double>("num_slices");
 //      double nStacks = ellipsoidEle->Get<double>("num_stacks");
 //      dart::dynamics::SoftBodyNodeHelper::setEllipsoid(
@@ -167,15 +172,21 @@ void DARTLink::Load(sdf::ElementPtr _sdf)
       gzerr << "Unknown soft shape" << std::endl;
     }
 
-    this->dataPtr->dtBodyNode = dtSoftBodyNode;
+    // Create DART SoftBodyNode properties
+    dart::dynamics::BodyNode::AspectProperties properties(bodyName);
+    this->dataPtr->dtProperties.reset(
+          new dart::dynamics::SoftBodyNode::Properties(
+            properties, softProperties));
+
+    this->dataPtr->isSoftBody;
   }
   else
   {
-    // Create DART BodyNode
-    this->dataPtr->dtBodyNode = new dart::dynamics::BodyNode();
+    // Create DART BodyNode properties
+    dart::dynamics::BodyNode::AspectProperties properties(bodyName);
+    this->dataPtr->dtProperties.reset(
+          new dart::dynamics::BodyNode::Properties(properties));
   }
-
-  Link::Load(_sdf);
 }
 
 //////////////////////////////////////////////////
@@ -183,33 +194,23 @@ void DARTLink::Init()
 {
   Link::Init();
 
+  this->dataPtr->Initialize();
+
+  // DARTModel::Load() should be called first
+  GZ_ASSERT(this->dataPtr->dtBodyNode,
+            "DART BodyNode is not initialized.");
+
   // Name
   std::string bodyName = this->GetName();
   this->dataPtr->dtBodyNode->setName(bodyName);
 
-  // Mass
-  double mass = this->inertial->GetMass();
-  this->dataPtr->dtBodyNode->setMass(mass);
-
   // Inertia
-  double Ixx = this->inertial->GetIXX();
-  double Iyy = this->inertial->GetIYY();
-  double Izz = this->inertial->GetIZZ();
-  double Ixy = this->inertial->GetIXY();
-  double Ixz = this->inertial->GetIXZ();
-  double Iyz = this->inertial->GetIYZ();
-  this->dataPtr->dtBodyNode->setMomentOfInertia(Ixx, Iyy, Izz, Ixy, Ixz, Iyz);
-
-  // Visual
-  this->visuals;
-
-  // COG offset
-  math::Vector3 cog = this->inertial->GetCoG();
-  this->dataPtr->dtBodyNode->setLocalCOM(DARTTypes::ConvVec3(cog));
+  this->UpdateMass();
 
   // Gravity mode
   this->SetGravityMode(this->sdf->Get<bool>("gravity"));
 
+#if DART_MAJOR_MINOR_VERSION_AT_MOST(6, 9)
   // Friction coefficient
 
   /// \todo FIXME: Friction Parameters
@@ -248,7 +249,11 @@ void DARTLink::Init()
   hackAvgMu1 /= static_cast<double>(numCollisions);
   hackAvgMu2 /= static_cast<double>(numCollisions);
 
-  this->dataPtr->dtBodyNode->setFrictionCoeff(0.5 * (hackAvgMu1 + hackAvgMu2));
+  float coeff = 0.5 * (hackAvgMu1 + hackAvgMu2);
+  // friction coefficient may not be negative in DART
+  coeff = std::max(0.0f, coeff);
+  this->dataPtr->dtBodyNode->setFrictionCoeff(coeff);
+#endif
 
   // We don't add dart body node to the skeleton here because dart body node
   // should be set its parent joint before being added. This body node will be
@@ -266,36 +271,56 @@ void DARTLink::UpdateMass()
 {
   if (this->dataPtr->dtBodyNode && this->inertial)
   {
-    this->dataPtr->dtBodyNode->setMass(this->inertial->GetMass());
-    auto Ixxyyzz = this->inertial->GetPrincipalMoments();
-    auto Ixyxzyz = this->inertial->GetProductsofInertia();
+    double nFragments = 1.0 + this->dataPtr->dtSlaveNodes.size();
+    double mass = this->inertial->Mass()/nFragments;
+    this->dataPtr->dtBodyNode->setMass(mass);
+
+    const ignition::math::Quaterniond R_inertial = this->inertial->Pose().Rot();
+
+    const ignition::math::Matrix3d I_link =
+        ignition::math::Matrix3d(R_inertial)
+        *this->inertial->MOI()
+        *ignition::math::Matrix3d(R_inertial.Inverse())
+        *(1.0/nFragments);
+
     this->dataPtr->dtBodyNode->setMomentOfInertia(
-        Ixxyyzz[0], Ixxyyzz[1], Ixxyyzz[2],
-        Ixyxzyz[0], Ixyxzyz[1], Ixyxzyz[2]);
-    auto cog = DARTTypes::ConvVec3(this->inertial->GetCoG());
+        I_link(0, 0), I_link(1, 1), I_link(2, 2),
+        I_link(0, 1), I_link(0, 2), I_link(1, 2));
+
+    const auto cog = DARTTypes::ConvVec3(this->inertial->CoG());
     this->dataPtr->dtBodyNode->setLocalCOM(cog);
+
+    for (auto slaveNode : this->dataPtr->dtSlaveNodes)
+    {
+      slaveNode.first->setMass(mass);
+      slaveNode.first->setMomentOfInertia(
+        I_link(0, 0), I_link(1, 1), I_link(2, 2),
+        I_link(0, 1), I_link(0, 2), I_link(1, 2));
+      slaveNode.first->setLocalCOM(cog);
+    }
   }
 }
 
 //////////////////////////////////////////////////
 void DARTLink::OnPoseChange()
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache("DARTLink::OnPoseChange",
+                         std::bind(&DARTLink::OnPoseChange, this));
+    return;
+  }
+
   Link::OnPoseChange();
 
   // DART body node always have its parent joint.
   dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
 
-  // This is for the case this function called before DARTModel::Init() is
-  // called.
-  if (joint == NULL)
-    return;
-
-  dart::dynamics::FreeJoint *freeJoint =
-      dynamic_cast<dart::dynamics::FreeJoint*>(joint);
-  if (freeJoint)
+  if (dart::dynamics::FreeJoint *freeJoint =
+          dynamic_cast<dart::dynamics::FreeJoint*>(joint))
   {
     // If the parent joint is free joint, set the 6 dof to fit the target pose.
-    const Eigen::Isometry3d &W = DARTTypes::ConvPose(this->GetWorldPose());
+    const Eigen::Isometry3d &W = DARTTypes::ConvPose(this->WorldPose());
     const Eigen::Isometry3d &T1 = joint->getTransformFromParentBodyNode();
     const Eigen::Isometry3d &InvT2 = joint->getTransformFromChildBodyNode();
     Eigen::Isometry3d P = Eigen::Isometry3d::Identity();
@@ -312,25 +337,7 @@ void DARTLink::OnPoseChange()
     // generalized coordinates. On the other hand, the position part just takes
     // the last three components of the generalized coordinates without any
     // conversion.
-    Eigen::Vector6d q;
-    q.head<3>() = dart::math::logMap(Q.linear());
-    q.tail<3>() = Q.translation();
-    freeJoint->setPositions(q);
-    // TODO: The above 4 lines will be reduced to single line as:
-    // freeJoint->setPositions(FreeJoint::convertToPositions(Q));
-    // after the following PR is merged:
-    // https://github.com/dartsim/dart/pull/322
-
-    // Update all the transformations of the links in the parent model.
-    freeJoint->getSkeleton()->computeForwardKinematics(true, false, false);
-    // TODO: This kinematic updating will be done automatically after pull
-    // request (https://github.com/dartsim/dart/pull/319) is merged so that
-    // we don't need this line anymore.
-  }
-  else
-  {
-    gzdbg << "OnPoseChange() doesn't make sense if the parent joint "
-          << "is not free joint (6-dof).\n";
+    freeJoint->setPositions(dart::dynamics::FreeJoint::convertToPositions(Q));
   }
 }
 
@@ -348,19 +355,19 @@ bool DARTLink::GetEnabled() const
 }
 
 //////////////////////////////////////////////////
-void DARTLink::SetLinearVel(const math::Vector3 &_vel)
+void DARTLink::SetLinearVel(const ignition::math::Vector3d &_vel)
 {
-  // DART body node always have its parent joint.
-  dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
-
-  // This is for the case this function called before DARTModel::Init() is
-  // called.
-  if (joint == NULL)
+  if (!this->dataPtr->IsInitialized())
   {
-    gzerr << "DARTModel::Init() should be called first.\n";
+    this->dataPtr->Cache("WorldLinearVel",
+                         std::bind(&DARTLink::SetLinearVel, this, _vel),
+                         _vel);
     return;
   }
 
+  // DART body node always have its parent joint.
+  dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
+
   // Check if the parent joint is free joint
   dart::dynamics::FreeJoint *freeJoint =
       dynamic_cast<dart::dynamics::FreeJoint*>(joint);
@@ -379,13 +386,13 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
     if (dtBodyNode->getParentBodyNode())
     {
       // Local transformation from the parent link frame to this link frame
-      Eigen::Isometry3d T = freeJoint->getLocalTransform();
+      Eigen::Isometry3d T = freeJoint->getRelativeTransform();
 
       // Parent link's linear and angular velocities
       Eigen::Vector3d parentLinVel =
-          dtBodyNode->getParentBodyNode()->getBodyLinearVelocity();
+          dtBodyNode->getParentBodyNode()->getLinearVelocity();
       Eigen::Vector3d parentAngVel =
-          dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
+          dtBodyNode->getParentBodyNode()->getAngularVelocity();
 
       // The effect of the parent link's velocities
       Eigen::Vector3d propagatedLinVel =
@@ -406,9 +413,6 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
     freeJoint->setVelocity(3, genVel[0]);
     freeJoint->setVelocity(4, genVel[1]);
     freeJoint->setVelocity(5, genVel[2]);
-
-    // Update spatial velocities of all the links in the model
-    freeJoint->getSkeleton()->computeForwardKinematics(false, true, false);
   }
   else
   {
@@ -418,14 +422,22 @@ void DARTLink::SetLinearVel(const math::Vector3 &_vel)
 }
 
 //////////////////////////////////////////////////
-void DARTLink::SetAngularVel(const math::Vector3 &_vel)
+void DARTLink::SetAngularVel(const ignition::math::Vector3d &_vel)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache("WorldAngularVel",
+                         std::bind(&DARTLink::SetAngularVel, this, _vel),
+                         _vel);
+    return;
+  }
+
   // DART body node always have its parent joint.
   dart::dynamics::Joint *joint = this->dataPtr->dtBodyNode->getParentJoint();
 
   // This is for the case this function called before DARTModel::Init() is
   // called.
-  if (joint == NULL)
+  if (joint == nullptr)
   {
     gzerr << "DARTModel::Init() should be called first.\n";
     return;
@@ -449,11 +461,11 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
     if (dtBodyNode->getParentBodyNode())
     {
       // Local transformation from the parent link frame to this link frame
-      Eigen::Isometry3d T = freeJoint->getLocalTransform();
+      Eigen::Isometry3d T = freeJoint->getRelativeTransform();
 
       // Parent link's linear and angular velocities
       Eigen::Vector3d parentAngVel =
-          dtBodyNode->getParentBodyNode()->getBodyAngularVelocity();
+          dtBodyNode->getParentBodyNode()->getAngularVelocity();
 
       // The effect of the parent link's velocities
       Eigen::Vector3d propagatedAngVel = T.linear().transpose() * parentAngVel;
@@ -472,161 +484,274 @@ void DARTLink::SetAngularVel(const math::Vector3 &_vel)
     freeJoint->setVelocity(0, genVel[0]);
     freeJoint->setVelocity(1, genVel[1]);
     freeJoint->setVelocity(2, genVel[2]);
-
-    // Update spatial velocities of all the links in the model
-    freeJoint->getSkeleton()->computeForwardKinematics(false, true, false);
   }
   else
   {
-    gzdbg << "DARTLink::SetLinearVel() doesn't make sense if the parent joint "
+    gzdbg << "DARTLink::SetAngularVel() doesn't make sense if the parent joint "
           << "is not free joint (6-dof).\n";
   }
 }
 
 //////////////////////////////////////////////////
-void DARTLink::SetForce(const math::Vector3 &_force)
+void DARTLink::SetForce(const ignition::math::Vector3d &_force)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "Force", std::bind(&DARTLink::SetForce, this, _force));
+    return;
+  }
+
   // DART assume that _force is external force.
   this->dataPtr->dtBodyNode->setExtForce(DARTTypes::ConvVec3(_force));
 }
 
 //////////////////////////////////////////////////
-void DARTLink::SetTorque(const math::Vector3 &_torque)
+void DARTLink::SetTorque(const ignition::math::Vector3d &_torque)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "Torque", std::bind(&DARTLink::SetTorque, this, _torque));
+    return;
+  }
+
   // DART assume that _torque is external torque.
   this->dataPtr->dtBodyNode->setExtTorque(DARTTypes::ConvVec3(_torque));
 }
 
 //////////////////////////////////////////////////
-void DARTLink::AddForce(const math::Vector3 &_force)
+void DARTLink::AddForce(const ignition::math::Vector3d &_force)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "Force", std::bind(&DARTLink::AddForce, this, _force));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force));
 }
 
 /////////////////////////////////////////////////
-void DARTLink::AddRelativeForce(const math::Vector3 &_force)
+void DARTLink::AddRelativeForce(const ignition::math::Vector3d &_force)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "RelativeForce",
+          std::bind(&DARTLink::AddRelativeForce, this, _force));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
                                 Eigen::Vector3d::Zero(),
                                 true, true);
 }
 
 /////////////////////////////////////////////////
-void DARTLink::AddForceAtWorldPosition(const math::Vector3 &_force,
-                                        const math::Vector3 &_pos)
+void DARTLink::AddForceAtWorldPosition(const ignition::math::Vector3d &_force,
+                                        const ignition::math::Vector3d &_pos)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "ForceAtWorldPosition",
+          std::bind(&DARTLink::AddForceAtWorldPosition, this, _force, _pos));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_pos),
                                 DARTTypes::ConvVec3(_force),
                                 false, false);
 }
 
 /////////////////////////////////////////////////
-void DARTLink::AddForceAtRelativePosition(const math::Vector3 &_force,
-                                          const math::Vector3 &_relpos)
+void DARTLink::AddForceAtRelativePosition(
+    const ignition::math::Vector3d &_force,
+    const ignition::math::Vector3d &_relpos)
 {
-  this->dataPtr->dtBodyNode->addExtForce(DARTTypes::ConvVec3(_force),
-                                DARTTypes::ConvVec3(_relpos),
-                                true, true);
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "ForceAtRelativePosition",
+          std::bind(
+            &DARTLink::AddForceAtRelativePosition, this, _force, _relpos));
+    return;
+  }
+
+  this->dataPtr->dtBodyNode->addExtForce(
+        DARTTypes::ConvVec3(_force),
+        DARTTypes::ConvVec3(_relpos) + this->dataPtr->dtBodyNode->getLocalCOM(),
+        false, true);
 }
 
 //////////////////////////////////////////////////
-void DARTLink::AddLinkForce(const math::Vector3 &/*_force*/,
-    const math::Vector3 &/*_offset*/)
+void DARTLink::AddLinkForce(
+    const ignition::math::Vector3d &_force,
+    const ignition::math::Vector3d &_offset)
 {
-  gzlog << "DARTLink::AddLinkForce not yet implemented (issue #1477)."
-        << std::endl;
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "AddLinkForce",
+          std::bind(
+            &DARTLink::AddLinkForce, this, _force, _offset));
+    return;
+  }
+
+  this->dataPtr->dtBodyNode->addExtForce(
+        DARTTypes::ConvVec3(_force),
+        DARTTypes::ConvVec3(_offset),
+        true, true);
 }
 
 /////////////////////////////////////////////////
-void DARTLink::AddTorque(const math::Vector3 &_torque)
+void DARTLink::AddTorque(const ignition::math::Vector3d &_torque)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "Torque", std::bind(&DARTLink::AddTorque, this, _torque));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque));
 }
 
 /////////////////////////////////////////////////
-void DARTLink::AddRelativeTorque(const math::Vector3 &_torque)
+void DARTLink::AddRelativeTorque(const ignition::math::Vector3d &_torque)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "RelativeTorque",
+          std::bind(&DARTLink::AddRelativeTorque, this, _torque));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->addExtTorque(DARTTypes::ConvVec3(_torque), true);
 }
 
 //////////////////////////////////////////////////
-gazebo::math::Vector3 DARTLink::GetWorldLinearVel(
-    const math::Vector3 &_offset) const
+ignition::math::Vector3d DARTLink::WorldLinearVel(
+    const ignition::math::Vector3d &_offset) const
 {
-  Eigen::Vector3d linVel =
-      this->dataPtr->dtBodyNode->getWorldLinearVelocity(
+  if (!this->dataPtr->IsInitialized())
+    return this->dataPtr->GetCached<ignition::math::Vector3d>("WorldLinearVel");
+
+  Eigen::Vector3d linVel = this->dataPtr->dtBodyNode->getLinearVelocity(
         DARTTypes::ConvVec3(_offset));
 
-  return DARTTypes::ConvVec3(linVel);
+  return DARTTypes::ConvVec3Ign(linVel);
 }
 
 //////////////////////////////////////////////////
-math::Vector3 DARTLink::GetWorldLinearVel(
-    const gazebo::math::Vector3 &_offset,
-    const gazebo::math::Quaternion &_q) const
+ignition::math::Vector3d DARTLink::WorldLinearVel(
+    const ignition::math::Vector3d &_offset,
+    const ignition::math::Quaterniond &_q) const
 {
+  if (!this->dataPtr->IsInitialized())
+    return this->dataPtr->GetCached<ignition::math::Vector3d>("WorldLinearVel");
+
   Eigen::Matrix3d R1 = Eigen::Matrix3d(DARTTypes::ConvQuat(_q));
   Eigen::Vector3d worldOffset = R1 * DARTTypes::ConvVec3(_offset);
   Eigen::Vector3d bodyOffset =
       this->dataPtr->dtBodyNode->getTransform().linear().transpose() *
       worldOffset;
   Eigen::Vector3d linVel =
-      this->dataPtr->dtBodyNode->getWorldLinearVelocity(bodyOffset);
+      this->dataPtr->dtBodyNode->getLinearVelocity(bodyOffset);
 
-  return DARTTypes::ConvVec3(linVel);
+  return DARTTypes::ConvVec3Ign(linVel);
 }
 
 //////////////////////////////////////////////////
-math::Vector3 DARTLink::GetWorldCoGLinearVel() const
+ignition::math::Vector3d DARTLink::WorldCoGLinearVel() const
 {
-  Eigen::Vector3d linVel = this->dataPtr->dtBodyNode->getWorldCOMVelocity();
+  if (!this->dataPtr->IsInitialized())
+  {
+    return this->dataPtr->GetCached<ignition::math::Vector3d>(
+        "WorldCoGLinearVel");
+  }
 
-  return DARTTypes::ConvVec3(linVel);
+  Eigen::Vector3d linVel = this->dataPtr->dtBodyNode->getCOMLinearVelocity();
+
+  return DARTTypes::ConvVec3Ign(linVel);
 }
 
 //////////////////////////////////////////////////
-math::Vector3 DARTLink::GetWorldAngularVel() const
+ignition::math::Vector3d DARTLink::WorldAngularVel() const
 {
-  Eigen::Vector3d angVel = this->dataPtr->dtBodyNode->getWorldAngularVelocity();
+  if (!this->dataPtr->IsInitialized())
+  {
+    return this->dataPtr->GetCached<ignition::math::Vector3d>(
+        "WorldAngularVel");
+  }
 
-  return DARTTypes::ConvVec3(angVel);
+  Eigen::Vector3d angVel = this->dataPtr->dtBodyNode->getAngularVelocity();
+
+  return DARTTypes::ConvVec3Ign(angVel);
 }
 
 /////////////////////////////////////////////////
-math::Vector3 DARTLink::GetWorldForce() const
+ignition::math::Vector3d DARTLink::WorldForce() const
 {
+  if (!this->dataPtr->IsInitialized())
+    return this->dataPtr->GetCached<ignition::math::Vector3d>("WorldForce");
+
   Eigen::Vector6d F = this->dataPtr->dtBodyNode->getExternalForceGlobal();
-  return DARTTypes::ConvVec3(F.tail<3>());
+  return DARTTypes::ConvVec3Ign(F.tail<3>());
 }
 
 //////////////////////////////////////////////////
-math::Vector3 DARTLink::GetWorldTorque() const
+ignition::math::Vector3d DARTLink::WorldTorque() const
 {
-  // TODO: Need verification
-  math::Vector3 torque;
+  if (!this->dataPtr->IsInitialized())
+    return this->dataPtr->GetCached<ignition::math::Vector3d>("WorldTorque");
 
-  Eigen::Isometry3d W = this->dataPtr->dtBodyNode->getTransform();
-  Eigen::Matrix6d G   = this->dataPtr->dtBodyNode->getSpatialInertia();
-  Eigen::VectorXd V   = this->dataPtr->dtBodyNode->getBodyVelocity();
-  Eigen::VectorXd dV  = this->dataPtr->dtBodyNode->getBodyAcceleration();
-  Eigen::Vector6d F   = G * dV - dart::math::dad(V, G * V);
+  // DART uses spatial forces which express the torque in the first three
+  // components of the 6D vector
+  const Eigen::Vector6d f = this->dataPtr->dtBodyNode->getExternalForceGlobal();
 
-  torque = DARTTypes::ConvVec3(W.linear() * F.head<3>());
+  Eigen::Isometry3d tf_com = Eigen::Isometry3d::Identity();
+  // We use the negative of the global COM location in order to transform the
+  // spatial vector to be with respect to the the body's center of mass.
+  tf_com.translation() = -this->dataPtr->dtBodyNode->getCOM(
+        dart::dynamics::Frame::World());
 
-  return torque;
+  const Eigen::Vector6d f_com = dart::math::dAdInvT(tf_com, f);
+
+  return DARTTypes::ConvVec3Ign(f_com.head<3>());
 }
 
 //////////////////////////////////////////////////
 void DARTLink::SetGravityMode(bool _mode)
 {
   this->sdf->GetElement("gravity")->Set(_mode);
+
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "GravityMode", std::bind(&DARTLink::SetGravityMode, this, _mode));
+    return;
+  }
+
   this->dataPtr->dtBodyNode->setGravityMode(_mode);
+  for (auto slaveNode : this->dataPtr->dtSlaveNodes)
+  {
+    slaveNode.first->setGravityMode(_mode);
+  }
 }
 
 //////////////////////////////////////////////////
 bool DARTLink::GetGravityMode() const
 {
-  return this->dataPtr->dtBodyNode->getGravityMode();
+  GZ_ASSERT(!this->dataPtr->IsInitialized() ||
+            (this->dataPtr->dtBodyNode->getGravityMode() ==
+             this->sdf->Get<bool>("gravity")),
+            "Gazebo and DART disagree in gravity mode of the link.");
+
+  return this->sdf->Get<bool>("gravity");
 }
 
 //////////////////////////////////////////////////
@@ -634,17 +759,31 @@ void DARTLink::SetSelfCollide(bool _collide)
 {
   this->sdf->GetElement("self_collide")->Set(_collide);
 
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "SelfCollide",
+          std::bind(&DARTLink::SetSelfCollide, this, _collide));
+    return;
+  }
+
+
+  /*
+  Following code is disabled currently because the newer DART version (from 6) does not
+  support CollisionManager::enablePair() and CollisionMangager::disablePair().
+
   dart::dynamics::BodyNode *dtBodyNode = this->dataPtr->dtBodyNode;
 
   // If this function is called before the body node is not added to a skeleton,
   // the body node does not have parent skeleton. So we just return here. Self
   // collision setting will be done later in DARTModel::Init().
-  if (dtBodyNode->getSkeleton() == NULL)
+  if (dtBodyNode->getSkeleton() == nullptr)
     return;
 
-  dart::simulation::World *dtWorld = this->dataPtr->dartPhysics->GetDARTWorld();
-  dart::dynamics::Skeleton *dtSkeleton = dtBodyNode->getSkeleton();
-  dart::collision::CollisionDetector *dtCollDet =
+  dart::simulation::WorldPtr dtWorld =
+      this->dataPtr->dartPhysics->DARTWorld();
+  dart::dynamics::SkeletonPtr dtSkeleton = dtBodyNode->getSkeleton();
+  dart::collision::CollisionDetectorPtr dtCollDet =
       dtWorld->getConstraintSolver()->getCollisionDetector();
 
   Link_V links = this->GetModel()->GetLinks();
@@ -664,11 +803,11 @@ void DARTLink::SetSelfCollide(bool _collide)
         if (links[i].get() != this && links[i]->GetSelfCollide())
         {
           dart::dynamics::BodyNode *itdtBodyNode =
-            boost::dynamic_pointer_cast<DARTLink>(links[i])->GetDARTBodyNode();
+            boost::dynamic_pointer_cast<DARTLink>(links[i])->DARTBodyNode();
 
           // If this->dataPtr->dtBodyNode and itdtBodyNode are connected then
           // don't enable the pair.
-          // Please see: https://bitbucket.org/osrf/gazebo/issue/899
+          // Please see: https://github.com/osrf/gazebo/issues/899
           if ((dtBodyNode->getParentBodyNode() == itdtBodyNode) ||
               itdtBodyNode->getParentBodyNode() == this->dataPtr->dtBodyNode)
             continue;
@@ -683,20 +822,21 @@ void DARTLink::SetSelfCollide(bool _collide)
     // the pairs of which both of the links in the pair is not self collidable.
     else
     {
-      dtSkeleton->enableSelfCollision();
+      dtSkeleton->enableSelfCollisionCheck();
+      dtSkeleton->setAdjacentBodyCheck(false);
 
       for (size_t i = 0; i < links.size() - 1; ++i)
       {
         for (size_t j = i + 1; j < links.size(); ++j)
         {
           dart::dynamics::BodyNode *itdtBodyNode1 =
-            boost::dynamic_pointer_cast<DARTLink>(links[i])->GetDARTBodyNode();
+            boost::dynamic_pointer_cast<DARTLink>(links[i])->DARTBodyNode();
           dart::dynamics::BodyNode *itdtBodyNode2 =
-            boost::dynamic_pointer_cast<DARTLink>(links[j])->GetDARTBodyNode();
+            boost::dynamic_pointer_cast<DARTLink>(links[j])->DARTBodyNode();
 
           // If this->dataPtr->dtBodyNode and itdtBodyNode are connected then
           // don't enable the pair.
-          // Please see: https://bitbucket.org/osrf/gazebo/issue/899
+          // Please see: https://github.com/osrf/gazebo/issues/899
           if ((itdtBodyNode1->getParentBodyNode() == itdtBodyNode2) ||
               itdtBodyNode2->getParentBodyNode() == itdtBodyNode1)
             dtCollDet->disablePair(itdtBodyNode1, itdtBodyNode2);
@@ -718,7 +858,7 @@ void DARTLink::SetSelfCollide(bool _collide)
         if (links[i].get() != this)
         {
           dart::dynamics::BodyNode *itdtBodyNode =
-            boost::dynamic_pointer_cast<DARTLink>(links[i])->GetDARTBodyNode();
+            boost::dynamic_pointer_cast<DARTLink>(links[i])->DARTBodyNode();
           dtCollDet->disablePair(dtBodyNode, itdtBodyNode);
         }
       }
@@ -736,8 +876,12 @@ void DARTLink::SetSelfCollide(bool _collide)
       }
     }
     if (isAllLinksNotCollidable)
-      dtSkeleton->disableSelfCollision();
+    {
+      dtSkeleton->disableSelfCollisionCheck();
+      dtSkeleton->setAdjacentBodyCheck(false);
+    }
   }
+  */
 }
 
 //////////////////////////////////////////////////
@@ -778,24 +922,30 @@ void DARTLink::SetAutoDisable(bool /*_disable*/)
 //////////////////////////////////////////////////
 void DARTLink::SetLinkStatic(bool _static)
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "LinkStatic", std::bind(&DARTLink::SetLinkStatic, this, _static));
+    return;
+  }
+
   if (_static == this->dataPtr->staticLink)
     return;
 
   if (_static == true)
   {
     // Add weld joint constraint to DART
-    this->dataPtr->dtWeldJointConst =
-        new dart::constraint::WeldJointConstraint(this->dataPtr->dtBodyNode);
-    GetDARTWorld()->getConstraintSolver()->addConstraint(
+    this->dataPtr->dtWeldJointConst.reset(
+        new dart::constraint::WeldJointConstraint(this->dataPtr->dtBodyNode));
+    DARTWorld()->getConstraintSolver()->addConstraint(
         this->dataPtr->dtWeldJointConst);
   }
   else
   {
     // Remove ball and revolute joint constraints from DART
-    GetDARTWorld()->getConstraintSolver()->removeConstraint(
+    this->DARTWorld()->getConstraintSolver()->removeConstraint(
         this->dataPtr->dtWeldJointConst);
-    delete this->dataPtr->dtWeldJointConst;
-    this->dataPtr->dtWeldJointConst = NULL;
+    this->dataPtr->dtWeldJointConst.reset();
   }
 
   this->dataPtr->staticLink = _static;
@@ -804,10 +954,18 @@ void DARTLink::SetLinkStatic(bool _static)
 //////////////////////////////////////////////////
 void DARTLink::updateDirtyPoseFromDARTTransformation()
 {
+  if (!this->dataPtr->IsInitialized())
+  {
+    this->dataPtr->Cache(
+          "DirtyPoseFromDARTTransformation",
+          std::bind(&DARTLink::updateDirtyPoseFromDARTTransformation, this));
+    return;
+  }
+
   // Step 1: get dart body's transformation
   // Step 2: set gazebo link's pose using the transformation
-  math::Pose newPose = DARTTypes::ConvPose(
-                         this->dataPtr->dtBodyNode->getTransform());
+  ignition::math::Pose3d newPose = DARTTypes::ConvPoseIgn(
+      this->dataPtr->dtBodyNode->getTransform());
 
   // Set the new pose to this link
   this->dirtyPose = newPose;
@@ -821,13 +979,13 @@ void DARTLink::updateDirtyPoseFromDARTTransformation()
 DARTPhysicsPtr DARTLink::GetDARTPhysics(void) const
 {
   return boost::dynamic_pointer_cast<DARTPhysics>(
-        this->GetWorld()->GetPhysicsEngine());
+        this->GetWorld()->Physics());
 }
 
 //////////////////////////////////////////////////
-dart::simulation::World *DARTLink::GetDARTWorld(void) const
+dart::simulation::WorldPtr DARTLink::DARTWorld(void) const
 {
-  return GetDARTPhysics()->GetDARTWorld();
+  return this->GetDARTPhysics()->DARTWorld();
 }
 
 //////////////////////////////////////////////////
@@ -837,7 +995,52 @@ DARTModelPtr DARTLink::GetDARTModel() const
 }
 
 //////////////////////////////////////////////////
-dart::dynamics::BodyNode *DARTLink::GetDARTBodyNode() const
+DARTBodyNodePropPtr DARTLink::DARTProperties() const
+{
+  return this->dataPtr->dtProperties;
+}
+
+//////////////////////////////////////////////////
+void DARTLink::SetDARTBodyNode(dart::dynamics::BodyNode *_dtBodyNode)
+{
+  this->dataPtr->dtBodyNode = _dtBodyNode;
+}
+
+//////////////////////////////////////////////////
+void DARTLink::AddSlaveBodyNode(dart::dynamics::BodyNode *_dtBodyNode)
+{
+  dart::constraint::WeldJointConstraintPtr dtWeldJointConst;
+  dtWeldJointConst.reset(new dart::constraint::WeldJointConstraint(
+      this->dataPtr->dtBodyNode, _dtBodyNode));
+  this->DARTWorld()->getConstraintSolver()->addConstraint(dtWeldJointConst);
+  this->dataPtr->dtSlaveNodes.push_back(std::pair<dart::dynamics::BodyNode *,
+      dart::constraint::WeldJointConstraintPtr>(_dtBodyNode, dtWeldJointConst));
+  this->UpdateMass();
+}
+
+//////////////////////////////////////////////////
+bool DARTLink::RemoveSlaveBodyNode(dart::dynamics::BodyNode *_dtBodyNode)
+{
+  for (auto iter = this->dataPtr->dtSlaveNodes.begin();
+      iter != this->dataPtr->dtSlaveNodes.end(); ++iter)
+  {
+    if (iter->first == _dtBodyNode)
+    {
+      // Remove the weld constraint between the slave and master body nodes
+      this->DARTWorld()->getConstraintSolver()->removeConstraint(iter->second);
+      // Remove the slave body node from the skeleton
+      iter->first->remove();
+      // Redestribute the link's mass between the remaining body nodes
+      this->dataPtr->dtSlaveNodes.erase(iter);
+      this->UpdateMass();
+      return true;
+    }
+  }
+  return false;
+}
+
+//////////////////////////////////////////////////
+dart::dynamics::BodyNode *DARTLink::DARTBodyNode() const
 {
   return this->dataPtr->dtBodyNode;
 }
@@ -853,3 +1056,10 @@ void DARTLink::AddDARTChildJoint(DARTJointPtr _dartChildJoint)
 {
   this->dataPtr->dartChildJoints.push_back(_dartChildJoint);
 }
+
+//////////////////////////////////////////////////
+bool DARTLink::IsSoftBody() const
+{
+  return this->dataPtr->isSoftBody;
+}
+
