@@ -22,7 +22,13 @@
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
 #include <ignition/common/Profiler.hh>
+#include <ignition/math/Angle.hh>
 #include <ignition/math/Helpers.hh>
+#include <ignition/math/Matrix4.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Quaternion.hh>
+#include <ignition/math/Vector2.hh>
+#include <ignition/math/Vector3.hh>
 #include <sdf/sdf.hh>
 
 #ifndef _WIN32
@@ -38,6 +44,7 @@
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/VideoEncoder.hh"
+#include "gazebo/common/CommonIface.hh"
 
 #include "gazebo/rendering/ogre_gazebo.h"
 #include "gazebo/rendering/RTShaderSystem.hh"
@@ -95,7 +102,7 @@ Camera::Camera(const std::string &_name, ScenePtr _scene,
 
   this->sceneNode = NULL;
 
-  this->screenshotPath = getenv("HOME");
+  this->screenshotPath = getenv(HOMEDIR);
   this->screenshotPath += "/.gazebo/pictures";
 
   // Connect to the render signal
@@ -120,6 +127,8 @@ Camera::Camera(const std::string &_name, ScenePtr _scene,
   this->dataPtr->node->Init();
 
   this->dataPtr->antiAliasingValue = 4;
+
+  this->dataPtr->cameraIntrinsicMatrix = ignition::math::Matrix3d::Identity;
 }
 
 //////////////////////////////////////////////////
@@ -234,6 +243,9 @@ void Camera::UpdateCameraIntrinsics(
     _cameraIntrinsicsFx, _cameraIntrinsicsFy,
     _cameraIntrinsicsCx, _cameraIntrinsicsCy,
     _cameraIntrinsicsS, clipNear, clipFar);
+  this->dataPtr->cameraIntrinsicMatrix = this->BuildIntrinsicMatrix(
+             _cameraIntrinsicsFx, _cameraIntrinsicsFy,
+             _cameraIntrinsicsCx, _cameraIntrinsicsCy);
 
   if (this->camera != nullptr)
   {
@@ -328,6 +340,24 @@ ignition::math::Matrix4d Camera::BuildProjectionMatrix(
            _intrinsicsS, _clipNear, _clipFar);
 }
 
+//////////////////////////////////////////////////
+ignition::math::Matrix3d Camera::BuildIntrinsicMatrix(
+    const double _intrinsicsFx, const double _intrinsicsFy,
+    const double _intrinsicsCx, double _intrinsicsCy)
+{
+  return ignition::math::Matrix3d(
+          _intrinsicsFx,
+          0,
+          _intrinsicsCx,
+          0,
+          _intrinsicsFy,
+          _intrinsicsCy,
+          0,
+          0,
+          1
+  );
+}
+
 //////////////////////////////////////////////////
 void Camera::Init()
 {
@@ -813,11 +843,8 @@ void Camera::SetClipDist()
 
   if (this->camera)
   {
-    if (!this->cameraUsingIntrinsics)
-    {
-      this->camera->setNearClipDistance(clipElem->Get<double>("near"));
-      this->camera->setFarClipDistance(clipElem->Get<double>("far"));
-    }
+    this->camera->setNearClipDistance(clipElem->Get<double>("near"));
+    this->camera->setFarClipDistance(clipElem->Get<double>("far"));
     this->camera->setRenderingDistance(clipElem->Get<double>("far"));
   }
   else
@@ -1094,6 +1121,30 @@ double Camera::FarClip() const
     return 0;
 }
 
+//////////////////////////////////////////////////
+double Camera::ImageFocalLengthX() const
+{
+  return this->dataPtr->cameraIntrinsicMatrix(0, 0);
+}
+
+//////////////////////////////////////////////////
+double Camera::ImageFocalLengthY() const
+{
+  return this->dataPtr->cameraIntrinsicMatrix(1, 1);
+}
+
+//////////////////////////////////////////////////
+double Camera::ImageOpticalCenterX() const
+{
+  return this->dataPtr->cameraIntrinsicMatrix(0, 2);
+}
+
+//////////////////////////////////////////////////
+double Camera::ImageOpticalCenterY() const
+{
+  return this->dataPtr->cameraIntrinsicMatrix(1, 2);
+}
+
 //////////////////////////////////////////////////
 unsigned int Camera::ViewportWidth() const
 {
@@ -2037,6 +2088,7 @@ void Camera::UpdateFOV()
     {
       this->camera->setAspectRatio(ratio);
       this->camera->setFOVy(Ogre::Radian(this->LimitFOV(vfov)));
+      this->CalculateIntrinsicsFromProjectionMatrix();
     }
   }
 }
@@ -2135,6 +2187,23 @@ ignition::math::Matrix4d Camera::ProjectionMatrix() const
   return Conversions::ConvertIgn(this->camera->getProjectionMatrix());
 }
 
+///////////////////////////////////////////////
+void Camera::CalculateIntrinsicsFromProjectionMatrix()
+{
+  const ignition::math::Matrix4d& projectionMat = this->ProjectionMatrix();
+
+  const double width = this->imageWidth;
+  const double height = this->imageHeight;
+
+  double fX = (projectionMat(0, 0) * width) / 2;
+  double fY = (projectionMat(1, 1) * height) / 2;
+  double cX = -(width * (projectionMat(0, 2) - 1.0)) / 2;
+  double cY = height + (height * (projectionMat(1, 2) - 1.0)) / 2;
+
+  this->dataPtr->cameraIntrinsicMatrix = this->BuildIntrinsicMatrix(
+      fX, fY, cX, cY);
+}
+
 //////////////////////////////////////////////////
 event::ConnectionPtr Camera::ConnectNewImageFrame(
     std::function<void (const unsigned char *, unsigned int, unsigned int,
