@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Open Source Robotics Foundation
+ * Copyright (C) 2014 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,13 @@
 
 #include <sstream>
 #include <string>
+#include <ignition/common/Profiler.hh>
+#include <ignition/math/Angle.hh>
+#include <ignition/math/Matrix4.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Vector2.hh>
+#include <ignition/math/Vector3.hh>
+#include <ignition/math/Vector4.hh>
 
 #include "gazebo/rendering/ogre_gazebo.h"
 
@@ -123,6 +130,15 @@ OculusCamera::OculusCamera(const std::string &_name, ScenePtr _scene)
 
   this->dataPtr->controlSub = this->dataPtr->node->Subscribe("~/world_control",
                                            &OculusCamera::OnControl, this);
+  {
+    // Also subscribe to WorldControl messages over ZeroMQ-based gz-transport
+    std::string worldControlTopic("/world_control");
+    if (!this->dataPtr->ignNode.Subscribe(worldControlTopic,
+                                          &OculusCamera::OnWorldControl, this))
+    {
+      gzerr << "Error advertising topic [" << worldControlTopic << "]\n";
+    }
+  }
 
   // Oculus is now ready.
   this->dataPtr->ready = true;
@@ -155,7 +171,13 @@ void OculusCamera::Load(sdf::ElementPtr _sdf)
 //////////////////////////////////////////////////
 void OculusCamera::OnControl(ConstWorldControlPtr &_data)
 {
-  if (_data->has_reset() && _data->reset().has_all() && _data->reset().all())
+  this->OnWorldControl(*_data);
+}
+
+//////////////////////////////////////////////////
+void OculusCamera::OnWorldControl(const msgs::WorldControl &_data)
+{
+  if (_data.has_reset() && _data.reset().has_all() && _data.reset().all())
   {
     this->ResetSensor();
   }
@@ -234,6 +256,7 @@ void OculusCamera::RenderImpl()
 //////////////////////////////////////////////////
 void OculusCamera::Update()
 {
+  IGN_PROFILE("rendering::OculusCamera::Update");
   if (!this->Ready())
     return;
 
@@ -305,15 +328,15 @@ bool OculusCamera::AttachToVisualImpl(VisualPtr _visual,
   Camera::AttachToVisualImpl(_visual, _inheritOrientation);
   if (_visual)
   {
-    math::Pose origPose = this->WorldPose();
-    double yaw = _visual->GetWorldPose().rot.GetAsEuler().z;
+    ignition::math::Pose3d origPose = this->WorldPose();
+    double yaw = _visual->WorldPose().Rot().Euler().Z();
 
-    double zDiff = origPose.pos.z - _visual->GetWorldPose().pos.z;
+    double zDiff = origPose.Pos().Z() - _visual->WorldPose().Pos().Z();
     double pitch = 0;
 
     if (fabs(zDiff) > 1e-3)
     {
-      double dist = _visual->GetWorldPose().pos.Distance(
+      double dist = _visual->WorldPose().Pos().Distance(
           this->WorldPose().Pos());
       pitch = acos(zDiff/dist);
     }
@@ -321,9 +344,9 @@ bool OculusCamera::AttachToVisualImpl(VisualPtr _visual,
     this->Yaw(ignition::math::Angle(yaw));
     this->Pitch(ignition::math::Angle(pitch));
 
-    math::Box bb = _visual->GetBoundingBox();
-    math::Vector3 pos = bb.GetCenter();
-    pos.z = bb.max.z;
+    auto bb = _visual->BoundingBox();
+    auto pos = bb.Center();
+    pos.Z(bb.Max().Z());
   }
 
   return true;
@@ -362,12 +385,6 @@ void OculusCamera::Resize(unsigned int /*_w*/, unsigned int /*_h*/)
   }
 }
 
-//////////////////////////////////////////////////
-bool OculusCamera::MoveToPosition(const math::Pose &_pose, double _time)
-{
-  return Camera::MoveToPosition(_pose.Ign(), _time);
-}
-
 //////////////////////////////////////////////////
 void OculusCamera::MoveToVisual(const std::string &_name)
 {
@@ -389,58 +406,67 @@ void OculusCamera::MoveToVisual(VisualPtr _visual)
     this->scene->OgreSceneManager()->destroyAnimation("cameratrack");
   }
 
-  math::Box box = _visual->GetBoundingBox();
-  math::Vector3 size = box.GetSize();
-  double maxSize = std::max(std::max(size.x, size.y), size.z);
-
-  math::Vector3 start = this->WorldPose().Pos();
+  // Start from current position
+  ignition::math::Vector3d start = this->WorldPose().Pos();
   start.Correct();
-  math::Vector3 end = box.GetCenter() + _visual->GetWorldPose().pos;
-  end.Correct();
-  math::Vector3 dir = end - start;
+
+  // Center of visual
+  auto box = _visual->BoundingBox();
+  ignition::math::Vector3d visCenter = box.Center() +
+    _visual->WorldPose().Pos();
+  visCenter.Correct();
+
+  // Direction from start to visual center
+  ignition::math::Vector3d dir = visCenter - start;
   dir.Correct();
   dir.Normalize();
 
-  double dist = start.Distance(end) - maxSize;
+  // Distance to move
+  ignition::math::Vector3d size = box.Size();
+  double maxSize = size.Max();
+  double dist = start.Distance(visCenter) - maxSize;
 
-  math::Vector3 mid = start + dir*(dist*.5);
-  mid.z = box.GetCenter().z + box.GetSize().z + 2.0;
+  // Find midway point and change its Z
+  ignition::math::Vector3d mid = start + dir*(dist*.5);
+  mid.Z(box.Center().Z() + box.Size().Z() + 2.0);
 
-  dir = end - mid;
+  // Direction from mid to visual center
+  dir = visCenter - mid;
   dir.Correct();
-
-  dist = mid.Distance(end) - maxSize;
-
-  double yawAngle = atan2(dir.y, dir.x);
-  double pitchAngle = atan2(-dir.z, sqrt(dir.x*dir.x + dir.y*dir.y));
-  Ogre::Quaternion yawFinal(Ogre::Radian(yawAngle), Ogre::Vector3(0, 0, 1));
-  Ogre::Quaternion pitchFinal(Ogre::Radian(pitchAngle), Ogre::Vector3(0, 1, 0));
-
   dir.Normalize();
 
+  // Get new distance
+  dist = mid.Distance(visCenter) - maxSize;
+
+  // Scale to fit in view
   double scale = maxSize / tan((this->HFOV()/2.0).Radian());
 
-  end = mid + dir*(dist - scale);
+  // End position
+  auto end = mid + dir*(dist - scale);
 
-  // dist = start.Distance(end);
-  // double vel = 5.0;
-  double time = 0.5;  // dist / vel;
+  // Orientation
+  auto mat = ignition::math::Matrix4d::LookAt(end, visCenter);
 
+  // Time
+  double time = 0.5;
+
+  // Ogre animation
   Ogre::Animation *anim =
     this->scene->OgreSceneManager()->createAnimation("cameratrack", time);
   anim->setInterpolationMode(Ogre::Animation::IM_SPLINE);
 
   Ogre::NodeAnimationTrack *strack = anim->createNodeTrack(0, this->sceneNode);
 
+  // Start keyframe
   Ogre::TransformKeyFrame *key;
-
   key = strack->createNodeKeyFrame(0);
-  key->setTranslate(Ogre::Vector3(start.x, start.y, start.z));
+  key->setTranslate(Conversions::Convert(start));
   key->setRotation(this->sceneNode->getOrientation());
 
+  // End keyframe
   key = strack->createNodeKeyFrame(time);
-  key->setTranslate(Ogre::Vector3(end.x, end.y, end.z));
-  key->setRotation(yawFinal);
+  key->setTranslate(Conversions::Convert(end));
+  key->setRotation(Conversions::Convert(mat.Rotation()));
 
   this->animState =
     this->scene->OgreSceneManager()->createAnimationState("cameratrack");
