@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,24 +14,14 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <string>
 #include <iostream>
 #include <functional>
 #include <boost/filesystem.hpp>
 #include <sys/types.h>
 
-#ifdef __APPLE__
-# include <QtCore/qglobal.h>
-#endif
-
 // Not Apple or Windows
-#if not defined( Q_OS_MAC) && not defined(_WIN32)
+#if not defined(__APPLE__) && not defined(_WIN32)
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <GL/glx.h>
@@ -45,6 +35,8 @@
 
 #include "gazebo/gazebo_config.h"
 
+#include <ignition/common/Profiler.hh>
+
 #include "gazebo/common/CommonIface.hh"
 #include "gazebo/common/Events.hh"
 #include "gazebo/common/Exception.hh"
@@ -118,7 +110,9 @@ void RenderEngine::Load()
     // Make the root
     try
     {
-      this->dataPtr->root = new Ogre::Root();
+      // empty strings for config filenames (plugins.cfg and ogre.cfg)
+      // so ogre doesn't try to look for them.
+      this->dataPtr->root = new Ogre::Root("", "");
     }
     catch(Ogre::Exception &e)
     {
@@ -260,12 +254,6 @@ ScenePtr RenderEngine::GetScene(unsigned int index)
   }
 }
 
-//////////////////////////////////////////////////
-unsigned int RenderEngine::GetSceneCount() const
-{
-  return this->SceneCount();
-}
-
 //////////////////////////////////////////////////
 unsigned int RenderEngine::SceneCount() const
 {
@@ -275,17 +263,20 @@ unsigned int RenderEngine::SceneCount() const
 //////////////////////////////////////////////////
 void RenderEngine::PreRender()
 {
+  IGN_PROFILE("rendering::RenderEngine::PreRender");
   this->dataPtr->root->_fireFrameStarted();
 }
 
 //////////////////////////////////////////////////
 void RenderEngine::Render()
 {
+  IGN_PROFILE("rendering::RenderEngine::Render");
 }
 
 //////////////////////////////////////////////////
 void RenderEngine::PostRender()
 {
+  IGN_PROFILE("rendering::RenderEngine::PostRender");
   // _fireFrameRenderingQueued was here for CEGUI to work. Leaving because
   // it shouldn't harm anything, and we don't want to introduce
   // a regression.
@@ -299,7 +290,7 @@ void RenderEngine::Init()
   if (this->dataPtr->renderPathType == NONE)
   {
     gzwarn << "Cannot initialize render engine since "
-           << "render path type is NONE. Ignore this warning if"
+           << "render path type is NONE. Ignore this warning if "
            << "rendering has been turned off on purpose.\n";
     return;
   }
@@ -333,15 +324,16 @@ void RenderEngine::Init()
 //////////////////////////////////////////////////
 void RenderEngine::Fini()
 {
+  // TODO: this was causing a segfault on shutdown
+  // Windows are created on load so clear them even
+  // if render engine is not initialized
+  this->dataPtr->windowManager->Fini();
+
   if (!this->dataPtr->initialized)
     return;
 
   this->dataPtr->connections.clear();
 
-  // TODO: this was causing a segfault on shutdown
-  // Close all the windows first;
-  this->dataPtr->windowManager->Fini();
-
   RTShaderSystem::Instance()->Fini();
 
   // Deallocate memory for every scene
@@ -358,7 +350,6 @@ void RenderEngine::Fini()
   // TODO: this was causing a segfault. Need to debug, and put back in
   if (this->dataPtr->root)
   {
-    this->dataPtr->root->shutdown();
     /*const Ogre::Root::PluginInstanceList ll =
      this->dataPtr->root->getInstalledPlugins();
 
@@ -388,7 +379,7 @@ void RenderEngine::Fini()
   this->dataPtr->scenes.clear();
 
   // Not Apple or Windows
-# if not defined( Q_OS_MAC) && not defined(_WIN32)
+# if not defined(__APPLE__) && not defined(_WIN32)
   if (this->dummyDisplay)
   {
     glXDestroyContext(static_cast<Display*>(this->dummyDisplay),
@@ -426,17 +417,23 @@ void RenderEngine::LoadPlugins()
     std::vector<std::string>::iterator piter;
 
 #ifdef __APPLE__
-    std::string prefix = "lib";
     std::string extension = ".dylib";
+#elif defined(_WIN32)
+    std::string extension = ".dll";
 #else
-    std::string prefix = "";
     std::string extension = ".so";
 #endif
 
-    plugins.push_back(path+"/"+prefix+"RenderSystem_GL");
-    plugins.push_back(path+"/"+prefix+"Plugin_ParticleFX");
-    plugins.push_back(path+"/"+prefix+"Plugin_BSPSceneManager");
-    plugins.push_back(path+"/"+prefix+"Plugin_OctreeSceneManager");
+    plugins.push_back(path+"/RenderSystem_GL");
+    plugins.push_back(path+"/Plugin_ParticleFX");
+    plugins.push_back(path+"/Plugin_BSPSceneManager");
+    plugins.push_back(path+"/Plugin_OctreeSceneManager");
+
+#if OGRE_VERSION_MAJOR == 1 && OGRE_VERSION_MINOR >= 11
+    // OGRE 1.11 moved FreeImage codec support to a plugin
+    // See: https://github.com/OGRECave/ogre/blob/master/Docs/1.11-Notes.md
+    plugins.push_back(path + "/Codec_FreeImage");
+#endif
 
 #ifdef HAVE_OCULUS
     plugins.push_back(path+"/Plugin_CgProgramManager");
@@ -461,7 +458,7 @@ void RenderEngine::LoadPlugins()
           if ((*piter).find("RenderSystem") != std::string::npos)
           {
             gzerr << "Unable to load Ogre Plugin[" << *piter
-                  << "]. Rendering will not be possible."
+                  << "]. Rendering will not be possible. "
                   << "Make sure you have installed OGRE and Gazebo properly.\n";
           }
         }
@@ -473,7 +470,7 @@ void RenderEngine::LoadPlugins()
 /////////////////////////////////////////////////
 void RenderEngine::AddResourcePath(const std::string &_uri)
 {
-  if (_uri == "__default__" || _uri.empty())
+  if (_uri.find("__default__") != std::string::npos || _uri.empty())
     return;
 
   std::string path = common::find_file_path(_uri);
@@ -486,6 +483,7 @@ void RenderEngine::AddResourcePath(const std::string &_uri)
 
   try
   {
+    path = boost::filesystem::path(path).make_preferred().string();
     if (!Ogre::ResourceGroupManager::getSingleton().resourceLocationExists(
           path, "General"))
     {
@@ -513,6 +511,7 @@ void RenderEngine::AddResourcePath(const std::string &_uri)
           if (dIter->filename().extension() == ".material")
           {
             boost::filesystem::path fullPath = path / dIter->filename();
+            fullPath = fullPath.make_preferred();
 
             Ogre::DataStreamPtr stream =
               Ogre::ResourceGroupManager::getSingleton().openResource(
@@ -546,7 +545,7 @@ void RenderEngine::AddResourcePath(const std::string &_uri)
   }
   catch(Ogre::Exception &/*_e*/)
   {
-    gzthrow("Unable to load Ogre Resources.\nMake sure the"
+    gzthrow("Unable to load Ogre Resources.\nMake sure the "
         "resources path in the world file is set correctly.");
   }
 }
@@ -568,7 +567,6 @@ void RenderEngine::SetupResources()
 
   std::list<std::string> mediaDirs;
   mediaDirs.push_back("media");
-  mediaDirs.push_back("Media");
 
   for (iter = paths.begin(); iter != paths.end(); ++iter)
   {
@@ -591,10 +589,16 @@ void RenderEngine::SetupResources()
           std::make_pair(prefix, "General"));
       archNames.push_back(
           std::make_pair(prefix + "/skyx", "SkyX"));
+#if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0) && !defined(__APPLE__))
+      archNames.push_back(
+          std::make_pair(prefix + "/rtshaderlib150", "General"));
+#endif
       archNames.push_back(
           std::make_pair(prefix + "/rtshaderlib", "General"));
+#if (OGRE_RESOURCEMANAGER_STRICT == 0)
       archNames.push_back(
           std::make_pair(prefix + "/materials/programs", "General"));
+#endif
       archNames.push_back(
           std::make_pair(prefix + "/materials/scripts", "General"));
       archNames.push_back(
@@ -615,20 +619,35 @@ void RenderEngine::SetupResources()
           std::make_pair(prefix + "/gui/layouts", "Layouts"));
       archNames.push_back(
           std::make_pair(prefix + "/gui/animations", "Animations"));
-    }
-
-    for (aiter = archNames.begin(); aiter != archNames.end(); ++aiter)
+#if (OGRE_RESOURCEMANAGER_STRICT != 0)
+    try
     {
-      try
-      {
-        Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
-            aiter->first, "FileSystem", aiter->second);
-      }
-      catch(Ogre::Exception &/*_e*/)
-      {
-        gzthrow("Unable to load Ogre Resources. Make sure the resources path "
-            "in the world file is set correctly.");
-      }
+      Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
+          boost::filesystem::path(
+              prefix + "/materials/programs").make_preferred().string(),
+          "FileSystem", "General", true);
+    }
+    catch(Ogre::Exception &/*_e*/)
+    {
+      gzerr << "Unable to load Ogre Resources. Make sure the resources path "
+            << "in the world file is set correctly.";
+    }
+#endif
+    }
+  }
+
+  for (aiter = archNames.begin(); aiter != archNames.end(); ++aiter)
+  {
+    try
+    {
+      Ogre::ResourceGroupManager::getSingleton().addResourceLocation(
+          boost::filesystem::path(aiter->first).make_preferred().string(),
+          "FileSystem", aiter->second);
+    }
+    catch(Ogre::Exception &/*_e*/)
+    {
+      gzthrow("Unable to load Ogre Resources. Make sure the resources path "
+          "in the world file is set correctly.");
     }
   }
 }
@@ -719,7 +738,7 @@ bool RenderEngine::CreateContext()
 {
   bool result = true;
 
-#if defined Q_OS_MAC || _WIN32
+#if defined __APPLE__ || _WIN32
   this->dummyDisplay = 0;
 #else
   try
@@ -781,6 +800,12 @@ void RenderEngine::CheckSystemCapabilities()
   Ogre::RenderSystemCapabilities::ShaderProfiles::const_iterator iter;
 
   capabilities = this->dataPtr->root->getRenderSystem()->getCapabilities();
+  if (!capabilities)
+  {
+    gzerr << "Cannot get render system capabilities" << std::endl;
+    return;
+  }
+
   profiles = capabilities->getSupportedShaderProfiles();
 
   bool hasFragmentPrograms =
@@ -798,7 +823,13 @@ void RenderEngine::CheckSystemCapabilities()
   // int multiRenderTargetCount = capabilities->getNumMultiRenderTargets();
 
   bool hasFBO =
+#if OGRE_VERSION_MAJOR == 1 && OGRE_VERSION_MINOR >= 11
+    // All APIs targeted by OGRE supported this capability,
+    // see https://ogrecave.github.io/ogre/api/1.10/group___render_system.html#gga3d2965b7f378ebdcfe8a4a6cf74c3de7a8a0ececdc95122ac3063fc4f27d6402c
+    true;
+#else
     capabilities->hasCapability(Ogre::RSC_FBO);
+#endif
 
   bool hasGLSL =
     std::find(profiles.begin(), profiles.end(), "glsl") != profiles.end();
@@ -808,7 +839,7 @@ void RenderEngine::CheckSystemCapabilities()
            << "Fixed function rendering will be used.\n";
 
   if (!hasGLSL)
-    gzwarn << "GLSL is missing."
+    gzwarn << "GLSL is missing. "
            << "Fixed function rendering will be used.\n";
 
   if (!hasFBO)
@@ -846,15 +877,22 @@ std::vector<unsigned int> RenderEngine::FSAALevels() const
 }
 
 #if (OGRE_VERSION >= ((1 << 16) | (9 << 8) | 0))
-/////////////////////////////////////////////////
-Ogre::OverlaySystem *RenderEngine::GetOverlaySystem() const
-{
-  return this->OverlaySystem();
-}
-
 /////////////////////////////////////////////////
 Ogre::OverlaySystem *RenderEngine::OverlaySystem() const
 {
   return this->dataPtr->overlaySystem;
 }
 #endif
+
+//////////////////////////////////////////////////
+RenderEngine* RenderEngine::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<RenderEngine>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
