@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,21 +14,21 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
+#include "ignition/common/Profiler.hh"
 
 #include "gazebo/transport/transport.hh"
 
 #include "gazebo/physics/PhysicsIface.hh"
+#include "gazebo/physics/Link.hh"
+#include "gazebo/physics/Joint.hh"
 #include "gazebo/physics/World.hh"
+#include "gazebo/physics/PhysicsEngine.hh"
 
 #include "gazebo/common/Timer.hh"
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Plugin.hh"
+#include "gazebo/common/SdfFrameSemantics.hh"
 
 #include "gazebo/rendering/Camera.hh"
 #include "gazebo/rendering/Distortion.hh"
@@ -47,6 +47,8 @@ using namespace sensors;
 
 sdf::ElementPtr SensorPrivate::sdfSensor;
 
+bool Sensor::useStrictRate = false;
+
 //////////////////////////////////////////////////
 Sensor::Sensor(SensorCategory _cat)
 : dataPtr(new SensorPrivate)
@@ -87,24 +89,70 @@ void Sensor::Load(const std::string &_worldName, sdf::ElementPtr _sdf)
 //////////////////////////////////////////////////
 void Sensor::Load(const std::string &_worldName)
 {
+  this->world = physics::get_world(_worldName);
+
   if (this->sdf->HasElement("pose"))
   {
     this->pose =
       this->sdf->Get<ignition::math::Pose3d>("pose");
+
+    std::string relativeTo =
+      this->sdf->GetElement("pose")->Get<std::string>("relative_to", "").first;
+
+    // Handle frame semantics if relativeTo has been set
+    if (!relativeTo.empty())
+    {
+      auto parentLink = boost::dynamic_pointer_cast<physics::Link>(
+          this->world->EntityByName(this->ParentName()));
+      auto parentJoint = boost::dynamic_pointer_cast<physics::Joint>(
+          this->world->BaseByName(this->ParentName()));
+
+      if (nullptr != parentLink)
+      {
+        auto linkSDFDom = parentLink->GetSDFDom();
+        if (nullptr != linkSDFDom )
+        {
+          auto *sensorSDFDom = linkSDFDom->SensorByName(this->Name());
+          if (nullptr != sensorSDFDom)
+          {
+            this->pose = common::resolveSdfPose(sensorSDFDom->SemanticPose());
+          }
+        }
+      }
+      else if (nullptr != parentJoint)
+      {
+        // sdf::Joint doesn't have API for accessing joint sensors. For now,
+        // manually resolve the pose
+        auto jointSDFDom = parentJoint->GetSDFDom();
+        if (nullptr != jointSDFDom)
+        {
+          // Suppose the sensor's pose is given in frame A, denoted by X_SA. To
+          // find the sensor's pose relative to the joint, X_SJ, we first find
+          // the joint's pose relative to frame A, X_JA and compute the product
+          // X_SJ = X_JA^-1 X_SA
+          auto jointPoseInA =
+            common::resolveSdfPose(jointSDFDom->SemanticPose(), relativeTo);
+          this->pose = jointPoseInA.Inverse() *
+            this->sdf->Get<ignition::math::Pose3d>("pose");
+        }
+      }
+    }
   }
 
   if (this->sdf->Get<bool>("always_on"))
     this->SetActive(true);
 
-  this->world = physics::get_world(_worldName);
+  this->useStrictRate = rendering::lockstep_enabled();
 
   if (this->dataPtr->category == IMAGE)
+  {
     this->scene = rendering::get_scene(_worldName);
+  }
 
   // loaded, but not updated
   this->lastUpdateTime = common::Time(0.0);
 
-  this->node->Init(this->world->GetName());
+  this->node->Init(this->world->Name());
   this->dataPtr->sensorPub =
     this->node->Advertise<msgs::Sensor>("~/sensor");
 }
@@ -137,36 +185,18 @@ void Sensor::SetParent(const std::string &_name, const uint32_t _id)
   this->parentId = _id;
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetParentName() const
-{
-  return this->ParentName();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::ParentName() const
 {
   return this->parentName;
 }
 
-//////////////////////////////////////////////////
-uint32_t Sensor::GetId() const
-{
-  return this->Id();
-}
-
 //////////////////////////////////////////////////
 uint32_t Sensor::Id() const
 {
   return this->dataPtr->id;
 }
 
-//////////////////////////////////////////////////
-uint32_t Sensor::GetParentId() const
-{
-  return this->ParentId();
-}
-
 //////////////////////////////////////////////////
 uint32_t Sensor::ParentId() const
 {
@@ -183,7 +213,7 @@ bool Sensor::NeedsUpdate()
   if (this->dataPtr->category == IMAGE && this->scene)
     simTime = this->scene->SimTime();
   else
-    simTime = this->world->GetSimTime();
+    simTime = this->world->SimTime();
 
   // case when last update occurred in the future probably due to
   // world reset
@@ -206,44 +236,62 @@ void Sensor::Update(const bool _force)
 {
   if (this->IsActive() || _force)
   {
-    common::Time simTime;
-    if (this->dataPtr->category == IMAGE && this->scene)
-      simTime = this->scene->SimTime();
-    else
-      simTime = this->world->GetSimTime();
-
+    if (this->useStrictRate)
     {
-      std::lock_guard<std::mutex> lock(this->dataPtr->mutexLastUpdateTime);
-
-      if (simTime <= this->lastUpdateTime && !_force)
+      // rendering sensors (IMAGE category) has its own mechanism
+      // for throttling and lockstepping with physics. So throttle just
+      // physics sensors
+      if (this->dataPtr->category != IMAGE && !this->NeedsUpdate() && !_force)
         return;
 
-      // Adjust time-to-update period to compensate for delays caused by another
-      // sensor's update in the same thread.
-      // NOTE: If you change this equation, also change the matching equation in
-      // Sensor::NeedsUpdate
-      common::Time adjustedElapsed = simTime -
-        this->lastUpdateTime + this->dataPtr->updateDelay;
-
-      if (adjustedElapsed < this->updatePeriod && !_force)
-        return;
-
-      this->dataPtr->updateDelay = std::max(common::Time::Zero,
-          adjustedElapsed - this->updatePeriod);
-
-      // if delay is more than a full update period, then give up trying
-      // to catch up. This happens normally when the sensor just changed from
-      // an inactive to an active state, or the sensor just cannot hit its
-      // target update rate (worst case).
-      if (this->dataPtr->updateDelay >= this->updatePeriod)
-        this->dataPtr->updateDelay = common::Time::Zero;
+      if (this->UpdateImpl(_force))
+      {
+        std::lock_guard<std::mutex> lock(this->dataPtr->mutexLastUpdateTime);
+        this->lastUpdateTime = this->world->SimTime();;
+        this->updated();
+      }
     }
-
-    if (this->UpdateImpl(_force))
+    else
     {
-      std::lock_guard<std::mutex> lock(this->dataPtr->mutexLastUpdateTime);
-      this->lastUpdateTime = simTime;
-      this->dataPtr->updated();
+      common::Time simTime;
+      if (this->dataPtr->category == IMAGE && this->scene)
+        simTime = this->scene->SimTime();
+      else
+        simTime = this->world->SimTime();
+
+      {
+        std::lock_guard<std::mutex> lock(this->dataPtr->mutexLastUpdateTime);
+
+        if (simTime <= this->lastUpdateTime && !_force)
+          return;
+
+        // Adjust time-to-update period to compensate for delays caused by
+        // another sensor's update in the same thread.
+        // NOTE: If you change this equation, also change the matching equation
+        // in Sensor::NeedsUpdate
+        common::Time adjustedElapsed = simTime -
+          this->lastUpdateTime + this->dataPtr->updateDelay;
+
+        if (adjustedElapsed < this->updatePeriod && !_force)
+          return;
+
+        this->dataPtr->updateDelay = std::max(common::Time::Zero,
+            adjustedElapsed - this->updatePeriod);
+
+        // if delay is more than a full update period, then give up trying
+        // to catch up. This happens normally when the sensor just changed from
+        // an inactive to an active state, or the sensor just cannot hit its
+        // target update rate (worst case).
+        if (this->dataPtr->updateDelay >= this->updatePeriod)
+          this->dataPtr->updateDelay = common::Time::Zero;
+      }
+
+      if (this->UpdateImpl(_force))
+      {
+        std::lock_guard<std::mutex> lock(this->dataPtr->mutexLastUpdateTime);
+        this->lastUpdateTime = simTime;
+        this->updated();
+      }
     }
   }
 }
@@ -272,12 +320,6 @@ void Sensor::Fini()
   this->world.reset();
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetName() const
-{
-  return this->Name();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::Name() const
 {
@@ -288,17 +330,10 @@ std::string Sensor::Name() const
   return "";
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetScopedName() const
-{
-  return this->ScopedName();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::ScopedName() const
 {
-  return this->world->GetName() + "::" +
-         this->parentName + "::" + this->Name();
+  return this->world->Name() + "::" + this->parentName + "::" + this->Name();
 }
 
 //////////////////////////////////////////////////
@@ -363,12 +398,6 @@ void Sensor::SetPose(const ignition::math::Pose3d &_pose)
   }
 }
 
-//////////////////////////////////////////////////
-double Sensor::GetUpdateRate()
-{
-  return this->UpdateRate();
-}
-
 //////////////////////////////////////////////////
 double Sensor::UpdateRate() const
 {
@@ -387,60 +416,30 @@ void Sensor::SetUpdateRate(const double _hz)
     this->updatePeriod = 0.0;
 }
 
-//////////////////////////////////////////////////
-common::Time Sensor::GetLastUpdateTime()
-{
-  return this->LastUpdateTime();
-}
-
 //////////////////////////////////////////////////
 common::Time Sensor::LastUpdateTime() const
 {
   return this->lastUpdateTime;
 }
 
-//////////////////////////////////////////////////
-common::Time Sensor::GetLastMeasurementTime()
-{
-  return this->LastMeasurementTime();
-}
-
 //////////////////////////////////////////////////
 common::Time Sensor::LastMeasurementTime() const
 {
   return this->lastMeasurementTime;
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetType() const
-{
-  return this->Type();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::Type() const
 {
   return this->sdf->Get<std::string>("type");
 }
 
-//////////////////////////////////////////////////
-bool Sensor::GetVisualize() const
-{
-  return this->Visualize();
-}
-
 //////////////////////////////////////////////////
 bool Sensor::Visualize() const
 {
   return this->sdf->Get<bool>("visualize");
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetTopic() const
-{
-  return this->Topic();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::Topic() const
 {
@@ -490,33 +489,21 @@ void Sensor::FillMsg(msgs::Sensor &_msg)
     if (distortion)
     {
       msgs::Distortion *distortionMsg = camMsg->mutable_distortion();
-      distortionMsg->set_k1(distortion->GetK1());
-      distortionMsg->set_k2(distortion->GetK2());
-      distortionMsg->set_k3(distortion->GetK3());
-      distortionMsg->set_p1(distortion->GetP1());
-      distortionMsg->set_p2(distortion->GetP2());
-      distortionMsg->mutable_center()->set_x(distortion->GetCenter().x);
-      distortionMsg->mutable_center()->set_y(distortion->GetCenter().y);
+      distortionMsg->set_k1(distortion->K1());
+      distortionMsg->set_k2(distortion->K2());
+      distortionMsg->set_k3(distortion->K3());
+      distortionMsg->set_p1(distortion->P1());
+      distortionMsg->set_p2(distortion->P2());
+      distortionMsg->mutable_center()->set_x(distortion->Center().X());
+      distortionMsg->mutable_center()->set_y(distortion->Center().Y());
     }
   }
 }
 
-//////////////////////////////////////////////////
-std::string Sensor::GetWorldName() const
-{
-  return this->WorldName();
-}
-
 //////////////////////////////////////////////////
 std::string Sensor::WorldName() const
 {
-  return this->world->GetName();
-}
-
-//////////////////////////////////////////////////
-SensorCategory Sensor::GetCategory() const
-{
-  return this->Category();
+  return this->world->Name();
 }
 
 //////////////////////////////////////////////////
@@ -525,12 +512,6 @@ SensorCategory Sensor::Category() const
   return this->dataPtr->category;
 }
 
-//////////////////////////////////////////////////
-NoisePtr Sensor::GetNoise(const SensorNoiseType _type) const
-{
-  return this->Noise(_type);
-}
-
 //////////////////////////////////////////////////
 NoisePtr Sensor::Noise(const SensorNoiseType _type) const
 {
@@ -554,11 +535,18 @@ void Sensor::ResetLastUpdateTime()
 //////////////////////////////////////////////////
 event::ConnectionPtr Sensor::ConnectUpdated(std::function<void()> _subscriber)
 {
-  return this->dataPtr->updated.Connect(_subscriber);
+  return this->updated.Connect(_subscriber);
 }
 
 //////////////////////////////////////////////////
-void Sensor::DisconnectUpdated(event::ConnectionPtr &_c)
+double Sensor::NextRequiredTimestamp() const
 {
-  this->dataPtr->updated.Disconnect(_c);
+  // implementation by default: next required timestamp is ignored
+  return std::numeric_limits<double>::quiet_NaN();
+}
+
+//////////////////////////////////////////////////
+bool Sensor::StrictRate() const
+{
+  return this->useStrictRate;
 }
