@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,13 +14,7 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 
 #include "gazebo/msgs/msgs.hh"
 #include "gazebo/common/Console.hh"
@@ -33,6 +27,7 @@
 using namespace gazebo;
 using namespace transport;
 
+#if TBB_VERSION_MAJOR < 2021
 /// TBB task to process nodes.
 class TopicManagerProcessTask : public tbb::task
 {
@@ -43,20 +38,30 @@ class TopicManagerProcessTask : public tbb::task
             return NULL;
           }
 };
+#endif
 
 /// TBB task to establish subscriber to publisher connection.
+#if TBB_VERSION_MAJOR < 2021
 class TopicManagerConnectionTask : public tbb::task
+#else
+class TopicManagerConnectionTask
+#endif
 {
   /// \brief Constructor.
   /// \param[in] _pub Publish message
-  public: TopicManagerConnectionTask(msgs::Publish _pub) : pub(_pub) {}
+  public: explicit TopicManagerConnectionTask(msgs::Publish _pub) : pub(_pub) {}
 
   /// Implements the necessary execute function
+#if TBB_VERSION_MAJOR < 2021
   public: tbb::task *execute()
           {
             TopicManager::Instance()->ConnectSubToPub(pub);
             return NULL;
           }
+#else
+  public: void operator()() const
+          { TopicManager::Instance()->ConnectSubToPub(pub); }
+#endif
 
   /// \brief Publish message
   private: msgs::Publish pub;
@@ -70,8 +75,6 @@ ConnectionManager::ConnectionManager()
   this->stop = false;
   this->stopped = true;
 
-  this->serverConn = NULL;
-
   this->eventConnections.push_back(
       event::Events::ConnectStop(boost::bind(&ConnectionManager::Stop, this)));
 }
@@ -79,10 +82,9 @@ ConnectionManager::ConnectionManager()
 //////////////////////////////////////////////////
 ConnectionManager::~ConnectionManager()
 {
+  boost::mutex::scoped_lock lock(this->updateMutex);
   this->eventConnections.clear();
 
-  delete this->serverConn;
-  this->serverConn = NULL;
   this->Fini();
 }
 
@@ -93,10 +95,10 @@ bool ConnectionManager::Init(const std::string &_masterHost,
 {
   this->stop = false;
   this->masterConn.reset(new Connection());
-  delete this->serverConn;
-  this->serverConn = new Connection();
+  this->serverConn.reset(new Connection());
 
   // Create a new TCP server on a free port
+  using namespace boost::placeholders;
   this->serverConn->Listen(0,
       boost::bind(&ConnectionManager::OnAccept, this, _1));
 
@@ -208,6 +210,12 @@ bool ConnectionManager::Init(const std::string &_masterHost,
   return true;
 }
 
+//////////////////////////////////////////////////
+bool ConnectionManager::IsInitialized() const
+{
+  return this->initialized;
+}
+
 //////////////////////////////////////////////////
 void ConnectionManager::Fini()
 {
@@ -215,6 +223,9 @@ void ConnectionManager::Fini()
     return;
 
   this->Stop();
+  if (this->initialized)
+    while (this->stopped == false)
+      common::Time::MSleep(100);
 
   if (this->masterConn)
   {
@@ -227,12 +238,14 @@ void ConnectionManager::Fini()
   {
     this->serverConn->ProcessWriteQueue();
     this->serverConn->Shutdown();
-    delete this->serverConn;
-    this->serverConn = NULL;
+    this->serverConn.reset();
   }
 
   this->eventConnections.clear();
-  this->connections.clear();
+  {
+    boost::recursive_mutex::scoped_lock lock(this->connectionMutex);
+    this->connections.clear();
+  }
   this->publishers.clear();
   this->namespaces.clear();
   this->masterMessages.clear();
@@ -244,10 +257,6 @@ void ConnectionManager::Fini()
 void ConnectionManager::Stop()
 {
   this->stop = true;
-  this->updateCondition.notify_all();
-  if (this->initialized)
-    while (this->stopped == false)
-      common::Time::MSleep(100);
 }
 
 //////////////////////////////////////////////////
@@ -275,17 +284,11 @@ void ConnectionManager::RunUpdate()
   if (this->masterConn)
     this->masterConn->ProcessWriteQueue();
 
-  // Use TBB to process nodes. Need more testing to see if this makes
-  // a difference.
-  // TopicManagerProcessTask *task = new(tbb::task::allocate_root())
-  //   TopicManagerProcessTask();
-  // tbb::task::enqueue(*task);
+  boost::recursive_mutex::scoped_lock lock(this->connectionMutex);
+
   TopicManager::Instance()->ProcessNodes();
-  {
-    boost::recursive_mutex::scoped_lock lock(this->connectionMutex);
-    iter = this->connections.begin();
-    endIter = this->connections.end();
-  }
+  iter = this->connections.begin();
+  endIter = this->connections.end();
 
   while (iter != endIter)
   {
@@ -296,7 +299,6 @@ void ConnectionManager::RunUpdate()
     }
     else
     {
-      boost::recursive_mutex::scoped_lock lock(this->connectionMutex);
       iter = this->connections.erase(iter);
     }
   }
@@ -331,6 +333,7 @@ bool ConnectionManager::IsRunning() const
 //////////////////////////////////////////////////
 void ConnectionManager::OnMasterRead(const std::string &_data)
 {
+  using namespace boost::placeholders;
   if (this->masterConn && this->masterConn->IsOpen())
     this->masterConn->AsyncRead(
         boost::bind(&ConnectionManager::OnMasterRead, this, _1));
@@ -406,9 +409,13 @@ void ConnectionManager::ProcessMessage(const std::string &_data)
     if (pub.host() != this->serverConn->GetLocalAddress() ||
         pub.port() != this->serverConn->GetLocalPort())
     {
+#if TBB_VERSION_MAJOR < 2021
       TopicManagerConnectionTask *task = new(tbb::task::allocate_root())
       TopicManagerConnectionTask(pub);
       tbb::task::enqueue(*task);
+#else
+      this->taskGroup.run<TopicManagerConnectionTask>(pub);
+#endif
     }
   }
   // publisher_subscribe. This occurs when we try to subscribe to a topic, and
@@ -453,6 +460,7 @@ void ConnectionManager::ProcessMessage(const std::string &_data)
 //////////////////////////////////////////////////
 void ConnectionManager::OnAccept(ConnectionPtr _newConnection)
 {
+  using namespace boost::placeholders;
   _newConnection->AsyncRead(
       boost::bind(&ConnectionManager::OnRead, this, _newConnection, _1));
 
@@ -468,6 +476,7 @@ void ConnectionManager::OnRead(ConnectionPtr _connection,
   if (_data.empty())
   {
     gzerr << "Data was empty, try again\n";
+    using namespace boost::placeholders;
     _connection->AsyncRead(
         boost::bind(&ConnectionManager::OnRead, this, _connection, _1));
     return;
@@ -703,3 +712,16 @@ void ConnectionManager::TriggerUpdate()
 {
   this->updateCondition.notify_all();
 }
+
+//////////////////////////////////////////////////
+ConnectionManager* ConnectionManager::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<ConnectionManager>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
