@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,10 +14,12 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
+#include <gazebo/gazebo_config.h>
+
+#ifndef USE_EXTERNAL_TINYXML2
+#include <gazebo/tinyxml2.h>
+#else
+#include <tinyxml2.h>
 #endif
 
 #include <algorithm>
@@ -44,7 +46,7 @@
 #include "gazebo/util/LogPlay.hh"
 
 using namespace gazebo;
-using namespace util;
+using namespace gazebo::util;
 
 /////////////////////////////////////////////////
 LogPlay::LogPlay()
@@ -118,8 +120,13 @@ void LogPlay::Open(const std::string &_logFile)
   {
     gzerr << "Unable to load file[" << _logFile << "]. "
       << "Check the Gazebo server log file for more information.\n";
+#ifdef TINYXML2_MAJOR_VERSION_GE_6
+    const char *errorStr1 = this->dataPtr->xmlDoc.ErrorStr();
+    const char *errorStr2 = nullptr;
+#else
     const char *errorStr1 = this->dataPtr->xmlDoc.GetErrorStr1();
     const char *errorStr2 = this->dataPtr->xmlDoc.GetErrorStr2();
+#endif
     if (errorStr1)
       gzlog << "Log Error 1:\n" << errorStr1 << std::endl;
     if (errorStr2)
@@ -155,19 +162,16 @@ void LogPlay::Open(const std::string &_logFile)
   if (!this->dataPtr->logCurrXml)
     gzthrow("Unable to find the first chunk");
 
-  if (!this->ChunkData(this->dataPtr->logCurrXml, this->dataPtr->currentChunk))
+  if (!this->dataPtr->ChunkData(this->dataPtr->logCurrXml,
+                                this->dataPtr->currentChunk))
+  {
     gzthrow("Unable to decode log file");
+  }
 
   this->dataPtr->start = 0;
   this->dataPtr->end = -1 * this->dataPtr->kEndFrame.size();
 }
 
-/////////////////////////////////////////////////
-std::string LogPlay::GetHeader() const
-{
-  return this->Header();
-}
-
 /////////////////////////////////////////////////
 std::string LogPlay::Header() const
 {
@@ -186,12 +190,6 @@ std::string LogPlay::Header() const
   return stream.str();
 }
 
-/////////////////////////////////////////////////
-uint64_t LogPlay::GetInitialIterations() const
-{
-  return this->InitialIterations();
-}
-
 /////////////////////////////////////////////////
 uint64_t LogPlay::InitialIterations() const
 {
@@ -208,6 +206,7 @@ bool LogPlay::HasIterations() const
 void LogPlay::ReadHeader()
 {
   this->dataPtr->randSeed = ignition::math::Rand::Seed();
+
   tinyxml2::XMLElement *headerXml, *childXml;
 
   this->dataPtr->logVersion.clear();
@@ -276,7 +275,7 @@ void LogPlay::ReadLogTimes()
       return;
     }
 
-    if (!this->ChunkData(chunkXml, chunk))
+    if (!this->dataPtr->ChunkData(chunkXml, chunk))
       return;
 
     // Find the first <sim_time> of the log.
@@ -308,7 +307,7 @@ void LogPlay::ReadLogTimes()
     return;
   }
 
-  if (!this->ChunkData(lastChunk, chunk))
+  if (!this->dataPtr->ChunkData(lastChunk, chunk))
     return;
 
   // Update the last <sim_time> of the log.
@@ -352,7 +351,7 @@ bool LogPlay::ReadIterations()
     }
 
     std::string chunk;
-    if (!this->ChunkData(chunkXml, chunk))
+    if (!this->dataPtr->ChunkData(chunkXml, chunk))
       return false;
 
     // Find the first <iterations> of the log.
@@ -382,83 +381,41 @@ bool LogPlay::IsOpen() const
   return this->dataPtr->logStartXml != NULL;
 }
 
-/////////////////////////////////////////////////
-std::string LogPlay::GetLogVersion() const
-{
-  return this->LogVersion();
-}
-
 /////////////////////////////////////////////////
 std::string LogPlay::LogVersion() const
 {
   return this->dataPtr->logVersion;
 }
 
-/////////////////////////////////////////////////
-std::string LogPlay::GetGazeboVersion() const
-{
-  return this->GazeboVersion();
-}
-
 /////////////////////////////////////////////////
 std::string LogPlay::GazeboVersion() const
 {
   return this->dataPtr->gazeboVersion;
 }
 
-/////////////////////////////////////////////////
-uint32_t LogPlay::GetRandSeed() const
-{
-  return this->RandSeed();
-}
-
 /////////////////////////////////////////////////
 uint32_t LogPlay::RandSeed() const
 {
   return this->dataPtr->randSeed;
 }
 
-/////////////////////////////////////////////////
-common::Time LogPlay::GetLogStartTime() const
-{
-  return this->LogStartTime();
-}
-
 /////////////////////////////////////////////////
 common::Time LogPlay::LogStartTime() const
 {
   return this->dataPtr->logStartTime;
 }
 
-/////////////////////////////////////////////////
-common::Time LogPlay::GetLogEndTime() const
-{
-  return this->LogEndTime();
-}
-
 /////////////////////////////////////////////////
 common::Time LogPlay::LogEndTime() const
 {
   return this->dataPtr->logEndTime;
 }
 
-/////////////////////////////////////////////////
-std::string LogPlay::GetFilename() const
-{
-  return this->Filename();
-}
-
 /////////////////////////////////////////////////
 std::string LogPlay::Filename() const
 {
-  return boost::filesystem::basename(this->dataPtr->filename) +
-    boost::filesystem::extension(this->dataPtr->filename);
-}
-
-/////////////////////////////////////////////////
-std::string LogPlay::GetFullPathFilename() const
-{
-  return this->FullPathFilename();
+  boost::filesystem::path path(this->dataPtr->filename);
+  return path.stem().string() + path.extension().string();
 }
 
 /////////////////////////////////////////////////
@@ -468,12 +425,6 @@ std::string LogPlay::FullPathFilename() const
   return boost::filesystem::canonical(logFilename).string();
 }
 
-/////////////////////////////////////////////////
-uintmax_t LogPlay::GetFileSize() const
-{
-  return this->FileSize();
-}
-
 /////////////////////////////////////////////////
 uintmax_t LogPlay::FileSize() const
 {
@@ -594,8 +545,11 @@ bool LogPlay::Rewind()
     return false;
   }
 
-  if (!this->ChunkData(this->dataPtr->logCurrXml, this->dataPtr->currentChunk))
+  if (!this->dataPtr->ChunkData(this->dataPtr->logCurrXml,
+                                this->dataPtr->currentChunk))
+  {
     return false;
+  }
 
   // Skip first <sdf> block (it doesn't have a world state).
   this->dataPtr->end = this->dataPtr->currentChunk.find(
@@ -631,8 +585,11 @@ bool LogPlay::Forward()
     return false;
   }
 
-  if (!this->ChunkData(this->dataPtr->logCurrXml, this->dataPtr->currentChunk))
+  if (!this->dataPtr->ChunkData(this->dataPtr->logCurrXml,
+                                this->dataPtr->currentChunk))
+  {
     return false;
+  }
 
   this->dataPtr->start = this->dataPtr->currentChunk.size() - 1;
   this->dataPtr->end = this->dataPtr->currentChunk.size() - 1;
@@ -730,12 +687,6 @@ bool LogPlay::Seek(const common::Time &_time)
   return true;
 }
 
-/////////////////////////////////////////////////
-bool LogPlay::GetChunk(unsigned int _index, std::string &_data)
-{
-  return this->Chunk(_index, _data);
-}
-
 /////////////////////////////////////////////////
 bool LogPlay::Chunk(unsigned int _index, std::string &_data) const
 {
@@ -751,13 +702,15 @@ bool LogPlay::Chunk(unsigned int _index, std::string &_data) const
   }
 
   if (this->dataPtr->logCurrXml && count == _index)
-    return this->ChunkData(this->dataPtr->logCurrXml, _data);
+    return this->dataPtr->ChunkData(this->dataPtr->logCurrXml, _data);
   else
     return false;
 }
 
 /////////////////////////////////////////////////
-bool LogPlay::ChunkData(tinyxml2::XMLElement *_xml, std::string &_data) const
+bool LogPlayPrivate::ChunkData(
+    tinyxml2::XMLElement *_xml,
+    std::string &_data)
 {
   // Make sure we have valid xml pointer
   if (!_xml)
@@ -767,18 +720,17 @@ bool LogPlay::ChunkData(tinyxml2::XMLElement *_xml, std::string &_data) const
   }
 
   /// Get the chunk's encoding
-  this->dataPtr->encoding = _xml->Attribute("encoding");
+  this->encoding = _xml->Attribute("encoding");
 
   // Make sure there is an encoding value.
-  if (this->dataPtr->encoding.empty())
+  if (this->encoding.empty())
   {
-    gzthrow("Encoding missing for a chunk in log file[" +
-        this->dataPtr->filename + "]");
+    gzthrow("Encoding missing for a chunk in log file[" + this->filename + "]");
   }
 
-  if (this->dataPtr->encoding == "txt")
+  if (this->encoding == "txt")
     _data = _xml->GetText();
-  else if (this->dataPtr->encoding == "bz2")
+  else if (this->encoding == "bz2")
   {
     std::string data = _xml->GetText();
     std::string buffer;
@@ -797,7 +749,7 @@ bool LogPlay::ChunkData(tinyxml2::XMLElement *_xml, std::string &_data) const
       _data += '\0';
     }
   }
-  else if (this->dataPtr->encoding == "zlib")
+  else if (this->encoding == "zlib")
   {
     std::string data = _xml->GetText();
     std::string buffer;
@@ -818,32 +770,20 @@ bool LogPlay::ChunkData(tinyxml2::XMLElement *_xml, std::string &_data) const
   }
   else
   {
-    gzerr << "Invalid encoding[" << this->dataPtr->encoding << "] in log file["
-      << this->dataPtr->filename << "]\n";
+    gzerr << "Invalid encoding[" << this->encoding << "] in log file["
+      << this->filename << "]\n";
     return false;
   }
 
   return true;
 }
 
-/////////////////////////////////////////////////
-std::string LogPlay::GetEncoding() const
-{
-  return this->Encoding();
-}
-
 /////////////////////////////////////////////////
 std::string LogPlay::Encoding() const
 {
   return this->dataPtr->encoding;
 }
 
-/////////////////////////////////////////////////
-unsigned int LogPlay::GetChunkCount() const
-{
-  return this->ChunkCount();
-}
-
 /////////////////////////////////////////////////
 unsigned int LogPlay::ChunkCount() const
 {
@@ -867,8 +807,11 @@ bool LogPlay::NextChunk()
     return false;
 
   this->dataPtr->logCurrXml = next;
-  if (!this->ChunkData(this->dataPtr->logCurrXml, this->dataPtr->currentChunk))
+  if (!this->dataPtr->ChunkData(this->dataPtr->logCurrXml,
+                                this->dataPtr->currentChunk))
+  {
     return false;
+  }
 
   this->dataPtr->start = 0;
   this->dataPtr->end = -1 * this->dataPtr->kEndFrame.size();
@@ -884,11 +827,27 @@ bool LogPlay::PrevChunk()
     return false;
 
   this->dataPtr->logCurrXml = prev;
-  if (!this->ChunkData(this->dataPtr->logCurrXml, this->dataPtr->currentChunk))
+  if (!this->dataPtr->ChunkData(this->dataPtr->logCurrXml,
+                                this->dataPtr->currentChunk))
+  {
     return false;
+  }
 
   this->dataPtr->start = this->dataPtr->currentChunk.size() - 1;
   this->dataPtr->end = this->dataPtr->currentChunk.size() - 1;
 
   return true;
 }
+
+//////////////////////////////////////////////////
+LogPlay* LogPlay::Instance()
+{
+#ifndef _WIN32
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+  return SingletonT<LogPlay>::Instance();
+#ifndef _WIN32
+  #pragma GCC diagnostic pop
+#endif
+}
