@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,7 +14,6 @@
  * limitations under the License.
  *
 */
-
 #include <gtest/gtest.h>
 #include <boost/filesystem.hpp>
 
@@ -34,26 +33,22 @@ TEST_F(LogRecord_TEST, Constructor)
 {
   gazebo::util::LogRecord *recorder = gazebo::util::LogRecord::Instance();
 
-#ifndef _WIN32
-  const char *homePath = common::getEnv("HOME");
-#else
-  const char *homePath = common::getEnv("HOMEPATH");
-#endif
+  const char *homePath = common::getEnv(HOMEDIR);
 
   EXPECT_TRUE(homePath != NULL);
 
   common::SystemPaths *paths = common::SystemPaths::Instance();
-  boost::filesystem::path logPath = paths->GetTmpPath() + "/gazebo";
+  boost::filesystem::path logPath = paths->TmpPath() + "/gazebo";
   if (homePath)
     logPath = boost::filesystem::path(homePath);
   logPath /= "/.gazebo/log/";
 
   // Make sure the log path is correct
-  EXPECT_EQ(recorder->GetBasePath(), logPath.string());
+  EXPECT_EQ(recorder->BasePath(), logPath.string());
 
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_FALSE(recorder->GetRunning());
-  EXPECT_TRUE(recorder->GetFirstUpdate());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_FALSE(recorder->Running());
+  EXPECT_TRUE(recorder->FirstUpdate());
 
   // Init without a subdirectory
   EXPECT_FALSE(recorder->Init(""));
@@ -79,7 +74,7 @@ TEST_F(LogRecord_TEST, StartErrors)
   // Double start
   {
     EXPECT_TRUE(recorder->Start("bz2"));
-    EXPECT_TRUE(recorder->GetRunning());
+    EXPECT_TRUE(recorder->Running());
     EXPECT_FALSE(recorder->Start("bz2"));
   }
 
@@ -87,9 +82,9 @@ TEST_F(LogRecord_TEST, StartErrors)
   recorder->Stop();
 
   // Make sure everything has reset.
-  EXPECT_FALSE(recorder->GetRunning());
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_EQ(recorder->GetRunTime(), gazebo::common::Time());
+  EXPECT_FALSE(recorder->Running());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_EQ(recorder->RunTime(), gazebo::common::Time());
 
   // Logger may still be writing so make sure we exit cleanly
   int i = 0;
@@ -111,27 +106,27 @@ TEST_F(LogRecord_TEST, Start_bzip2)
   EXPECT_TRUE(recorder->Start("bz2"));
 
   // Make sure the right flags have been set
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_TRUE(recorder->GetRunning());
-  EXPECT_TRUE(recorder->GetFirstUpdate());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_TRUE(recorder->Running());
+  EXPECT_TRUE(recorder->FirstUpdate());
 
   // Make sure the right encoding is set
-  EXPECT_EQ(recorder->GetEncoding(), std::string("bz2"));
+  EXPECT_EQ(recorder->Encoding(), std::string("bz2"));
 
   // Make sure the log directories exist
-  EXPECT_TRUE(boost::filesystem::exists(recorder->GetBasePath()));
-  EXPECT_TRUE(boost::filesystem::is_directory(recorder->GetBasePath()));
+  EXPECT_TRUE(boost::filesystem::exists(recorder->BasePath()));
+  EXPECT_TRUE(boost::filesystem::is_directory(recorder->BasePath()));
 
   // Run time should be zero since no update has been triggered.
-  EXPECT_EQ(recorder->GetRunTime(), gazebo::common::Time());
+  EXPECT_EQ(recorder->RunTime(), gazebo::common::Time());
 
   // Stop recording.
   recorder->Stop();
 
   // Make sure everything has reset.
-  EXPECT_FALSE(recorder->GetRunning());
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_EQ(recorder->GetRunTime(), gazebo::common::Time());
+  EXPECT_FALSE(recorder->Running());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_EQ(recorder->RunTime(), gazebo::common::Time());
 
   // Logger may still be writing so make sure we exit cleanly
   int i = 0;
@@ -153,27 +148,27 @@ TEST_F(LogRecord_TEST, Start_zlib)
   EXPECT_TRUE(recorder->Start("zlib"));
 
   // Make sure the right flags have been set
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_TRUE(recorder->GetRunning());
-  EXPECT_TRUE(recorder->GetFirstUpdate());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_TRUE(recorder->Running());
+  EXPECT_TRUE(recorder->FirstUpdate());
 
   // Make sure the right encoding is set
-  EXPECT_EQ(recorder->GetEncoding(), std::string("zlib"));
+  EXPECT_EQ(recorder->Encoding(), std::string("zlib"));
 
   // Make sure the log directories exist
-  EXPECT_TRUE(boost::filesystem::exists(recorder->GetBasePath()));
-  EXPECT_TRUE(boost::filesystem::is_directory(recorder->GetBasePath()));
+  EXPECT_TRUE(boost::filesystem::exists(recorder->BasePath()));
+  EXPECT_TRUE(boost::filesystem::is_directory(recorder->BasePath()));
 
   // Run time should be zero since no update has been triggered.
-  EXPECT_EQ(recorder->GetRunTime(), gazebo::common::Time());
+  EXPECT_EQ(recorder->RunTime(), gazebo::common::Time());
 
   // Stop recording.
   recorder->Stop();
 
   // Make sure everything has reset.
-  EXPECT_FALSE(recorder->GetRunning());
-  EXPECT_FALSE(recorder->GetPaused());
-  EXPECT_EQ(recorder->GetRunTime(), gazebo::common::Time());
+  EXPECT_FALSE(recorder->Running());
+  EXPECT_FALSE(recorder->Paused());
+  EXPECT_EQ(recorder->RunTime(), gazebo::common::Time());
 
   // Logger may still be writing so make sure we exit cleanly
   int i = 0;
@@ -185,6 +180,48 @@ TEST_F(LogRecord_TEST, Start_zlib)
   }
 }
 
+/////////////////////////////////////////////////
+/// \brief Test LogRecord filter
+TEST_F(LogRecord_TEST, Filter)
+{
+  gazebo::util::LogRecord *recorder = gazebo::util::LogRecord::Instance();
+
+  // check default values
+  EXPECT_DOUBLE_EQ(recorder->Period(), -1);
+  EXPECT_EQ(recorder->Filter(), std::string());
+
+  // filter by period
+  recorder->SetPeriod(0.02);
+  EXPECT_DOUBLE_EQ(recorder->Period(), 0.02);
+
+  recorder->SetPeriod(0);
+  EXPECT_DOUBLE_EQ(recorder->Period(), 0);
+
+
+  // filter by regex string
+  recorder->SetFilter("robot*");
+  EXPECT_EQ(recorder->Filter(), "robot*");
+
+  recorder->SetFilter("");
+  EXPECT_EQ(recorder->Filter(), "");
+}
+
+/////////////////////////////////////////////////
+/// \brief Test LogRecord record resources
+TEST_F(LogRecord_TEST, RecordResources)
+{
+  gazebo::util::LogRecord *recorder = gazebo::util::LogRecord::Instance();
+
+  // check default values
+  EXPECT_FALSE(recorder->RecordResources());
+
+  recorder->SetRecordResources(true);
+  EXPECT_TRUE(recorder->RecordResources());
+
+  recorder->SetRecordResources(false);
+  EXPECT_FALSE(recorder->RecordResources());
+}
+
 /////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
