@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,7 +14,8 @@
  * limitations under the License.
  *
 */
-
+#include <functional>
+#include <ignition/math/Rand.hh>
 #include "gazebo/sensors/Noise.hh"
 #include "RaySensorNoisePlugin.hh"
 
@@ -45,12 +46,13 @@ void RaySensorNoisePlugin::Load(sensors::SensorPtr _parent,
     return;
   }
 
-  sensors::NoisePtr noise = _parent->GetNoise(sensors::RAY_NOISE);
+  sensors::NoisePtr noise = _parent->Noise(sensors::RAY_NOISE);
 
   if (noise)
   {
     noise->SetCustomNoiseCallback(
-      boost::bind(&RaySensorNoisePlugin::OnApplyNoise, this, _1));
+      std::bind(&RaySensorNoisePlugin::OnApplyNoise, this,
+        std::placeholders::_1));
   }
   else
   {
@@ -63,7 +65,7 @@ void RaySensorNoisePlugin::Load(sensors::SensorPtr _parent,
 double RaySensorNoisePlugin::OnApplyNoise(double _in)
 {
   // Apply alternating random noise.
-  double randNoise = math::Rand::GetDblUniform(0, this->fixedNoiseRate);
+  double randNoise = ignition::math::Rand::DblUniform(0, this->fixedNoiseRate);
   this->sign *= -1;
   return _in + this->sign*randNoise*_in;
 }
