@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Open Source Robotics Foundation
+ * Copyright (C) 2014 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include <boost/algorithm/string/trim.hpp>
 #include <boost/thread/mutex.hpp>
 #include <boost/filesystem.hpp>
+#include <ignition/transport/Node.hh>
 
 #include <gazebo/common/CommonIface.hh>
 #include <gazebo/msgs/msgs.hh>
@@ -40,10 +41,10 @@ pid_t g_pid = -1;
 boost::condition_variable g_msgCondition;
 
 /////////////////////////////////////////////////
-void WorldControlCB(ConstWorldControlPtr &_msg)
+void WorldControlCB(const msgs::WorldControl &_msg)
 {
   boost::mutex::scoped_lock lock(g_mutex);
-  g_msgDebugOut = _msg->DebugString();
+  g_msgDebugOut = _msg.DebugString();
   g_msgCondition.notify_all();
 }
 
@@ -57,7 +58,11 @@ bool custom_exec(std::string _cmd)
 std::string custom_exec_str(std::string _cmd)
 {
   _cmd += " 2>&1";
+#ifdef _WIN32
+  FILE *pipe = _popen(_cmd.c_str(), "r");
+#else
   FILE *pipe = popen(_cmd.c_str(), "r");
+#endif
 
   if (!pipe)
     return "ERROR";
@@ -71,7 +76,11 @@ std::string custom_exec_str(std::string _cmd)
       result += buffer;
   }
 
+#ifdef _WIN32
+  _pclose(pipe);
+#else
   pclose(pipe);
+#endif
   return result;
 }
 
@@ -141,10 +150,8 @@ TEST_F(gzTest, Stress)
 {
   init();
 
-  gazebo::transport::NodePtr node(new gazebo::transport::Node());
-  node->Init();
-  gazebo::transport::SubscriberPtr sub =
-    node->Subscribe("~/world_control", &WorldControlCB, true);
+  ignition::transport::Node ignNode;
+  ignNode.Subscribe("/world_control", &WorldControlCB);
 
   // Run the transport loop: starts a new thread
   gazebo::transport::run();
