@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Open Source Robotics Foundation
+ * Copyright (C) 2014 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,24 +14,19 @@
  * limitations under the License.
  *
 */
-
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <stdio.h>
 #include <signal.h>
 #include <tinyxml.h>
 #include <boost/filesystem.hpp>
 #include <boost/algorithm/string.hpp>
+#include <streambuf>
 
 #include <gazebo/common/common.hh>
 #include <gazebo/transport/transport.hh>
 #include <sdf/sdf.hh>
-#include "gz_topic.hh"
 #include "gz_log.hh"
+#include "gz_marker.hh"
+#include "gz_topic.hh"
 #include "gz.hh"
 
 using namespace gazebo;
@@ -46,6 +41,7 @@ Command::Command(const std::string &_name, const std::string &_brief)
   : name(_name), brief(_brief), visibleOptions("Options"), argc(0), argv(NULL)
 {
   this->visibleOptions.add_options()
+    ("verbose", "Print extra information")
     ("help,h", "Print this help message");
 }
 
@@ -74,7 +70,7 @@ void Command::ListOptions()
   {
     pieces.clear();
     std::string formatName = (*iter)->format_name();
-    boost::split(pieces, formatName, boost::is_any_of(" "));
+    pieces = common::split(formatName, " ");
 
     if (pieces.empty())
     {
@@ -154,6 +150,7 @@ bool Command::Run(int _argc, char **_argv)
   }
 
   // The SDF find file callback.
+  using namespace boost::placeholders;
   sdf::setFindCallback(boost::bind(&gazebo::common::find_file, _1));
 
   // Hidden options
@@ -188,6 +185,11 @@ bool Command::Run(int _argc, char **_argv)
     return true;
   }
 
+  if (this->vm.count("verbose"))
+  {
+    gazebo::common::Console::SetQuiet(false);
+  }
+
   if (!this->TransportInit())
   {
     std::cerr << "An instance of Gazebo is not running.\n";
@@ -236,12 +238,25 @@ bool WorldCommand::RunImpl()
   if (this->vm.count("world-name"))
     worldName = this->vm["world-name"].as<std::string>();
 
-  transport::NodePtr node(new transport::Node());
-  node->Init(worldName);
+  ignition::transport::Node ignNode;
 
-  transport::PublisherPtr pub =
-    node->Advertise<msgs::WorldControl>("~/world_control");
-  pub->WaitForConnection();
+  const std::string topic{"/world_control"};
+  auto pub = ignNode.Advertise<msgs::WorldControl>(topic);
+
+  // Wait for subscribers
+  unsigned int maxSleep = 30;
+  unsigned int sleep = 0;
+  unsigned int mSleep = 100;
+  for (; sleep < maxSleep && !pub.HasConnections(); ++sleep)
+  {
+    common::Time::MSleep(mSleep);
+  }
+  if (sleep == maxSleep)
+  {
+    gzerr << "No subscribers to topic [" << topic <<"], timed out after " <<
+        maxSleep * mSleep << "ms." << std::endl;
+    return false;
+  }
 
   msgs::WorldControl msg;
   bool good = false;
@@ -283,9 +298,13 @@ bool WorldCommand::RunImpl()
   }
 
   if (good)
-    pub->Publish(msg, true);
+  {
+    pub.Publish(msg);
+  }
   else
+  {
     this->Help();
+  }
 
   return true;
 }
@@ -355,9 +374,7 @@ bool PhysicsCommand::RunImpl()
 
   if (this->vm.count("gravity"))
   {
-    std::vector<std::string> values;
-    boost::split(values, this->vm["gravity"].as<std::string>(),
-        boost::is_any_of(","));
+    auto values = common::split(this->vm["gravity"].as<std::string>(), ",");
 
     msg.mutable_gravity()->set_x(boost::lexical_cast<double>(values[0]));
     msg.mutable_gravity()->set_y(boost::lexical_cast<double>(values[1]));
@@ -431,22 +448,22 @@ bool ModelCommand::RunImpl()
     return false;
   }
 
-  math::Pose pose;
-  math::Vector3 rpy;
+  ignition::math::Pose3d pose;
+  ignition::math::Vector3d rpy;
 
   if (this->vm.count("pose-x"))
-    pose.pos.x = this->vm["pose-x"].as<double>();
+    pose.Pos().X(this->vm["pose-x"].as<double>());
   if (this->vm.count("pose-y"))
-    pose.pos.y = this->vm["pose-y"].as<double>();
+    pose.Pos().Y(this->vm["pose-y"].as<double>());
   if (this->vm.count("pose-z"))
-    pose.pos.z = this->vm["pose-z"].as<double>();
+    pose.Pos().Z(this->vm["pose-z"].as<double>());
   if (this->vm.count("pose-R"))
-    rpy.x = this->vm["pose-R"].as<double>();
+    rpy.X(this->vm["pose-R"].as<double>());
   if (this->vm.count("pose-P"))
-    rpy.y = this->vm["pose-P"].as<double>();
+    rpy.Y(this->vm["pose-P"].as<double>());
   if (this->vm.count("pose-Y"))
-    rpy.z = this->vm["pose-Y"].as<double>();
-  pose.rot.SetFromEuler(rpy);
+    rpy.Z(this->vm["pose-Y"].as<double>());
+  pose.Rot().Euler(rpy);
 
   transport::NodePtr node(new transport::Node());
   node->Init(worldName);
@@ -477,13 +494,32 @@ bool ModelCommand::RunImpl()
       return false;
     }
 
-    if (!sdf::readFile(filename, sdf))
+    sdf::Errors errors;
+    if (!sdf::readFileWithoutConversion(filename, sdf, errors))
     {
       std::cerr << "Error: SDF parsing the xml failed\n";
       return false;
     }
 
-    return this->ProcessSpawn(sdf, modelName, pose, node);
+    std::string resolvedFileName = filename;
+    if (common::isDirectory(filename))
+    {
+      // Use sdf::getModelFilePath() instead of filename because filename
+      // might be a model rectory
+      resolvedFileName = sdf::getModelFilePath(filename);
+      if (resolvedFileName.empty())
+      {
+        std::cerr << "The provided file is a directory, but a model could not "
+                  << "be found" << std::endl;
+      }
+    }
+    std::ifstream sdfFile(resolvedFileName);
+
+    // Using const std::string sdfString(arg, arg) confuses the compiler, so use
+    // move assignment
+    const auto sdfString = std::string(std::istreambuf_iterator<char>(sdfFile),
+                                       std::istreambuf_iterator<char>());
+    return this->ProcessSpawn(sdfString, modelName, pose, node);
   }
   else if (this->vm.count("spawn-string"))
   {
@@ -503,13 +539,14 @@ bool ModelCommand::RunImpl()
       return false;
     }
 
-    if (!sdf::readString(sdfString, sdf))
+    sdf::Errors errors;
+    if (!sdf::readStringWithoutConversion(sdfString, sdf, errors))
     {
       std::cerr << "Error: SDF parsing the xml failed\n";
       return false;
     }
 
-    return this->ProcessSpawn(sdf, modelName, pose, node);
+    return this->ProcessSpawn(sdfString, modelName, pose, node);
   }
   else if (this->vm.count("info") || this->vm.count("pose"))
   {
@@ -541,7 +578,7 @@ bool ModelCommand::RunImpl()
 
     msgs::Model msg;
     msg.set_name(modelName);
-    msgs::Set(msg.mutable_pose(), pose.Ign());
+    msgs::Set(msg.mutable_pose(), pose);
     pub->Publish(msg, true);
   }
 
@@ -549,27 +586,56 @@ bool ModelCommand::RunImpl()
 }
 
 /////////////////////////////////////////////////
-bool ModelCommand::ProcessSpawn(sdf::SDFPtr _sdf,
-    const std::string &_name, const math::Pose &_pose, transport::NodePtr _node)
+bool ModelCommand::ProcessSpawn(const std::string &_sdfString,
+    const std::string &_name, const ignition::math::Pose3d &_pose,
+    transport::NodePtr _node)
 {
-  sdf::ElementPtr modelElem = _sdf->Root()->GetElement("model");
-
-  if (!modelElem)
-  {
-    gzerr << "Unable to find <model> element.\n";
-    return false;
-  }
-
-  // Set the model name
-  if (!_name.empty())
-    modelElem->GetAttribute("name")->SetFromString(_name);
-
   transport::PublisherPtr pub = _node->Advertise<msgs::Factory>("~/factory");
   pub->WaitForConnection();
 
+  // To set the name of the model, we have to parse the file, set the name and
+  // reserialize to SDFormat. However, loading it using libsdformat will result
+  // in upconversion, which we would like to avoid because the resulting
+  // document will lose its OriginalVersion attribute. So, instead, we parse and
+  // edit the model name with TinyXML.
+
+  TiXmlDocument doc;
+  doc.Parse(_sdfString.c_str());
+  auto *root = doc.RootElement();
+  if (nullptr == root)
+  {
+    std::cerr << "Invalid XML file" << std::endl;
+    std::cout << _sdfString << std::endl;
+    return false;
+  }
+
+  // Handle SDFormat and URDF separately
+  if (root->ValueStr() == "sdf")
+  {
+    auto *model = root->FirstChildElement("model");
+    if (nullptr != model)
+    {
+      model->SetAttribute("name", _name);
+    }
+    else
+    {
+      std::cerr << "Could not find <model> tag in SDFormat file" << std::endl;
+    }
+  }
+  else if (root->ValueStr() == "robot")
+  {
+    root->SetAttribute("name", _name);
+  }
+  else
+  {
+    std::cerr << "Unknown file format" << std::endl;
+  }
+
+  TiXmlPrinter printer;
+  doc.Accept(&printer);
   msgs::Factory msg;
-  msg.set_sdf(_sdf->ToString());
-  msgs::Set(msg.mutable_pose(), _pose.Ign());
+  msg.set_sdf(printer.Str());
+  msgs::Set(msg.mutable_pose(), _pose);
   pub->Publish(msg, true);
 
   return true;
@@ -585,13 +651,14 @@ JointCommand::JointCommand()
     ("world-name,w", po::value<std::string>(), "World name.")
     ("model-name,m", po::value<std::string>(), "Model name.")
     ("joint-name,j", po::value<std::string>(), "Joint name.")
-    ("delete,d", "Delete a model.")
-    ("force,f", po::value<double>(), "Force to apply to a joint.")
-    ("pos-t", po::value<double>(), "Target angle.")
+    ("force,f", po::value<double>(), "Force to apply to a joint (N).")
+    ("pos-t", po::value<double>(),
+     "Target angle(rad) for rotation joints or position (m) for linear joints.")
     ("pos-p", po::value<double>(), "Position proportional gain.")
     ("pos-i", po::value<double>(), "Position integral gain.")
     ("pos-d", po::value<double>(), "Position differential gain.")
-    ("vel-t", po::value<double>(), "Target speed.")
+    ("vel-t", po::value<double>(),
+     "Target speed (rad/s for rotational joints or m/s for linear joints).")
     ("vel-p", po::value<double>(), "Velocity proportional gain.")
     ("vel-i", po::value<double>(), "Velocity integral gain.")
     ("vel-d", po::value<double>(), "Velocity differential gain.");
@@ -605,16 +672,24 @@ void JointCommand::HelpDetailed()
     "\toption -w, is not specified, the first world found on \n"
     "\tthe Gazebo master will be used.\n"
     "\tA model name and joint name are required.\n"
+    "\n"
+    "\tIt is recommended to use only one type of command:\n"
+    "\tforce, position PID, or velocity PID.\n"
+    "\n"
+    "\tForce: Use --force to apply a force.\n"
+    "\n"
+    "\tPosition PID: Use --pos-t to specify a target position\n"
+    "\twith --pos-p, --pos-i, --pos-d to specify the PID parameters.\n"
+    "\n"
+    "\tVelocity PID: Use --vel-t to specify a target velocity\n"
+    "\twith --vel-p, --vel-i, --vel-d to specify the PID parameters.\n"
     << std::endl;
 }
 
 /////////////////////////////////////////////////
 bool JointCommand::RunImpl()
 {
-  std::string modelName, worldName, jointName;
-
-  if (this->vm.count("world-name"))
-    worldName = this->vm["world-name"].as<std::string>();
+  std::string modelName, jointName;
 
   if (this->vm.count("model-name"))
     modelName = this->vm["model-name"].as<std::string>();
@@ -636,50 +711,81 @@ bool JointCommand::RunImpl()
     return false;
   }
 
-  msgs::JointCmd msg;
+  ignition::msgs::JointCmd msg;
   msg.set_name(modelName + "::" + jointName);
 
   if (this->vm.count("force"))
-    msg.set_force(this->vm["force"].as<double>());
+    msg.mutable_force_optional()->set_data(this->vm["force"].as<double>());
 
   if (this->vm.count("pos-t"))
   {
-    msg.mutable_position()->set_target(this->vm["pos-t"].as<double>());
+    msg.mutable_position()->mutable_target_optional()->set_data(
+        this->vm["pos-t"].as<double>());
 
     if (this->vm.count("pos-p"))
-      msg.mutable_position()->set_p_gain(this->vm["pos-p"].as<double>());
+    {
+      msg.mutable_position()->mutable_p_gain_optional()->set_data(
+          this->vm["pos-p"].as<double>());
+    }
 
     if (this->vm.count("pos-i"))
-      msg.mutable_position()->set_i_gain(this->vm["pos-i"].as<double>());
+    {
+      msg.mutable_position()->mutable_i_gain_optional()->set_data(
+          this->vm["pos-i"].as<double>());
+    }
 
     if (this->vm.count("pos-d"))
-      msg.mutable_position()->set_d_gain(this->vm["pos-d"].as<double>());
+    {
+      msg.mutable_position()->mutable_d_gain_optional()->set_data(
+          this->vm["pos-d"].as<double>());
+    }
   }
 
   if (this->vm.count("vel-t"))
   {
-    msg.mutable_velocity()->set_target(this->vm["vel-t"].as<double>());
+    msg.mutable_velocity()->mutable_target_optional()->set_data(
+        this->vm["vel-t"].as<double>());
 
     if (this->vm.count("vel-p"))
-      msg.mutable_velocity()->set_p_gain(this->vm["vel-p"].as<double>());
+    {
+      msg.mutable_velocity()->mutable_p_gain_optional()->set_data(
+          this->vm["vel-p"].as<double>());
+    }
 
     if (this->vm.count("vel-i"))
-      msg.mutable_velocity()->set_i_gain(this->vm["vel-i"].as<double>());
+    {
+      msg.mutable_velocity()->mutable_i_gain_optional()->set_data(
+          this->vm["vel-i"].as<double>());
+    }
 
     if (this->vm.count("vel-d"))
-      msg.mutable_velocity()->set_d_gain(this->vm["vel-d"].as<double>());
+    {
+      msg.mutable_velocity()->mutable_d_gain_optional()->set_data(
+          this->vm["vel-d"].as<double>());
+    }
   }
 
-  transport::NodePtr node(new transport::Node());
-  node->Init(worldName);
+  std::string topic = std::string("/") + modelName + "/joint_cmd";
 
-  transport::PublisherPtr pub =
-    node->Advertise<msgs::JointCmd>(
-        std::string("~/") + modelName + "/joint_cmd");
+  ignition::transport::Node ignNode;
+  auto pub = ignNode.Advertise<ignition::msgs::JointCmd>(topic);
 
-  pub->WaitForConnection();
+  unsigned int maxSleep = 30;
+  unsigned int sleep = 0;
+  unsigned int mSleep = 100;
+  for (; sleep < maxSleep && !pub.HasConnections(); ++sleep)
+  {
+    common::Time::MSleep(mSleep);
+  }
 
-  pub->Publish(msg, true);
+  if (sleep == maxSleep)
+  {
+    gzerr << "No subscribers to topic [" << topic <<"], timed out after " <<
+        maxSleep * mSleep << "ms." << std::endl;
+    return false;
+  }
+
+  pub.Publish(msg);
 
   return true;
 }
@@ -759,8 +865,7 @@ bool CameraCommand::RunImpl()
 
         if (topicInfo.msg_type() == "gazebo.msgs.CameraCmd")
         {
-          std::vector<std::string> parts;
-          boost::split(parts, topics.data(i), boost::is_any_of("/"));
+          auto parts = common::split(topics.data(i), "/");
           std::cout << parts[parts.size()-2] << std::endl;
         }
       }
@@ -811,7 +916,7 @@ StatsCommand::StatsCommand()
   // Options that are visible to the user through help.
   this->visibleOptions.add_options()
     ("world-name,w", po::value<std::string>(), "World name.")
-    ("duration,d", po::value<double>(), "Duration (seconds) to run.")
+    ("duration,d", po::value<uint64_t>(), "Duration (seconds) to run.")
     ("plot,p", "Output comma-separated values, useful for processing and "
      "plotting.");
 }
@@ -843,7 +948,7 @@ bool StatsCommand::RunImpl()
   boost::mutex::scoped_lock lock(this->sigMutex);
   if (this->vm.count("duration"))
     this->sigCondition.timed_wait(lock,
-        boost::posix_time::seconds(this->vm["duration"].as<double>()));
+        boost::posix_time::seconds(this->vm["duration"].as<uint64_t>()));
   else
     this->sigCondition.wait(lock);
 
@@ -1216,6 +1321,7 @@ int main(int argc, char **argv)
   g_commandMap["camera"] = new CameraCommand();
   g_commandMap["help"] = new HelpCommand();
   g_commandMap["joint"] = new JointCommand();
+  g_commandMap["marker"] = new MarkerCommand();
   g_commandMap["model"] = new ModelCommand();
   g_commandMap["world"] = new WorldCommand();
   g_commandMap["physics"] = new PhysicsCommand();
